@@ -1,9 +1,15 @@
 import google.cloud.bigquery  # noqa: F401 fail early if bigquery is missing
 import ibis.common as com
+
 from ibis.config import options  # noqa: F401
 from ibis.bigquery.client import BigQueryClient
 from ibis.bigquery.compiler import dialect
 
+try:
+    from ibis.bigquery.udf import udf  # noqa: F401
+except ImportError:
+    pass
+
 
 def compile(expr, params=None):
     """
@@ -30,17 +36,18 @@ def verify(expr, params=None):
         return False
 
 
-def connect(project_id, dataset_id):
+def connect(project_id, dataset_id, credentials=None):
     """Create a BigQueryClient for use with Ibis
 
     Parameters
     ----------
     project_id: str
     dataset_id: str
+    credentials : google.auth.credentials.Credentials, optional, default None
 
     Returns
     -------
     BigQueryClient
     """
 
-    return BigQueryClient(project_id, dataset_id)
+    return BigQueryClient(project_id, dataset_id, credentials=credentials)
