@@ -1,3 +1,15 @@
+import datetime
+
+from functools import partial
+
+import regex as re
+
+import six
+
+import toolz
+
+from multipledispatch import Dispatcher
+
 import ibis
 import ibis.common as com
 
@@ -8,9 +20,19 @@ import ibis.expr.types as ir
 
 import ibis.sql.compiler as comp
 import ibis.expr.operations as ops
-from ibis.impala.compiler import ImpalaSelect, unary, fixed_arity
+import ibis.expr.lineage as lin
+
+from ibis.impala.compiler import (
+    ImpalaSelect, unary, fixed_arity, ImpalaTableSetFormatter, _reduction
+)
 from ibis.impala import compiler as impala_compiler
 
+from ibis.bigquery.datatypes import ibis_type_to_bigquery_type
+
+
+class BigQueryUDFNode(ops.ValueOp):
+    pass
+
 
 class BigQuerySelectBuilder(comp.SelectBuilder):
 
@@ -19,14 +41,45 @@ class BigQuerySelectBuilder(comp.SelectBuilder):
         return BigQuerySelect
 
 
+class BigQueryUDFDefinition(comp.DDL):
+
+    def __init__(self, expr, context):
+        self.expr = expr
+        self.context = context
+
+    def compile(self):
+        return self.expr.op().js
+
+
+class BigQueryUnion(comp.Union):
+    @staticmethod
+    def keyword(distinct):
+        return 'UNION DISTINCT' if distinct else 'UNION ALL'
+
+
+def find_bigquery_udf(expr):
+    if isinstance(expr.op(), BigQueryUDFNode):
+        result = expr
+    else:
+        result = None
+    return lin.proceed, result
+
+
 class BigQueryQueryBuilder(comp.QueryBuilder):
 
     select_builder = BigQuerySelectBuilder
+    union_class = BigQueryUnion
 
-    @property
-    def _union_class(self):
-        # return BigQueryUnion
-        raise NotImplementedError()
+    def generate_setup_queries(self):
+        queries = map(
+            partial(BigQueryUDFDefinition, context=self.context),
+            lin.traverse(find_bigquery_udf, self.expr)
+        )
+
+        # UDFs are uniquely identified by the name of the Node subclass we
+        # generate.
+        return list(
+            toolz.unique(queries, key=lambda x: type(x.expr.op()).__name__))
 
 
 def build_ast(expr, context):
@@ -34,16 +87,9 @@ def build_ast(expr, context):
     return builder.get_result()
 
 
-def _get_query(expr, context):
-    ast = build_ast(expr, context)
-    (query, rest) = (ast.queries[0], ast.queries[1:])
-    assert not rest
-    return query
-
-
 def to_sql(expr, context):
-    query = _get_query(expr, context)
-    compiled = query.compile()
+    query_ast = build_ast(expr, context)
+    compiled = query_ast.compile()
     return compiled
 
 
@@ -61,26 +107,25 @@ def _extract_field(sql_attr):
     return extract_field_formatter
 
 
-_sql_type_names = {
-    'int8': 'int64',
-    'int16': 'int64',
-    'int32': 'int64',
-    'int64': 'int64',
-    'float': 'float64',
-    'double': 'float64',
-    'string': 'string',
-    'boolean': 'boolean',
-    'timestamp': 'timestamp',
-    'date': 'date',
-}
+bigquery_cast = Dispatcher('bigquery_cast')
+
+
+@bigquery_cast.register(six.string_types, dt.Timestamp, dt.Integer)
+def bigquery_cast_timestamp_to_integer(compiled_arg, from_, to):
+    return 'UNIX_MICROS({})'.format(compiled_arg)
+
+
+@bigquery_cast.register(six.string_types, dt.DataType, dt.DataType)
+def bigquery_cast_generate(compiled_arg, from_, to):
+    sql_type = ibis_type_to_bigquery_type(to)
+    return 'CAST({} AS {})'.format(compiled_arg, sql_type)
 
 
 def _cast(translator, expr):
     op = expr.op()
     arg, target_type = op.args
     arg_formatted = translator.translate(arg)
-    sql_type = _sql_type_names[target_type.name.lower()]
-    return 'CAST({} AS {})'.format(arg_formatted, sql_type.upper())
+    return bigquery_cast(arg_formatted, arg.type(), target_type)
 
 
 def _struct_field(translator, expr):
@@ -118,7 +163,7 @@ def _string_find(translator, expr):
 
 def _translate_pattern(translator, pattern):
     # add 'r' to string literals to indicate to BigQuery this is a raw string
-    return 'r' * isinstance(pattern.op(), ir.Literal) + translator.translate(
+    return 'r' * isinstance(pattern.op(), ops.Literal) + translator.translate(
         pattern
     )
 
@@ -154,7 +199,7 @@ def _regex_replace(translator, expr):
 
 def _string_concat(translator, expr):
     return 'CONCAT({})'.format(
-        ', '.join(map(translator.translate, expr.op().args))
+        ', '.join(map(translator.translate, expr.op().arg))
     )
 
 
@@ -204,6 +249,21 @@ def _literal(translator, expr):
         if not np.isfinite(value):
             return 'CAST({!r} AS FLOAT64)'.format(str(value))
 
+    # special case literal timestamp, date, and time scalars
+    if isinstance(expr.op(), ops.Literal):
+        value = expr.op().value
+        if isinstance(expr, ir.DateScalar):
+            if isinstance(value, datetime.datetime):
+                raw_value = value.date()
+            else:
+                raw_value = value
+            return "DATE '{}'".format(raw_value)
+        elif isinstance(expr, ir.TimestampScalar):
+            return "TIMESTAMP '{}'".format(value)
+        elif isinstance(expr, ir.TimeScalar):
+            # TODO: define extractors on TimeValue expressions
+            return "TIME '{}'".format(value)
+
     try:
         return impala_compiler._literal(translator, expr)
     except NotImplementedError:
@@ -231,6 +291,7 @@ _date_units = {
     'Q': 'QUARTER',
     'W': 'WEEK',
     'M': 'MONTH',
+    'D': 'DAY',
 }
 
 
@@ -241,23 +302,21 @@ _timestamp_units = {
     'm': 'MINUTE',
     'h': 'HOUR',
 }
+_time_units = _timestamp_units.copy()
 _timestamp_units.update(_date_units)
 
 
 def _truncate(kind, units):
     def truncator(translator, expr):
-        op = expr.op()
-        arg, unit = op.args
-
-        arg = translator.translate(op.args[0])
-        try:
-            unit = units[unit]
-        except KeyError:
+        arg, unit = expr.op().args
+        trans_arg = translator.translate(arg)
+        valid_unit = units.get(unit)
+        if valid_unit is None:
             raise com.UnsupportedOperationError(
-                '{!r} unit is not supported in timestamp truncate'.format(unit)
+                'BigQuery does not support truncating {} values to unit '
+                '{!r}'.format(arg.type(), unit)
             )
-
-        return "{}_TRUNC({}, {})".format(kind, arg, unit)
+        return '{}_TRUNC({}, {})'.format(kind, trans_arg, valid_unit)
     return truncator
 
 
@@ -266,10 +325,11 @@ def _timestamp_op(func, units):
         op = expr.op()
         arg, offset = op.args
 
-        if offset.unit not in units:
+        unit = offset.type().unit
+        if unit not in units:
             raise com.UnsupportedOperationError(
                 'BigQuery does not allow binary operation '
-                '{} with INTERVAL offset {}'.format(func, offset.unit)
+                '{} with INTERVAL offset {}'.format(func, unit)
             )
         formatted_arg = translator.translate(arg)
         formatted_offset = translator.translate(offset)
@@ -279,6 +339,13 @@ def _timestamp_op(func, units):
     return _formatter
 
 
+STRFTIME_FORMAT_FUNCTIONS = {
+    dt.Date: 'DATE',
+    dt.Time: 'TIME',
+    dt.Timestamp: 'TIMESTAMP',
+}
+
+
 _operation_registry = impala_compiler._operation_registry.copy()
 _operation_registry.update({
     ops.ExtractYear: _extract_field('year'),
@@ -313,7 +380,9 @@ _operation_registry.update({
     ops.ArrayIndex: _array_index,
     ops.ArrayLength: unary('ARRAY_LENGTH'),
 
+    ops.HLLCardinality: _reduction('APPROX_COUNT_DISTINCT'),
     ops.Log: _log,
+    ops.Sign: unary('SIGN'),
     ops.Modulus: fixed_arity('MOD', 2),
 
     ops.Date: unary('DATE'),
@@ -321,11 +390,14 @@ _operation_registry.update({
     # BigQuery doesn't have these operations built in.
     # ops.ArrayRepeat: _array_repeat,
     # ops.ArraySlice: _array_slice,
-    ir.Literal: _literal,
+    ops.Literal: _literal,
     ops.Arbitrary: _arbitrary,
 
     ops.TimestampTruncate: _truncate('TIMESTAMP', _timestamp_units),
     ops.DateTruncate: _truncate('DATE', _date_units),
+    ops.TimeTruncate: _truncate('TIME', _timestamp_units),
+
+    ops.Time: unary('TIME'),
 
     ops.TimestampAdd: _timestamp_op(
         'TIMESTAMP_ADD', {'h', 'm', 's', 'ms', 'us'}),
@@ -334,6 +406,7 @@ _operation_registry.update({
 
     ops.DateAdd: _timestamp_op('DATE_ADD', {'D', 'W', 'M', 'Q', 'Y'}),
     ops.DateSub: _timestamp_op('DATE_SUB', {'D', 'W', 'M', 'Q', 'Y'}),
+    ops.TimestampNow: fixed_arity('CURRENT_TIMESTAMP', 0),
 })
 
 _invalid_operations = {
@@ -363,9 +436,64 @@ class BigQueryExprTranslator(impala_compiler.ImpalaExprTranslator):
         return '@{}'.format(expr.get_name())
 
 
+compiles = BigQueryExprTranslator.compiles
 rewrites = BigQueryExprTranslator.rewrites
 
 
+@compiles(ops.DayOfWeekIndex)
+def bigquery_day_of_week_index(t, e):
+    arg = e.op().args[0]
+    arg_formatted = t.translate(arg)
+    return 'MOD(EXTRACT(DAYOFWEEK FROM {}) + 5, 7)'.format(arg_formatted)
+
+
+@rewrites(ops.DayOfWeekName)
+def bigquery_day_of_week_name(e):
+    arg = e.op().args[0]
+    return arg.strftime('%A')
+
+
+@compiles(ops.Divide)
+def bigquery_compiles_divide(t, e):
+    return 'IEEE_DIVIDE({}, {})'.format(*map(t.translate, e.op().args))
+
+
+@compiles(ops.Strftime)
+def compiles_strftime(translator, expr):
+    arg, format_string = expr.op().args
+    arg_type = arg.type()
+    strftime_format_func_name = STRFTIME_FORMAT_FUNCTIONS[type(arg_type)]
+    fmt_string = translator.translate(format_string)
+    arg_formatted = translator.translate(arg)
+    if isinstance(arg_type, dt.Timestamp):
+        return 'FORMAT_{}({}, {}, {!r})'.format(
+            strftime_format_func_name,
+            fmt_string,
+            arg_formatted,
+            arg_type.timezone if arg_type.timezone is not None else 'UTC'
+        )
+    return 'FORMAT_{}({}, {})'.format(
+        strftime_format_func_name,
+        fmt_string,
+        arg_formatted
+    )
+
+
+@compiles(ops.StringToTimestamp)
+def compiles_string_to_timestamp(translator, expr):
+    arg, format_string, timezone_arg = expr.op().args
+    fmt_string = translator.translate(format_string)
+    arg_formatted = translator.translate(arg)
+    if timezone_arg is not None:
+        timezone_str = translator.translate(timezone_arg)
+        return 'PARSE_TIMESTAMP({}, {}, {})'.format(
+            fmt_string,
+            arg_formatted,
+            timezone_str
+        )
+    return 'PARSE_TIMESTAMP({}, {})'.format(fmt_string, arg_formatted)
+
+
 @rewrites(ops.Any)
 def bigquery_rewrite_any(expr):
     arg, = expr.op().args
@@ -390,10 +518,21 @@ def bigquery_rewrite_notall(expr):
     return (1 - arg.cast(dt.int64)).sum() != 0
 
 
+class BigQueryTableSetFormatter(ImpalaTableSetFormatter):
+    def _quote_identifier(self, name):
+        if re.match(r'^[A-Za-z][A-Za-z_0-9]*$', name):
+            return name
+        return '`{}`'.format(name)
+
+
 class BigQuerySelect(ImpalaSelect):
 
     translator = BigQueryExprTranslator
 
+    @property
+    def table_set_formatter(self):
+        return BigQueryTableSetFormatter
+
 
 @rewrites(ops.IdenticalTo)
 def identical_to(expr):
@@ -407,6 +546,46 @@ def log2(expr):
     return arg.log(2)
 
 
+@rewrites(ops.Sum)
+def bq_sum(expr):
+    arg = expr.op().args[0]
+    where = expr.op().args[1]
+    if isinstance(arg, ir.BooleanColumn):
+        return arg.cast('int64').sum(where=where)
+    else:
+        return expr
+
+
+@rewrites(ops.Mean)
+def bq_mean(expr):
+    arg = expr.op().args[0]
+    where = expr.op().args[1]
+    if isinstance(arg, ir.BooleanColumn):
+        return arg.cast('int64').mean(where=where)
+    else:
+        return expr
+
+
+UNIT_FUNCS = {
+    's': 'SECONDS',
+    'ms': 'MILLIS',
+    'us': 'MICROS',
+}
+
+
+@compiles(ops.TimestampFromUNIX)
+def compiles_timestamp_from_unix(t, e):
+    value, unit = e.op().args
+    return 'TIMESTAMP_{}({})'.format(UNIT_FUNCS[unit], t.translate(value))
+
+
+@compiles(ops.Floor)
+def compiles_floor(t, e):
+    bigquery_type = ibis_type_to_bigquery_type(e.type())
+    arg = e.op().arg
+    return 'CAST(FLOOR({}) AS {})'.format(t.translate(arg), bigquery_type)
+
+
 class BigQueryDialect(impala_compiler.ImpalaDialect):
 
     translator = BigQueryExprTranslator
