@@ -28,6 +28,7 @@ if not PY2:
     from inspect import signature, Parameter, _empty
     import unittest.mock as mock
     range = range
+    map = map
     import builtins
     import pickle
     maketrans = str.maketrans
@@ -45,6 +46,7 @@ else:
     lzip = zip
     zip = itertools.izip
     zip_longest = itertools.izip_longest
+    map = itertools.imap
 
     def viewkeys(x):
         return x.viewkeys()
@@ -107,11 +109,13 @@ except ImportError:
 
 # pandas compat
 try:
-    from pandas.api.types import (DatetimeTZDtype,  # noqa: F401
-                                  CategoricalDtype)  # noqa: F401
+    from pandas.api.types import (  # noqa: F401
+        DatetimeTZDtype, CategoricalDtype, infer_dtype
+    )
 except ImportError:
-    from pandas.types.dtypes import (DatetimeTZDtype,  # noqa: F401
-                                     CategoricalDtype)  # noqa: F401
+    from pandas.types.dtypes import (  # noqa: F401
+        DatetimeTZDtype, CategoricalDtype, infer_dtype
+    )
 
 try:
     from pandas.core.tools.datetimes import to_time, to_datetime  # noqa: F401
