@@ -1,29 +1,18 @@
-# Copyright 2014 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-import re
-import six
-import toolz
+import collections
 import datetime
 import itertools
+import numbers
+import re
+
+import six
 import pandas as pd
 
-from collections import namedtuple, OrderedDict
+import toolz
 from multipledispatch import Dispatcher
 
 import ibis.common as com
 from ibis.compat import PY2, builtins, functools
+import ibis.expr.types as ir
 
 
 class DataType(object):
@@ -92,9 +81,6 @@ class DataType(object):
     def _equal_part(self, other, cache=None):
         return True
 
-    def issubtype(self, parent):
-        return issubtype(self, parent)
-
     def castable(self, target, **kwargs):
         return castable(self, target, **kwargs)
 
@@ -102,12 +88,10 @@ class DataType(object):
         return cast(self, target, **kwargs)
 
     def scalar_type(self):
-        import ibis.expr.types as ir
-        return getattr(ir, '{}Scalar'.format(self.name))
+        return functools.partial(self.scalar, dtype=self)
 
     def array_type(self):
-        import ibis.expr.types as ir
-        return getattr(ir, '{}Column'.format(self.name))
+        return functools.partial(self.column, dtype=self)
 
 
 class Any(DataType):
@@ -127,6 +111,8 @@ class Primitive(DataType):
 
 
 class Null(DataType):
+    scalar = ir.NullScalar
+    column = ir.NullColumn
 
     __slots__ = ()
 
@@ -137,14 +123,18 @@ class Variadic(DataType):
 
 
 class Boolean(Primitive):
+    scalar = ir.BooleanScalar
+    column = ir.BooleanColumn
 
     __slots__ = ()
 
 
-Bounds = namedtuple('Bounds', ('lower', 'upper'))
+Bounds = collections.namedtuple('Bounds', ('lower', 'upper'))
 
 
 class Integer(Primitive):
+    scalar = ir.IntegerScalar
+    column = ir.IntegerColumn
 
     __slots__ = ()
 
@@ -163,6 +153,8 @@ class String(Variadic):
     Because of differences in the way different backends handle strings, we
     cannot assume that strings are UTF-8 encoded.
     """
+    scalar = ir.StringScalar
+    column = ir.StringColumn
 
     __slots__ = ()
 
@@ -177,42 +169,29 @@ class Binary(Variadic):
     but PostgreSQL has a TEXT type and a BYTEA type which are distinct types
     that behave differently.
     """
+    scalar = ir.BinaryScalar
+    column = ir.BinaryColumn
+
+    __slots__ = ()
 
 
 class Date(Primitive):
+    scalar = ir.DateScalar
+    column = ir.DateColumn
 
     __slots__ = ()
 
 
 class Time(Primitive):
+    scalar = ir.TimeScalar
+    column = ir.TimeColumn
 
     __slots__ = ()
 
 
-def parametric(cls):
-    type_name = cls.__name__
-    array_type_name = '{}Column'.format(type_name)
-    scalar_type_name = '{}Scalar'.format(type_name)
-
-    def array_type(self):
-        def constructor(op, name=None):
-            import ibis.expr.types as ir
-            return getattr(ir, array_type_name)(op, self, name=name)
-        return constructor
-
-    def scalar_type(self):
-        def constructor(op, name=None):
-            import ibis.expr.types as ir
-            return getattr(ir, scalar_type_name)(op, self, name=name)
-        return constructor
-
-    cls.array_type = array_type
-    cls.scalar_type = scalar_type
-    return cls
-
-
-@parametric
 class Timestamp(Primitive):
+    scalar = ir.TimestampScalar
+    column = ir.TimestampColumn
 
     __slots__ = 'timezone',
 
@@ -238,11 +217,18 @@ class Timestamp(Primitive):
 
 
 class SignedInteger(Integer):
-    pass
+
+    @property
+    def largest(self):
+        return int64
 
 
 class UnsignedInteger(Integer):
 
+    @property
+    def largest(self):
+        return uint64
+
     @property
     def bounds(self):
         exp = self._nbytes * 8 - 1
@@ -251,9 +237,15 @@ class UnsignedInteger(Integer):
 
 
 class Floating(Primitive):
+    scalar = ir.FloatingScalar
+    column = ir.FloatingColumn
 
     __slots__ = ()
 
+    @property
+    def largest(self):
+        return float64
+
 
 class Int8(SignedInteger):
 
@@ -285,26 +277,36 @@ class Int64(SignedInteger):
 
 class UInt8(UnsignedInteger):
 
+    __slots__ = ()
+
     _nbytes = 1
 
 
 class UInt16(UnsignedInteger):
 
+    __slots__ = ()
+
     _nbytes = 2
 
 
 class UInt32(UnsignedInteger):
 
+    __slots__ = ()
+
     _nbytes = 4
 
 
 class UInt64(UnsignedInteger):
 
+    __slots__ = ()
+
     _nbytes = 8
 
 
 class Halffloat(Floating):
 
+    __slots__ = ()
+
     _nbytes = 2
 
 
@@ -327,12 +329,31 @@ Float32 = Float
 Float64 = Double
 
 
-@parametric
 class Decimal(DataType):
+    scalar = ir.DecimalScalar
+    column = ir.DecimalColumn
 
     __slots__ = 'precision', 'scale'
 
     def __init__(self, precision, scale, nullable=True):
+        if not isinstance(precision, numbers.Integral):
+            raise TypeError('Decimal type precision must be an integer')
+        if not isinstance(scale, numbers.Integral):
+            raise TypeError('Decimal type scale must be an integer')
+        if precision < 0:
+            raise ValueError('Decimal type precision cannot be negative')
+        if not precision:
+            raise ValueError('Decimal type precision cannot be zero')
+        if scale < 0:
+            raise ValueError('Decimal type scale cannot be negative')
+        if precision < scale:
+            raise ValueError(
+                'Decimal type precision must be greater than or equal to '
+                'scale. Got precision={:d} and scale={:d}'.format(
+                    precision, scale
+                )
+            )
+
         super(Decimal, self).__init__(nullable=nullable)
         self.precision = precision
         self.scale = scale
@@ -347,12 +368,17 @@ class Decimal(DataType):
     def _equal_part(self, other, cache=None):
         return self.precision == other.precision and self.scale == other.scale
 
+    @property
+    def largest(self):
+        return Decimal(38, self.scale)
+
 
 assert hasattr(Decimal, '__hash__')
 
 
-@parametric
 class Interval(DataType):
+    scalar = ir.IntervalScalar
+    column = ir.IntervalColumn
 
     __slots__ = 'value_type', 'unit'
 
@@ -387,6 +413,10 @@ class Interval(DataType):
         self.unit = unit
         self.value_type = value_type
 
+    @property
+    def bounds(self):
+        return self.value_type.bounds
+
     @property
     def resolution(self):
         """Unit's name"""
@@ -403,8 +433,9 @@ class Interval(DataType):
                 self.value_type.equals(other.value_type, cache=cache))
 
 
-@parametric
 class Category(DataType):
+    scalar = ir.CategoryScalar
+    column = ir.CategoryColumn
 
     __slots__ = 'cardinality',
 
@@ -433,8 +464,9 @@ class Category(DataType):
             return infer(self.cardinality)
 
 
-@parametric
 class Struct(DataType):
+    scalar = ir.StructScalar
+    column = ir.StructColumn
 
     __slots__ = 'pairs',
 
@@ -456,7 +488,7 @@ class Struct(DataType):
             raise ValueError('names and types must have the same length')
 
         super(Struct, self).__init__(nullable=nullable)
-        self.pairs = OrderedDict(zip(names, types))
+        self.pairs = collections.OrderedDict(zip(names, types))
 
     @classmethod
     def from_tuples(self, pairs):
@@ -496,8 +528,9 @@ class Struct(DataType):
         )
 
 
-@parametric
 class Array(Variadic):
+    scalar = ir.ArrayScalar
+    column = ir.ArrayColumn
 
     __slots__ = 'value_type',
 
@@ -512,8 +545,26 @@ class Array(Variadic):
         return self.value_type.equals(other.value_type, cache=cache)
 
 
-@parametric
+class Set(Variadic):
+    scalar = ir.SetScalar
+    column = ir.SetColumn
+
+    __slots__ = 'value_type',
+
+    def __init__(self, value_type, nullable=True):
+        super(Set, self).__init__(nullable=nullable)
+        self.value_type = dtype(value_type)
+
+    def __str__(self):
+        return '{}<{}>'.format(self.name.lower(), self.value_type)
+
+    def _equal_part(self, other, cache=None):
+        return self.value_type.equals(other.value_type, cache=cache)
+
+
 class Enum(DataType):
+    scalar = ir.EnumScalar
+    column = ir.EnumColumn
 
     __slots__ = 'rep_type', 'value_type'
 
@@ -529,8 +580,9 @@ class Enum(DataType):
         )
 
 
-@parametric
 class Map(Variadic):
+    scalar = ir.MapScalar
+    column = ir.MapColumn
 
     __slots__ = 'key_type', 'value_type'
 
@@ -580,6 +632,7 @@ date = Date()
 time = Time()
 timestamp = Timestamp()
 interval = Interval()
+category = Category()
 
 
 _primitive_types = (
@@ -636,6 +689,7 @@ class Tokens(object):
     TIMESTAMP = 18
     TIME = 19
     INTERVAL = 20
+    SET = 21
 
     @staticmethod
     def name(value):
@@ -648,14 +702,14 @@ _token_names = dict(
 )
 
 
-Token = namedtuple('Token', ('type', 'value'))
+Token = collections.namedtuple('Token', ('type', 'value'))
 
 
 # Adapted from tokenize.String
 _STRING_REGEX = """('[^\n'\\\\]*(?:\\\\.[^\n'\\\\]*)*'|"[^\n"\\\\"]*(?:\\\\.[^\n"\\\\]*)*")"""  # noqa: E501
 
 
-_TYPE_RULES = OrderedDict(
+_TYPE_RULES = collections.OrderedDict(
     [
         # any, null, bool|boolean
         ('(?P<ANY>any)', lambda token: Token(Tokens.ANY, any)),
@@ -702,6 +756,7 @@ _TYPE_RULES = OrderedDict(
                 'varchar',
                 'char',
                 'array',
+                'set',
                 'map',
                 'struct',
                 'interval'
@@ -710,6 +765,7 @@ _TYPE_RULES = OrderedDict(
                 Tokens.VARCHAR,
                 Tokens.CHAR,
                 Tokens.ARRAY,
+                Tokens.SET,
                 Tokens.MAP,
                 Tokens.STRUCT,
                 Tokens.INTERVAL
@@ -827,6 +883,7 @@ class TypeParser(object):
         type : primitive
              | decimal
              | array
+             | set
              | map
              | struct
 
@@ -865,6 +922,8 @@ class TypeParser(object):
 
         array : "array" "<" type ">"
 
+        set : "set" "<" type ">"
+
         map : "map" "<" type "," type ">"
 
         struct : "struct" "<" field ":" type ("," field ":" type)* ">"
@@ -935,6 +994,14 @@ class TypeParser(object):
             self._expect(Tokens.RBRACKET)
             return Array(value_type)
 
+        elif self._accept(Tokens.SET):
+            self._expect(Tokens.LBRACKET)
+
+            value_type = self.type()
+
+            self._expect(Tokens.RBRACKET)
+            return Set(value_type)
+
         elif self._accept(Tokens.MAP):
             self._expect(Tokens.LBRACKET)
 
@@ -990,7 +1057,7 @@ validate_type = dtype
 
 @dtype.register(object)
 def default(value, **kwargs):
-    raise TypeError('Value {!r} is not a valid type or string'.format(value))
+    raise com.IbisTypeError('Value {!r} is not a valid datatype'.format(value))
 
 
 @dtype.register(DataType)
@@ -1000,7 +1067,26 @@ def from_ibis_dtype(value):
 
 @dtype.register(six.string_types)
 def from_string(value):
-    return TypeParser(value).parse()
+    try:
+        return TypeParser(value).parse()
+    except SyntaxError:
+        raise com.IbisTypeError(
+            '{!r} cannot be parsed as a datatype'.format(value)
+        )
+
+
+@dtype.register(list)
+def from_list(values):
+    if not values:
+        return Array(null)
+    return Array(highest_precedence(map(dtype, values)))
+
+
+@dtype.register(collections.Set)
+def from_set(values):
+    if not values:
+        return Set(null)
+    return Set(highest_precedence(map(dtype, values)))
 
 
 infer = Dispatcher('infer')
@@ -1012,8 +1098,9 @@ def higher_precedence(left, right):
     elif castable(right, left, upcast=True):
         return left
 
-    raise com.IbisTypeError('Cannot compute precedence for {} '
-                            'and {} types'.format(left, right))
+    raise com.IbisTypeError(
+        'Cannot compute precedence for {} and {} types'.format(left, right)
+    )
 
 
 def highest_precedence(dtypes):
@@ -1025,7 +1112,7 @@ def infer_dtype_default(value):
     raise com.InputTypeError(value)
 
 
-@infer.register(OrderedDict)
+@infer.register(collections.OrderedDict)
 def infer_struct(value):
     if not value:
         raise TypeError('Empty struct type not supported')
@@ -1046,10 +1133,17 @@ def infer_map(value):
 
 
 @infer.register(list)
-def infer_list(value):
-    if not value:
+def infer_list(values):
+    if not values:
         return Array(null)
-    return Array(highest_precedence(map(infer, value)))
+    return Array(highest_precedence(map(infer, values)))
+
+
+@infer.register((set, frozenset))
+def infer_set(values):
+    if not values:
+        return Set(null)
+    return Set(highest_precedence(map(infer, values)))
 
 
 @infer.register(datetime.time)
@@ -1064,7 +1158,10 @@ def infer_date(value):
 
 @infer.register(datetime.datetime)
 def infer_timestamp(value):
-    return timestamp
+    if value.tzinfo:
+        return Timestamp(timezone=str(value.tzinfo))
+    else:
+        return timestamp
 
 
 @infer.register(datetime.timedelta)
@@ -1113,10 +1210,12 @@ def can_cast_subtype(source, target, **kwargs):
 
 
 @castable.register(Any, DataType)
+@castable.register(DataType, Any)
+@castable.register(Any, Any)
+@castable.register(Null, Any)
 @castable.register(Integer, Category)
 @castable.register(Integer, (Floating, Decimal))
 @castable.register(Floating, Decimal)
-@castable.register(Decimal, Floating)
 @castable.register((Date, Timestamp), (Date, Timestamp))
 def can_cast_any(source, target, **kwargs):
     return True
@@ -1127,7 +1226,18 @@ def can_cast_null(source, target, **kwargs):
     return target.nullable
 
 
-@castable.register(Integer, Integer)
+@castable.register(SignedInteger, UnsignedInteger)
+@castable.register(UnsignedInteger, SignedInteger)
+def can_cast_to_unsigned(source, target, value=None, **kwargs):
+    if value is None:
+        return False
+
+    bounds = target.bounds
+    return bounds.lower <= value <= bounds.upper
+
+
+@castable.register(SignedInteger, SignedInteger)
+@castable.register(UnsignedInteger, UnsignedInteger)
 def can_cast_integers(source, target, **kwargs):
     return target._nbytes >= source._nbytes
 
@@ -1142,6 +1252,12 @@ def can_cast_floats(source, target, upcast=False, **kwargs):
     return True
 
 
+@castable.register(Decimal, Decimal)
+def can_cast_decimals(source, target, **kwargs):
+    return (target.precision >= source.precision and
+            target.scale >= source.scale)
+
+
 @castable.register(Interval, Interval)
 def can_cast_intervals(source, target, **kwargs):
     return (
@@ -1173,7 +1289,8 @@ def can_cast_string_to_temporal(source, target, value=None, **kwargs):
 
 
 @castable.register(Array, Array)
-def can_cast_arrays(source, target, **kwargs):
+@castable.register(Set, Set)
+def can_cast_variadic(source, target, **kwargs):
     return castable(source.value_type, target.value_type)
 
 
@@ -1193,15 +1310,3 @@ def cast(source, target, **kwargs):
         raise com.IbisTypeError('Datatype {} cannot be implicitly '
                                 'casted to {}'.format(source, target))
     return target
-
-
-def issubtype(dtype, dtype_or_tuple):
-    if not isinstance(dtype_or_tuple, tuple):
-        parents = (dtype_or_tuple,)
-    for parent in parents:
-        if isinstance(dtype, type(parent)):
-            return True
-        elif isinstance(dtype, Any):
-            return True
-
-    return False
