@@ -12,11 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ibis.expr.tests.mocks import MockConnection
-from ibis.compat import unittest
-import ibis.expr.types as ir
-import ibis
+import unittest
 
+import ibis
+import ibis.expr.types as ir
+
+from ibis.expr.tests.mocks import MockConnection
 from ibis.tests.util import assert_equal
 
 
@@ -80,7 +81,7 @@ class TestAnalytics(unittest.TestCase):
 
         filtered = t.filter([delay_filter])
 
-        post_pred = filtered.op().predicates[1]
+        post_pred = filtered.op().predicates[0]
         assert delay_filter.to_filter().equals(post_pred)
 
     def test_topk_function_late_bind(self):
