@@ -1,29 +1,31 @@
 import pytest
 import datetime
+import pytz
 from collections import OrderedDict
 from multipledispatch.conflict import ambiguities
 
 import ibis
-
-import ibis.expr.api as api
-import ibis.expr.types as types
-import ibis.expr.rules as rules
-
-from ibis import IbisError
-from ibis.expr import datatypes as dt
-from ibis.expr.rules import highest_precedence_type
+import ibis.expr.datatypes as dt
+from ibis.common import IbisTypeError
 
 
 def test_validate_type():
     assert dt.validate_type is dt.dtype
 
 
-def test_array():
-    assert dt.dtype('ARRAY<DOUBLE>') == dt.Array(dt.double)
-
-
-def test_nested_array():
-    assert dt.dtype('array<array<string>>') == dt.Array(dt.Array(dt.string))
+@pytest.mark.parametrize(('spec', 'expected'), [
+    ('ARRAY<DOUBLE>', dt.Array(dt.double)),
+    ('array<array<string>>', dt.Array(dt.Array(dt.string))),
+    ('map<string, double>', dt.Map(dt.string, dt.double)),
+    ('map<int64, array<map<string, int8>>>',
+     dt.Map(dt.int64, dt.Array(dt.Map(dt.string, dt.int8)))),
+    ('set<uint8>', dt.Set(dt.uint8)),
+    ([dt.uint8], dt.Array(dt.uint8)),
+    ([dt.float32, dt.float64], dt.Array(dt.float64)),
+    ({dt.string}, dt.Set(dt.string))
+])
+def test_dtype(spec, expected):
+    assert dt.dtype(spec) == expected
 
 
 def test_array_with_string_value_type():
@@ -33,15 +35,6 @@ def test_array_with_string_value_type():
     )
 
 
-def test_map():
-    assert dt.dtype('map<string, double>') == dt.Map(dt.string, dt.double)
-
-
-def test_nested_map():
-    expected = dt.Map(dt.int64, dt.Array(dt.Map(dt.string, dt.int8)))
-    assert dt.dtype('map<int64, array<map<string, int8>>>') == expected
-
-
 def test_map_with_string_value_type():
     assert dt.Map('int32', 'double') == dt.Map(dt.int32, dt.double)
     assert dt.Map('int32', 'array<double>') == \
@@ -49,17 +42,17 @@ def test_map_with_string_value_type():
 
 
 def test_map_does_not_allow_non_primitive_keys():
-    with pytest.raises(SyntaxError):
+    with pytest.raises(IbisTypeError):
         dt.dtype('map<array<string>, double>')
 
 
 def test_token_error():
-    with pytest.raises(SyntaxError):
+    with pytest.raises(IbisTypeError):
         dt.dtype('array<string>>')
 
 
 def test_empty_complex_type():
-    with pytest.raises(SyntaxError):
+    with pytest.raises(IbisTypeError):
         dt.dtype('map<>')
 
 
@@ -126,7 +119,7 @@ def test_struct_with_string_types():
     'decimal(3,',
 ])
 def test_decimal_failure(case):
-    with pytest.raises(SyntaxError):
+    with pytest.raises(IbisTypeError):
         dt.dtype(case)
 
 
@@ -149,7 +142,7 @@ def test_char_varchar(spec):
     'char()'
 ])
 def test_char_varchar_invalid(spec):
-    with pytest.raises(SyntaxError):
+    with pytest.raises(IbisTypeError):
         dt.dtype(spec)
 
 
@@ -182,28 +175,25 @@ def test_primitive(spec, expected):
     assert dt.dtype(spec) == expected
 
 
-def test_precedence_with_no_arguments():
-    with pytest.raises(ValueError) as e:
-        highest_precedence_type([])
-    assert str(e.value) == 'Must pass at least one expression'
-
-
-def test_rule_instance_of():
-    class MyOperation(types.Node):
-        input_type = [rules.instance_of(types.IntegerValue)]
-
-    MyOperation([api.literal(5)])
-
-    with pytest.raises(IbisError):
-        MyOperation([api.literal('string')])
-
-
 def test_literal_mixed_type_fails():
     data = [1, 'a']
     with pytest.raises(TypeError):
         ibis.literal(data)
 
 
+def test_timestamp_literal_without_tz():
+    now_raw = datetime.datetime.utcnow()
+    assert now_raw.tzinfo is None
+    assert ibis.literal(now_raw).type().timezone is None
+
+
+def test_timestamp_literal_with_tz():
+    now_raw = datetime.datetime.utcnow()
+    now_utc = pytz.utc.localize(now_raw)
+    assert now_utc.tzinfo == pytz.UTC
+    assert ibis.literal(now_utc).type().timezone == str(pytz.UTC)
+
+
 def test_array_type_not_equals():
     left = dt.Array(dt.string)
     right = dt.Array(dt.int32)
@@ -240,8 +230,8 @@ def test_timestamp_with_timezone_parser_invalid_timezone():
 
 
 @pytest.mark.parametrize('unit', [
-    'Y', 'Q', 'M', 'W',  'D',  # date units
-    'h',  'm',  's',  'ms', 'us', 'ns'  # time units
+    'Y', 'Q', 'M', 'W', 'D',  # date units
+    'h', 'm', 's', 'ms', 'us', 'ns'  # time units
 ])
 def test_interval(unit):
     definition = "interval('{}')".format(unit)
@@ -284,7 +274,7 @@ def test_interval_unvalid_unit(unit):
     'interval("Y\')',
 ])
 def test_string_argument_parsing_failure_mode(case):
-    with pytest.raises(SyntaxError):
+    with pytest.raises(IbisTypeError):
         dt.dtype(case)
 
 
@@ -340,6 +330,9 @@ def test_time_valid():
 
     # parametric types
     (list('abc'), dt.Array(dt.string)),
+    (set('abc'), dt.Set(dt.string)),
+    ({1, 5, 5, 6}, dt.Set(dt.int8)),
+    (frozenset(list('abc')), dt.Set(dt.string)),
     ([1, 2, 3], dt.Array(dt.int8)),
     ([1, 128], dt.Array(dt.int16)),
     ([1, 128, 32768], dt.Array(dt.int32)),
@@ -390,6 +383,7 @@ def test_implicit_castable(source, target):
 @pytest.mark.parametrize(('source', 'target'), [
     (dt.string, dt.null),
     (dt.int32, dt.int16),
+    (dt.int16, dt.uint64),
     (dt.Decimal(12, 2), dt.int32),
     (dt.timestamp, dt.boolean),
     (dt.boolean, dt.interval),
