@@ -1,30 +1,18 @@
-# Copyright 2015 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+import operator
+
+import pytest
 
 import ibis.expr.api as api
 import ibis.expr.types as ir
+import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 
-import pytest
-
 
 def test_type_metadata(lineitem):
     col = lineitem.l_extendedprice
     assert isinstance(col, ir.DecimalColumn)
 
-    assert col.meta.precision == 12
-    assert col.meta.scale == 2
+    assert col.type() == dt.Decimal(12, 2)
 
 
 def test_cast_scalar_to_decimal():
@@ -32,24 +20,23 @@ def test_cast_scalar_to_decimal():
 
     casted = val.cast('decimal(15,5)')
     assert isinstance(casted, ir.DecimalScalar)
-    assert casted.meta.precision == 15
-    assert casted.meta.scale == 5
+    assert casted.type() == dt.Decimal(15, 5)
 
 
-def test_decimal_aggregate_function_behavior(lineitem):
-    # From the Impala documentation: "The result of an aggregate function
-    # such as MAX(), SUM(), or AVG() on DECIMAL values is promoted to a
-    # scale of 38, with the same precision as the underlying column. Thus,
-    # the result can represent the largest possible value at that
-    # particular precision."
+def test_decimal_sum_type(lineitem):
     col = lineitem.l_extendedprice
-    functions = ['sum', 'mean', 'max', 'min']
+    result = col.sum()
+    assert isinstance(result, ir.DecimalScalar)
+    assert result.type() == dt.Decimal(38, col.type().scale)
 
-    for func_name in functions:
-        result = getattr(col, func_name)()
-        assert isinstance(result, ir.DecimalScalar)
-        assert result.meta.precision == col.meta.precision
-        assert result.meta.scale == 38
+
+@pytest.mark.parametrize('func', ['mean', 'max', 'min'])
+def test_decimal_aggregate_function_type(lineitem, func):
+    col = lineitem.l_extendedprice
+    method = operator.methodcaller(func)
+    result = method(col)
+    assert isinstance(result, ir.DecimalScalar)
+    assert result.type() == col.type()
 
 
 def test_where(lineitem):
@@ -57,7 +44,8 @@ def test_where(lineitem):
 
     q = table.l_quantity
     expr = api.where(table.l_discount > 0,
-                     q * table.l_discount, api.null)
+                     q * table.l_discount,
+                     api.null)
 
     assert isinstance(expr, ir.DecimalColumn)
 
@@ -91,9 +79,30 @@ def test_precision_scale(lineitem):
     assert isinstance(s.op(), ops.DecimalScale)
 
 
-@pytest.mark.xfail(raises=AssertionError, reason='NYI')
-def test_invalid_precision_scale_combo():
-    assert False
+@pytest.mark.parametrize(
+    ('precision', 'scale'),
+    [
+        (-1, 3),  # negative precision
+        (0, 1),  # zero precision
+        (12, 38),  # precision less than scale
+        (33, -1),  # negative scale
+    ]
+)
+def test_invalid_precision_scale_combo(precision, scale):
+    with pytest.raises(ValueError):
+        dt.Decimal(precision, scale)
+
+
+@pytest.mark.parametrize(
+    ('precision', 'scale'),
+    [
+        (38.1, 3),  # non integral precision
+        (38, 3.1),  # non integral scale
+    ]
+)
+def test_invalid_precision_scale_type(precision, scale):
+    with pytest.raises(TypeError):
+        dt.Decimal(precision, scale)
 
 
 def test_decimal_str(lineitem):
