@@ -82,7 +82,7 @@ def test_zeroifnull(functional_alltypes):
     iresult = functional_alltypes.int_col.zeroifnull()
 
     assert type(dresult.op()) == ops.ZeroIfNull
-    assert type(dresult) == ir.DoubleColumn
+    assert type(dresult) == ir.FloatingColumn
 
     # Impala upconverts all ints to bigint. Hmm.
     assert type(iresult) == type(iresult)
@@ -90,7 +90,7 @@ def test_zeroifnull(functional_alltypes):
 
 def test_fillna(functional_alltypes):
     result = functional_alltypes.double_col.fillna(5)
-    assert isinstance(result, ir.DoubleColumn)
+    assert isinstance(result, ir.FloatingColumn)
 
     assert isinstance(result.op(), ops.IfNull)
 
@@ -99,58 +99,58 @@ def test_fillna(functional_alltypes):
 
     # Highest precedence type
     result = functional_alltypes.int_col.fillna(functional_alltypes.bigint_col)
-    assert isinstance(result, ir.Int64Column)
+    assert isinstance(result, ir.IntegerColumn)
 
 
 def test_ceil_floor(functional_alltypes, lineitem):
     cresult = functional_alltypes.double_col.ceil()
     fresult = functional_alltypes.double_col.floor()
-    assert isinstance(cresult, ir.Int64Column)
-    assert isinstance(fresult, ir.Int64Column)
+    assert isinstance(cresult, ir.IntegerColumn)
+    assert isinstance(fresult, ir.IntegerColumn)
     assert type(cresult.op()) == ops.Ceil
     assert type(fresult.op()) == ops.Floor
 
     cresult = ibis.literal(1.2345).ceil()
     fresult = ibis.literal(1.2345).floor()
-    assert isinstance(cresult, ir.Int64Scalar)
-    assert isinstance(fresult, ir.Int64Scalar)
+    assert isinstance(cresult, ir.IntegerScalar)
+    assert isinstance(fresult, ir.IntegerScalar)
 
     dec_col = lineitem.l_extendedprice
     cresult = dec_col.ceil()
     fresult = dec_col.floor()
     assert isinstance(cresult, ir.DecimalColumn)
-    assert cresult.meta == dec_col.meta
+    assert cresult.type() == dec_col.type()
 
     assert isinstance(fresult, ir.DecimalColumn)
-    assert fresult.meta == dec_col.meta
+    assert fresult.type() == dec_col.type()
 
 
 def test_sign(functional_alltypes, lineitem):
     result = functional_alltypes.double_col.sign()
-    assert isinstance(result, ir.FloatColumn)
+    assert isinstance(result, ir.FloatingColumn)
     assert type(result.op()) == ops.Sign
 
     result = ibis.literal(1.2345).sign()
-    assert isinstance(result, ir.FloatScalar)
+    assert isinstance(result, ir.FloatingScalar)
 
     dec_col = lineitem.l_extendedprice
     result = dec_col.sign()
-    assert isinstance(result, ir.FloatColumn)
+    assert isinstance(result, ir.DecimalColumn)
 
 
 def test_round(functional_alltypes, lineitem):
     result = functional_alltypes.double_col.round()
-    assert isinstance(result, ir.Int64Column)
+    assert isinstance(result, ir.IntegerColumn)
     assert result.op().args[1] is None
 
     result = functional_alltypes.double_col.round(2)
-    assert isinstance(result, ir.DoubleColumn)
+    assert isinstance(result, ir.FloatingColumn)
     assert result.op().args[1].equals(ibis.literal(2))
 
     # Even integers are double (at least in Impala, check with other DB
     # implementations)
     result = functional_alltypes.int_col.round(2)
-    assert isinstance(result, ir.DoubleColumn)
+    assert isinstance(result, ir.FloatingColumn)
 
     dec = lineitem.l_extendedprice
     result = dec.round()
@@ -160,7 +160,7 @@ def test_round(functional_alltypes, lineitem):
     assert isinstance(result, ir.DecimalColumn)
 
     result = ibis.literal(1.2345).round()
-    assert isinstance(result, ir.Int64Scalar)
+    assert isinstance(result, ir.IntegerScalar)
 
 
 def _check_unary_op(expr, fname, ex_op, ex_type):
@@ -183,26 +183,26 @@ def test_integer_promotions(sql_table, function):
     t = sql_table
 
     expr = function(t.v3, t.v4)
-    assert isinstance(expr, ir.Int64Column)
+    assert isinstance(expr, ir.IntegerColumn)
 
     expr = function(5, t.v3)
-    assert isinstance(expr, ir.Int64Column)
+    assert isinstance(expr, ir.IntegerColumn)
 
     expr = function(5, 12)
-    assert isinstance(expr, ir.Int64Scalar)
+    assert isinstance(expr, ir.IntegerScalar)
 
 
 def test_floats(sql_table, function):
     t = sql_table
 
     expr = function(t.v5)
-    assert isinstance(expr, ir.DoubleColumn)
+    assert isinstance(expr, ir.FloatingColumn)
 
     expr = function(5.5, t.v5)
-    assert isinstance(expr, ir.DoubleColumn)
+    assert isinstance(expr, ir.FloatingColumn)
 
     expr = function(5.5, 5)
-    assert isinstance(expr, ir.DoubleScalar)
+    assert isinstance(expr, ir.FloatingScalar)
 
 
 @pytest.mark.xfail(raises=AssertionError, reason='NYT')
