@@ -82,7 +82,25 @@ def test_cannot_upconvert(delta, target):
 ])
 def test_multiply(expr):
     assert isinstance(expr, ir.IntervalScalar)
-    assert expr.unit == 'D'
+    assert expr.type().unit == 'D'
+
+
+@pytest.mark.parametrize('expr', [
+    api.day() + api.day(),
+    api.day(2) + api.hour(4)
+])
+def test_add(expr):
+    assert isinstance(expr, ir.IntervalScalar)
+    assert expr.type().unit == 'D'
+
+
+@pytest.mark.parametrize('expr', [
+    api.day(3) - api.day(),
+    api.day(2) - api.hour(4)
+])
+def test_subtract(expr):
+    assert isinstance(expr, ir.IntervalScalar)
+    assert expr.type().unit == 'D'
 
 
 @pytest.mark.parametrize(('case', 'expected'), [
@@ -103,7 +121,7 @@ def test_multiply(expr):
 def test_downconvert_second_parts(case, expected):
     assert isinstance(case, ir.IntervalScalar)
     assert isinstance(expected, ir.IntervalScalar)
-    assert case.unit == expected.unit
+    assert case.type().unit == expected.type().unit
 
 
 @pytest.mark.parametrize(('case', 'expected'), [
@@ -117,7 +135,7 @@ def test_downconvert_second_parts(case, expected):
 def test_downconvert_hours(case, expected):
     assert isinstance(case, ir.IntervalScalar)
     assert isinstance(expected, ir.IntervalScalar)
-    assert case.unit == expected.unit
+    assert case.type().unit == expected.type().unit
 
 
 @pytest.mark.parametrize(('case', 'expected'), [
@@ -135,7 +153,7 @@ def test_downconvert_hours(case, expected):
 def test_downconvert_day(case, expected):
     assert isinstance(case, ir.IntervalScalar)
     assert isinstance(expected, ir.IntervalScalar)
-    assert case.unit == expected.unit
+    assert case.type().unit == expected.type().unit
 
 
 @pytest.mark.parametrize(('a', 'b', 'unit'), [
@@ -175,6 +193,15 @@ def test_interval_time_expr(table):
     assert isinstance(expr, ir.TimeColumn)
     assert isinstance(expr.op(), ops.TimeAdd)
 
+    expr = x - c
+    assert isinstance(expr, ir.TimeColumn)
+    assert isinstance(expr.op(), ops.TimeSub)
+
+    # test radd
+    expr = c - x
+    assert isinstance(expr, ir.TimeColumn)
+    assert isinstance(expr.op(), ops.TimeSub)
+
 
 def test_interval_date_expr(table):
     c = table.j
@@ -189,6 +216,15 @@ def test_interval_date_expr(table):
     assert isinstance(expr, ir.DateColumn)
     assert isinstance(expr.op(), ops.DateAdd)
 
+    expr = x - c
+    assert isinstance(expr, ir.DateColumn)
+    assert isinstance(expr.op(), ops.DateSub)
+
+    # test radd
+    expr = c - x
+    assert isinstance(expr, ir.DateColumn)
+    assert isinstance(expr.op(), ops.DateSub)
+
 
 def test_interval_timestamp_expr(table):
     c = table.i
@@ -203,6 +239,15 @@ def test_interval_timestamp_expr(table):
     assert isinstance(expr, ir.TimestampColumn)
     assert isinstance(expr.op(), ops.TimestampAdd)
 
+    expr = x - c
+    assert isinstance(expr, ir.TimestampColumn)
+    assert isinstance(expr.op(), ops.TimestampSub)
+
+    # test radd
+    expr = c - x
+    assert isinstance(expr, ir.TimestampColumn)
+    assert isinstance(expr.op(), ops.TimestampSub)
+
 
 @pytest.mark.xfail(raises=AssertionError, reason='NYI')
 def test_compound_offset():
