@@ -1,14 +1,15 @@
 import os
-
 import pytest
 
 pytest.importorskip('graphviz')
 
 import ibis  # noqa: E402
 import ibis.expr.types as ir  # noqa: E402
+import ibis.expr.rules as rlz  # noqa: E402
 import ibis.expr.visualize as viz  # noqa: E402
+import ibis.expr.operations as ops  # noqa: E402
 
-from ibis.expr import rules  # noqa: E402
+from ibis.expr.signature import Argument as Arg  # noqa: E402
 
 pytestmark = pytest.mark.skipif(
     int(os.environ.get('CONDA_BUILD', 0)) == 1, reason='CONDA_BUILD defined'
@@ -33,28 +34,25 @@ pytestmark = pytest.mark.skipif(
 def test_exprs(table, expr_func):
     expr = expr_func(table)
     graph = viz.to_graph(expr)
-    assert str(hash(repr(table.op()))) in graph.source
-    assert str(hash(repr(expr.op()))) in graph.source
+    assert str(hash(table.op())) in graph.source
+    assert str(hash(expr.op())) in graph.source
 
 
 def test_custom_expr():
     class MyExpr(ir.Expr):
         pass
 
-    class MyExprNode(ir.Node):
-
-        input_type = [
-            rules.string(name='foo'),
-            rules.number(name='bar'),
-        ]
+    class MyExprNode(ops.Node):
+        foo = Arg(rlz.string)
+        bar = Arg(rlz.numeric)
 
         def output_type(self):
             return MyExpr
 
-    op = MyExprNode(['Hello!', 42.3])
+    op = MyExprNode('Hello!', 42.3)
     expr = op.to_expr()
     graph = viz.to_graph(expr)
-    assert str(hash(repr(op))) in graph.source
+    assert str(hash(op)) in graph.source
 
 
 def test_custom_expr_with_not_implemented_type():
@@ -66,20 +64,17 @@ def test_custom_expr_with_not_implemented_type():
         def schema(self):
             raise NotImplementedError
 
-    class MyExprNode(ir.Node):
-
-        input_type = [
-            rules.string(name='foo'),
-            rules.number(name='bar'),
-        ]
+    class MyExprNode(ops.Node):
+        foo = Arg(rlz.string)
+        bar = Arg(rlz.numeric)
 
         def output_type(self):
             return MyExpr
 
-    op = MyExprNode(['Hello!', 42.3])
+    op = MyExprNode('Hello!', 42.3)
     expr = op.to_expr()
     graph = viz.to_graph(expr)
-    assert str(hash(repr(op))) in graph.source
+    assert str(hash(op)) in graph.source
 
 
 @pytest.mark.parametrize('how', ['inner', 'left', 'right', 'outer'])
@@ -89,7 +84,7 @@ def test_join(how):
     joined = left.join(right, left.b == right.b, how=how)
     result = joined[left.a, right.c]
     graph = viz.to_graph(result)
-    assert str(hash(repr(result.op()))) in graph.source
+    assert str(hash(result.op())) in graph.source
 
 
 def test_sort_by():
@@ -98,7 +93,7 @@ def test_sort_by():
         sum_a=t.a.sum().cast('double')
     ).sort_by('c')
     graph = viz.to_graph(expr)
-    assert str(hash(repr(expr.op()))) in graph.source
+    assert str(hash(expr.op())) in graph.source
 
 
 @pytest.mark.skipif(
@@ -121,3 +116,15 @@ def test_optional_graphviz_repr():
     # turn it back on
     ibis.options.graphviz_repr = True
     assert expr._repr_png_() is not None
+
+
+def test_between():
+    t = ibis.table([('a', 'int64'), ('b', 'string'), ('c', 'int32')])
+    expr = t.a.between(1, 1)
+    lower_bound, upper_bound = expr.op().args[1:]
+    graph = viz.to_graph(expr)
+    source = graph.source
+
+    # one for the node itself and one for the edge to between
+    assert source.count(str(hash(lower_bound.op()))) == 2
+    assert source.count(str(hash(upper_bound.op()))) == 2
