@@ -1,23 +1,9 @@
-# Copyright 2014 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import re
 import six
-import threading
 import time
 import weakref
 import traceback
+import threading
 
 from posixpath import join as pjoin
 from collections import deque
@@ -25,23 +11,23 @@ from collections import deque
 import numpy as np
 import pandas as pd
 
+import ibis.util as util
 import ibis.common as com
+import ibis.expr.types as ir
+import ibis.expr.rules as rlz
+import ibis.expr.schema as sch
+import ibis.expr.datatypes as dt
+import ibis.expr.operations as ops
 
 from ibis.config import options
-from ibis.client import (Query, AsyncQuery, Database,
-                         DatabaseEntity, SQLClient)
+from ibis.client import Query, Database, DatabaseEntity, SQLClient
 from ibis.compat import lzip, parse_version
 from ibis.filesystems import HDFS, WebHDFS
 from ibis.impala import udf, ddl
 from ibis.impala.compat import impyla, ImpylaError, HS2Error
 from ibis.impala.compiler import build_ast, ImpalaDialect
 from ibis.util import log
-from ibis.sql.compiler import DDL
-import ibis.expr.types as ir
-import ibis.expr.schema as sch
-import ibis.expr.operations as ops
-import ibis.expr.datatypes as dt
-import ibis.util as util
+from ibis.sql.compiler import DDL, DML
 
 
 class ImpalaDatabase(Database):
@@ -83,8 +69,6 @@ class ImpalaConnection(object):
         self.connection_pool = deque(maxlen=pool_size)
         self.connection_pool_size = 0
 
-        self.ping()
-
     def set_options(self, options):
         self.options.update(options)
 
@@ -108,15 +92,15 @@ class ImpalaConnection(object):
         elif key in self.options:
             del self.options[key]
 
-    def execute(self, query, async=False):
-        if isinstance(query, DDL):
+    def execute(self, query):
+        if isinstance(query, (DDL, DML)):
             query = query.compile()
 
         cursor = self._get_cursor()
         self.log(query)
 
         try:
-            cursor.execute(query, async=async)
+            cursor.execute(query)
         except Exception:
             cursor.release()
             self.error(
@@ -231,12 +215,9 @@ class ImpalaCursor(object):
             self.con.release(self)
             self.released = True
 
-    def execute(self, stmt, async=False):
+    def execute(self, stmt):
         self._cursor.execute_async(stmt)
-        if async:
-            return
-        else:
-            self._wait_synchronous()
+        self._wait_synchronous()
 
     def _wait_synchronous(self):
         # Wait to finish, but cancel if KeyboardInterrupt
@@ -363,90 +344,6 @@ def _chunks_to_pandas_array(chunks):
     return target
 
 
-class ImpalaAsyncQuery(ImpalaQuery, AsyncQuery):
-
-    def __init__(self, client, ddl):
-        super(ImpalaAsyncQuery, self).__init__(client, ddl)
-        self._cursor = None
-        self._exception = None
-        self._execute_thread = None
-        self._execute_complete = False
-        self._operation_active = False
-
-    def __del__(self):
-        try:
-            self._cursor.release()
-        except AttributeError:
-            pass
-
-    def execute(self):
-        if self._operation_active:
-            raise com.IbisError('operation already active')
-        con = self.client.con
-
-        # XXX: there is codegen overhead somewhere which causes execute_async
-        # to block, unfortunately. This threading hack works around it
-        def _async_execute():
-            try:
-                self._cursor = con.execute(self.compiled_ddl, async=True)
-            except Exception as e:
-                self._exception = e
-            self._execute_complete = True
-
-        self._execute_complete = False
-        self._operation_active = True
-        self._execute_thread = threading.Thread(target=_async_execute)
-        self._execute_thread.start()
-        return self
-
-    def _wait_execute(self):
-        if not self._operation_active:
-            raise com.IbisError('No active query')
-        if self._execute_thread.is_alive():
-            self._execute_thread.join()
-        elif self._exception is not None:
-            raise self._exception
-
-    def is_finished(self):
-        """
-        Return True if the operation is finished
-        """
-        from impala.error import ProgrammingError
-        self._wait_execute()
-        try:
-            return self._cursor.is_finished()
-        except ProgrammingError as e:
-            if 'state is not available' in e.args[0]:
-                return True
-            raise
-
-    def cancel(self):
-        """
-        Cancel the query (or attempt to)
-        """
-        self._wait_execute()
-        return self._cursor.cancel()
-
-    def status(self):
-        """
-        Retrieve Impala query status
-        """
-        self._wait_execute()
-        return self._cursor.status()
-
-    def wait(self, progress_bar=True):
-        raise NotImplementedError
-
-    def get_result(self):
-        """
-        Presuming the operation is completed, return the cursor result as would
-        be returned by the synchronous query API
-        """
-        self._wait_execute()
-        result = self._fetch(self._cursor)
-        return self._wrap_result(result)
-
-
 _HS2_TTypeId_to_dtype = {
     'BOOLEAN': 'bool',
     'TINYINT': 'int8',
@@ -464,16 +361,375 @@ _HS2_TTypeId_to_dtype = {
 }
 
 
+class ImpalaDatabaseTable(ops.DatabaseTable):
+    pass
+
+
+class ImpalaTable(ir.TableExpr, DatabaseEntity):
+
+    """
+    References a physical table in the Impala-Hive metastore
+    """
+
+    @property
+    def _qualified_name(self):
+        return self.op().args[0]
+
+    @property
+    def _unqualified_name(self):
+        return self._match_name()[1]
+
+    @property
+    def _client(self):
+        return self.op().args[2]
+
+    def _match_name(self):
+        m = ddl.fully_qualified_re.match(self._qualified_name)
+        if not m:
+            raise com.IbisError('Cannot determine database name from {0}'
+                                .format(self._qualified_name))
+        db, quoted, unquoted = m.groups()
+        return db, quoted or unquoted
+
+    @property
+    def _database(self):
+        return self._match_name()[0]
+
+    def compute_stats(self, incremental=False):
+        """
+        Invoke Impala COMPUTE STATS command to compute column, table, and
+        partition statistics.
+
+        See also ImpalaClient.compute_stats
+        """
+        return self._client.compute_stats(self._qualified_name,
+                                          incremental=incremental)
+
+    def invalidate_metadata(self):
+        self._client.invalidate_metadata(self._qualified_name)
+
+    def refresh(self):
+        self._client.refresh(self._qualified_name)
+
+    def metadata(self):
+        """
+        Return parsed results of DESCRIBE FORMATTED statement
+
+        Returns
+        -------
+        meta : TableMetadata
+        """
+        return self._client.describe_formatted(self._qualified_name)
+
+    describe_formatted = metadata
+
+    def files(self):
+        """
+        Return results of SHOW FILES statement
+        """
+        return self._client.show_files(self._qualified_name)
+
+    def drop(self):
+        """
+        Drop the table from the database
+        """
+        self._client.drop_table_or_view(self._qualified_name)
+
+    def truncate(self):
+        self._client.truncate_table(self._qualified_name)
+
+    def insert(self, obj=None, overwrite=False, partition=None,
+               values=None, validate=True):
+        """
+        Insert into Impala table. Wraps ImpalaClient.insert
+
+        Parameters
+        ----------
+        obj : TableExpr or pandas DataFrame
+        overwrite : boolean, default False
+          If True, will replace existing contents of table
+        partition : list or dict, optional
+          For partitioned tables, indicate the partition that's being inserted
+          into, either with an ordered list of partition keys or a dict of
+          partition field name to value. For example for the partition
+          (year=2007, month=7), this can be either (2007, 7) or {'year': 2007,
+          'month': 7}.
+        validate : boolean, default True
+          If True, do more rigorous validation that schema of table being
+          inserted is compatible with the existing table
+
+        Examples
+        --------
+        >>> t.insert(table_expr)  # doctest: +SKIP
+
+        # Completely overwrite contents
+        >>> t.insert(table_expr, overwrite=True)  # doctest: +SKIP
+        """
+        if isinstance(obj, pd.DataFrame):
+            from ibis.impala.pandas_interop import write_temp_dataframe
+            writer, expr = write_temp_dataframe(self._client, obj)
+        else:
+            expr = obj
+
+        if values is not None:
+            raise NotImplementedError
+
+        if validate:
+            existing_schema = self.schema()
+            insert_schema = expr.schema()
+            if not insert_schema.equals(existing_schema):
+                _validate_compatible(insert_schema, existing_schema)
+
+        if partition is not None:
+            partition_schema = self.partition_schema()
+            partition_schema_names = frozenset(partition_schema.names)
+            expr = expr.projection([
+                column for column in expr.columns
+                if column not in partition_schema_names
+            ])
+        else:
+            partition_schema = None
+
+        ast = build_ast(expr, ImpalaDialect.make_context())
+        select = ast.queries[0]
+        statement = ddl.InsertSelect(self._qualified_name,
+                                     select,
+                                     partition=partition,
+                                     partition_schema=partition_schema,
+                                     overwrite=overwrite)
+        return self._execute(statement)
+
+    def load_data(self, path, overwrite=False, partition=None):
+        """
+        Wraps the LOAD DATA DDL statement. Loads data into an Impala table by
+        physically moving data files.
+
+        Parameters
+        ----------
+        path : string
+        overwrite : boolean, default False
+          Overwrite the existing data in the entire table or indicated
+          partition
+        partition : dict, optional
+          If specified, the partition must already exist
+
+        Returns
+        -------
+        query : ImpalaQuery
+        """
+        if partition is not None:
+            partition_schema = self.partition_schema()
+        else:
+            partition_schema = None
+
+        stmt = ddl.LoadData(self._qualified_name, path,
+                            partition=partition,
+                            partition_schema=partition_schema)
+
+        return self._execute(stmt)
+
+    @property
+    def name(self):
+        return self.op().name
+
+    def rename(self, new_name, database=None):
+        """
+        Rename table inside Impala. References to the old table are no longer
+        valid.
+
+        Parameters
+        ----------
+        new_name : string
+        database : string
+
+        Returns
+        -------
+        renamed : ImpalaTable
+        """
+        m = ddl.fully_qualified_re.match(new_name)
+        if not m and database is None:
+            database = self._database
+        statement = ddl.RenameTable(self._qualified_name, new_name,
+                                    new_database=database)
+        self._client._execute(statement)
+
+        op = self.op().change_name(statement.new_qualified_name)
+        return type(self)(op)
+
+    def _execute(self, stmt):
+        return self._client._execute(stmt)
+
+    @property
+    def is_partitioned(self):
+        """
+        True if the table is partitioned
+        """
+        return self.metadata().is_partitioned
+
+    def partition_schema(self):
+        """
+        For partitioned tables, return the schema (names and types) for the
+        partition columns
+
+        Returns
+        -------
+        partition_schema : ibis Schema
+        """
+        schema = self.schema()
+        name_to_type = dict(zip(schema.names, schema.types))
+
+        result = self.partitions()
+
+        partition_fields = []
+        for x in result.columns:
+            if x not in name_to_type:
+                break
+            partition_fields.append((x, name_to_type[x]))
+
+        pnames, ptypes = zip(*partition_fields)
+        return sch.Schema(pnames, ptypes)
+
+    def add_partition(self, spec, location=None):
+        """
+        Add a new table partition, creating any new directories in HDFS if
+        necessary.
+
+        Partition parameters can be set in a single DDL statement, or you can
+        use alter_partition to set them after the fact.
+
+        Returns
+        -------
+        None (for now)
+        """
+        part_schema = self.partition_schema()
+        stmt = ddl.AddPartition(self._qualified_name, spec, part_schema,
+                                location=location)
+        return self._execute(stmt)
+
+    def alter(self, location=None, format=None, tbl_properties=None,
+              serde_properties=None):
+        """
+        Change setting and parameters of the table.
+
+        Parameters
+        ----------
+        location : string, optional
+          For partitioned tables, you may want the alter_partition function
+        format : string, optional
+        tbl_properties : dict, optional
+        serde_properties : dict, optional
+
+        Returns
+        -------
+        None (for now)
+        """
+        def _run_ddl(**kwds):
+            stmt = ddl.AlterTable(self._qualified_name, **kwds)
+            return self._execute(stmt)
+
+        return self._alter_table_helper(_run_ddl, location=location,
+                                        format=format,
+                                        tbl_properties=tbl_properties,
+                                        serde_properties=serde_properties)
+
+    def set_external(self, is_external=True):
+        """
+        Toggle EXTERNAL table property.
+        """
+        self.alter(tbl_properties={'EXTERNAL': is_external})
+
+    def alter_partition(self, spec, location=None, format=None,
+                        tbl_properties=None,
+                        serde_properties=None):
+        """
+        Change setting and parameters of an existing partition
+
+        Parameters
+        ----------
+        spec : dict or list
+          The partition keys for the partition being modified
+        location : string, optional
+        format : string, optional
+        tbl_properties : dict, optional
+        serde_properties : dict, optional
+
+        Returns
+        -------
+        None (for now)
+        """
+        part_schema = self.partition_schema()
+
+        def _run_ddl(**kwds):
+            stmt = ddl.AlterPartition(self._qualified_name, spec,
+                                      part_schema, **kwds)
+            return self._execute(stmt)
+
+        return self._alter_table_helper(_run_ddl, location=location,
+                                        format=format,
+                                        tbl_properties=tbl_properties,
+                                        serde_properties=serde_properties)
+
+    def _alter_table_helper(self, f, **alterations):
+        results = []
+        for k, v in alterations.items():
+            if v is None:
+                continue
+            result = f(**{k: v})
+            results.append(result)
+        return results
+
+    def drop_partition(self, spec):
+        """
+        Drop an existing table partition
+        """
+        part_schema = self.partition_schema()
+        stmt = ddl.DropPartition(self._qualified_name, spec, part_schema)
+        return self._execute(stmt)
+
+    def partitions(self):
+        """
+        Return a pandas.DataFrame giving information about this table's
+        partitions. Raises an exception if the table is not partitioned.
+
+        Returns
+        -------
+        partitions : pandas.DataFrame
+        """
+        return self._client.list_partitions(self._qualified_name)
+
+    def stats(self):
+        """
+        Return results of SHOW TABLE STATS as a DataFrame. If not partitioned,
+        contains only one row
+
+        Returns
+        -------
+        stats : pandas.DataFrame
+        """
+        return self._client.table_stats(self._qualified_name)
+
+    def column_stats(self):
+        """
+        Return results of SHOW COLUMN STATS as a pandas DataFrame
+
+        Returns
+        -------
+        column_stats : pandas.DataFrame
+        """
+        return self._client.column_stats(self._qualified_name)
+
+
 class ImpalaClient(SQLClient):
 
     """
     An Ibis client interface that uses Impala
     """
 
-    database_class = ImpalaDatabase
-    sync_query = ImpalaQuery
-    async_query = ImpalaAsyncQuery
     dialect = ImpalaDialect
+    database_class = ImpalaDatabase
+    query_class = ImpalaQuery
+    table_class = ImpalaDatabaseTable
+    table_expr_class = ImpalaTable
 
     def __init__(self, con, hdfs_client=None, **params):
         import hdfs
@@ -515,10 +771,6 @@ class ImpalaClient(SQLClient):
             self._kudu = KuduImpalaInterface(self)
         return self._kudu
 
-    @property
-    def _table_expr_klass(self):
-        return ImpalaTable
-
     def close(self):
         """
         Close Impala connection and drop any temporary objects
@@ -712,18 +964,14 @@ class ImpalaClient(SQLClient):
         schema : ibis Schema
         """
         qualified_name = self._fully_qualified_name(table_name, database)
-        query = 'DESCRIBE {0}'.format(qualified_name)
-        tuples = self.con.fetchall(query)
+        query = 'DESCRIBE {}'.format(qualified_name)
 
-        names, types, comments = zip(*tuples)
+        # only pull out the first two columns which are names and types
+        pairs = [row[:2] for row in self.con.fetchall(query)]
 
-        ibis_types = []
-        for t in types:
-            t = t.lower()
-            t = udf.parse_type(t)
-            ibis_types.append(t)
-
-        names = [x.lower() for x in names]
+        names, types = zip(*pairs)
+        ibis_types = [udf.parse_type(type.lower()) for type in types]
+        names = [name.lower() for name in names]
 
         return sch.Schema(names, ibis_types)
 
@@ -744,8 +992,7 @@ class ImpalaClient(SQLClient):
         Return current query options for the Impala session
         """
         query = 'SET'
-        tuples = self.con.fetchall(query)
-        return dict(tuples)
+        return dict(row[:2] for row in self.con.fetchall(query))
 
     def set_options(self, options):
         self.con.set_options(options)
@@ -1053,7 +1300,7 @@ class ImpalaClient(SQLClient):
         else:
             schema = self._get_table_schema(qualified_name)
             node = ImpalaTemporaryTable(qualified_name, schema, self)
-            t = self._table_expr_klass(node)
+            t = self.table_expr_class(node)
 
         # Compute number of rows in table for better default query planning
         cardinality = t.count().execute()
@@ -1218,20 +1465,9 @@ class ImpalaClient(SQLClient):
         database = database or self.current_database
 
         if isinstance(func, udf.ImpalaUDF):
-            stmt = ddl.CreateFunction(func.lib_path, func.so_symbol,
-                                      func.input_type,
-                                      func.output,
-                                      name, database)
+            stmt = ddl.CreateUDF(func, name=name, database=database)
         elif isinstance(func, udf.ImpalaUDA):
-            stmt = ddl.CreateAggregateFunction(func.lib_path,
-                                               func.input_type,
-                                               func.output,
-                                               func.update_fn,
-                                               func.init_fn,
-                                               func.merge_fn,
-                                               func.serialize_fn,
-                                               func.finalize_fn,
-                                               name, database)
+            stmt = ddl.CreateUDA(func, name=name, database=database)
         else:
             raise TypeError(func)
         self._execute(stmt)
@@ -1340,12 +1576,9 @@ class ImpalaClient(SQLClient):
         return result
 
     def _get_udfs(self, cur, klass):
-        from ibis.expr.rules import varargs
-        from ibis.expr.datatypes import validate_type
-
         def _to_type(x):
             ibis_type = udf._impala_type_to_ibis(x.lower())
-            return validate_type(ibis_type)
+            return dt.dtype(ibis_type)
 
         tuples = cur.fetchall()
         if len(tuples) > 0:
@@ -1364,7 +1597,7 @@ class ImpalaClient(SQLClient):
                         inputs.append(t)
                     else:
                         t = _to_type(var)
-                        inputs = varargs(t)
+                        inputs = rlz.listof(t)
                         # TODO
                         # inputs.append(varargs(t))
                         break
@@ -1405,8 +1638,7 @@ class ImpalaClient(SQLClient):
         """
         return len(self.list_udas(database=database, like=name)) > 0
 
-    def compute_stats(self, name, database=None, incremental=False,
-                      async=False):
+    def compute_stats(self, name, database=None, incremental=False):
         """
         Issue COMPUTE STATS command for a given table
 
@@ -1418,10 +1650,6 @@ class ImpalaClient(SQLClient):
         incremental : boolean, default False
           If True, issue COMPUTE INCREMENTAL STATS
         """
-        # TODO async + cancellation
-        if async:
-            raise NotImplementedError
-
         maybe_inc = 'INCREMENTAL ' if incremental else ''
         cmd = 'COMPUTE {0}STATS'.format(maybe_inc)
 
@@ -1545,7 +1773,7 @@ class ImpalaClient(SQLClient):
                 adapted_types.append(typename)
         return names, adapted_types
 
-    def write_dataframe(self, df, path, format='csv', async=False):
+    def write_dataframe(self, df, path, format='csv'):
         """
         Write a pandas DataFrame to indicated file path (default: HDFS) in the
         indicated format
@@ -1556,8 +1784,6 @@ class ImpalaClient(SQLClient):
         path : string
           Absolute output path
         format : {'csv'}, default 'csv'
-        async : boolean, default False
-          Not yet supported
 
         Returns
         -------
@@ -1565,9 +1791,6 @@ class ImpalaClient(SQLClient):
         """
         from ibis.impala.pandas_interop import DataFrameWriter
 
-        if async:
-            raise NotImplementedError
-
         writer = DataFrameWriter(self, df)
         return writer.write_csv(path)
 
@@ -1588,361 +1811,6 @@ class AggregateFunction(DatabaseEntity):
         pass
 
 
-class ImpalaTable(ir.TableExpr, DatabaseEntity):
-
-    """
-    References a physical table in the Impala-Hive metastore
-    """
-
-    @property
-    def _qualified_name(self):
-        return self.op().args[0]
-
-    @property
-    def _unqualified_name(self):
-        return self._match_name()[1]
-
-    @property
-    def _client(self):
-        return self.op().args[2]
-
-    def _match_name(self):
-        m = ddl.fully_qualified_re.match(self._qualified_name)
-        if not m:
-            raise com.IbisError('Cannot determine database name from {0}'
-                                .format(self._qualified_name))
-        db, quoted, unquoted = m.groups()
-        return db, quoted or unquoted
-
-    @property
-    def _database(self):
-        return self._match_name()[0]
-
-    def compute_stats(self, incremental=False, async=False):
-        """
-        Invoke Impala COMPUTE STATS command to compute column, table, and
-        partition statistics.
-
-        See also ImpalaClient.compute_stats
-        """
-        return self._client.compute_stats(self._qualified_name,
-                                          incremental=incremental,
-                                          async=async)
-
-    def invalidate_metadata(self):
-        self._client.invalidate_metadata(self._qualified_name)
-
-    def refresh(self):
-        self._client.refresh(self._qualified_name)
-
-    def metadata(self):
-        """
-        Return parsed results of DESCRIBE FORMATTED statement
-
-        Returns
-        -------
-        meta : TableMetadata
-        """
-        return self._client.describe_formatted(self._qualified_name)
-
-    describe_formatted = metadata
-
-    def files(self):
-        """
-        Return results of SHOW FILES statement
-        """
-        return self._client.show_files(self._qualified_name)
-
-    def drop(self):
-        """
-        Drop the table from the database
-        """
-        self._client.drop_table_or_view(self._qualified_name)
-
-    def truncate(self):
-        self._client.truncate_table(self._qualified_name)
-
-    def insert(self, obj=None, overwrite=False, partition=None,
-               values=None, validate=True):
-        """
-        Insert into Impala table. Wraps ImpalaClient.insert
-
-        Parameters
-        ----------
-        obj : TableExpr or pandas DataFrame
-        overwrite : boolean, default False
-          If True, will replace existing contents of table
-        partition : list or dict, optional
-          For partitioned tables, indicate the partition that's being inserted
-          into, either with an ordered list of partition keys or a dict of
-          partition field name to value. For example for the partition
-          (year=2007, month=7), this can be either (2007, 7) or {'year': 2007,
-          'month': 7}.
-        validate : boolean, default True
-          If True, do more rigorous validation that schema of table being
-          inserted is compatible with the existing table
-
-        Examples
-        --------
-        >>> t.insert(table_expr)  # doctest: +SKIP
-
-        # Completely overwrite contents
-        >>> t.insert(table_expr, overwrite=True)  # doctest: +SKIP
-        """
-        if isinstance(obj, pd.DataFrame):
-            from ibis.impala.pandas_interop import write_temp_dataframe
-            writer, expr = write_temp_dataframe(self._client, obj)
-        else:
-            expr = obj
-
-        if values is not None:
-            raise NotImplementedError
-
-        if validate:
-            existing_schema = self.schema()
-            insert_schema = expr.schema()
-            if not insert_schema.equals(existing_schema):
-                _validate_compatible(insert_schema, existing_schema)
-
-        if partition is not None:
-            partition_schema = self.partition_schema()
-            partition_schema_names = frozenset(partition_schema.names)
-            expr = expr.projection([
-                column for column in expr.columns
-                if column not in partition_schema_names
-            ])
-        else:
-            partition_schema = None
-
-        ast = build_ast(expr, ImpalaDialect.make_context())
-        select = ast.queries[0]
-        statement = ddl.InsertSelect(self._qualified_name,
-                                     select,
-                                     partition=partition,
-                                     partition_schema=partition_schema,
-                                     overwrite=overwrite)
-        return self._execute(statement)
-
-    def load_data(self, path, overwrite=False, partition=None):
-        """
-        Wraps the LOAD DATA DDL statement. Loads data into an Impala table by
-        physically moving data files.
-
-        Parameters
-        ----------
-        path : string
-        overwrite : boolean, default False
-          Overwrite the existing data in the entire table or indicated
-          partition
-        partition : dict, optional
-          If specified, the partition must already exist
-
-        Returns
-        -------
-        query : ImpalaQuery
-        """
-        if partition is not None:
-            partition_schema = self.partition_schema()
-        else:
-            partition_schema = None
-
-        stmt = ddl.LoadData(self._qualified_name, path,
-                            partition=partition,
-                            partition_schema=partition_schema)
-
-        return self._execute(stmt)
-
-    @property
-    def name(self):
-        return self.op().name
-
-    def rename(self, new_name, database=None):
-        """
-        Rename table inside Impala. References to the old table are no longer
-        valid.
-
-        Parameters
-        ----------
-        new_name : string
-        database : string
-
-        Returns
-        -------
-        renamed : ImpalaTable
-        """
-        m = ddl.fully_qualified_re.match(new_name)
-        if not m and database is None:
-            database = self._database
-        statement = ddl.RenameTable(self._qualified_name, new_name,
-                                    new_database=database)
-        self._client._execute(statement)
-
-        op = self.op().change_name(statement.new_qualified_name)
-        return ImpalaTable(op)
-
-    def _execute(self, stmt):
-        return self._client._execute(stmt)
-
-    @property
-    def is_partitioned(self):
-        """
-        True if the table is partitioned
-        """
-        return self.metadata().is_partitioned
-
-    def partition_schema(self):
-        """
-        For partitioned tables, return the schema (names and types) for the
-        partition columns
-
-        Returns
-        -------
-        partition_schema : ibis Schema
-        """
-        schema = self.schema()
-        name_to_type = dict(zip(schema.names, schema.types))
-
-        result = self.partitions()
-
-        partition_fields = []
-        for x in result.columns:
-            if x not in name_to_type:
-                break
-            partition_fields.append((x, name_to_type[x]))
-
-        pnames, ptypes = zip(*partition_fields)
-        return sch.Schema(pnames, ptypes)
-
-    def add_partition(self, spec, location=None):
-        """
-        Add a new table partition, creating any new directories in HDFS if
-        necessary.
-
-        Partition parameters can be set in a single DDL statement, or you can
-        use alter_partition to set them after the fact.
-
-        Returns
-        -------
-        None (for now)
-        """
-        part_schema = self.partition_schema()
-        stmt = ddl.AddPartition(self._qualified_name, spec, part_schema,
-                                location=location)
-        return self._execute(stmt)
-
-    def alter(self, location=None, format=None, tbl_properties=None,
-              serde_properties=None):
-        """
-        Change setting and parameters of the table.
-
-        Parameters
-        ----------
-        location : string, optional
-          For partitioned tables, you may want the alter_partition function
-        format : string, optional
-        tbl_properties : dict, optional
-        serde_properties : dict, optional
-
-        Returns
-        -------
-        None (for now)
-        """
-        def _run_ddl(**kwds):
-            stmt = ddl.AlterTable(self._qualified_name, **kwds)
-            return self._execute(stmt)
-
-        return self._alter_table_helper(_run_ddl, location=location,
-                                        format=format,
-                                        tbl_properties=tbl_properties,
-                                        serde_properties=serde_properties)
-
-    def set_external(self, is_external=True):
-        """
-        Toggle EXTERNAL table property.
-        """
-        self.alter(tbl_properties={'EXTERNAL': is_external})
-
-    def alter_partition(self, spec, location=None, format=None,
-                        tbl_properties=None,
-                        serde_properties=None):
-        """
-        Change setting and parameters of an existing partition
-
-        Parameters
-        ----------
-        spec : dict or list
-          The partition keys for the partition being modified
-        location : string, optional
-        format : string, optional
-        tbl_properties : dict, optional
-        serde_properties : dict, optional
-
-        Returns
-        -------
-        None (for now)
-        """
-        part_schema = self.partition_schema()
-
-        def _run_ddl(**kwds):
-            stmt = ddl.AlterPartition(self._qualified_name, spec,
-                                      part_schema, **kwds)
-            return self._execute(stmt)
-
-        return self._alter_table_helper(_run_ddl, location=location,
-                                        format=format,
-                                        tbl_properties=tbl_properties,
-                                        serde_properties=serde_properties)
-
-    def _alter_table_helper(self, f, **alterations):
-        results = []
-        for k, v in alterations.items():
-            if v is None:
-                continue
-            result = f(**{k: v})
-            results.append(result)
-        return results
-
-    def drop_partition(self, spec):
-        """
-        Drop an existing table partition
-        """
-        part_schema = self.partition_schema()
-        stmt = ddl.DropPartition(self._qualified_name, spec, part_schema)
-        return self._execute(stmt)
-
-    def partitions(self):
-        """
-        Return a pandas.DataFrame giving information about this table's
-        partitions. Raises an exception if the table is not partitioned.
-
-        Returns
-        -------
-        partitions : pandas.DataFrame
-        """
-        return self._client.list_partitions(self._qualified_name)
-
-    def stats(self):
-        """
-        Return results of SHOW TABLE STATS as a DataFrame. If not partitioned,
-        contains only one row
-
-        Returns
-        -------
-        stats : pandas.DataFrame
-        """
-        return self._client.table_stats(self._qualified_name)
-
-    def column_stats(self):
-        """
-        Return results of SHOW COLUMN STATS as a pandas DataFrame
-
-        Returns
-        -------
-        column_stats : pandas.DataFrame
-        """
-        return self._client.column_stats(self._qualified_name)
-
-
 class ImpalaTemporaryTable(ops.DatabaseTable):
 
     def __del__(self):
