@@ -1,19 +1,6 @@
-# Copyright 2014 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 from six import StringIO
 import datetime
+from operator import add, mul, sub
 
 import ibis
 import ibis.expr.analysis as L
@@ -139,6 +126,11 @@ def _cast(translator, expr):
 
     if isinstance(arg, ir.CategoryValue) and target_type == 'int32':
         return arg_formatted
+    if (
+        isinstance(arg, (ir.TimeValue, ir.DateValue, ir.TimestampValue)) and
+            target_type == 'int64'
+            ):
+        return '1000000 * unix_timestamp({})'.format(arg_formatted)
     else:
         sql_type = _type_to_sql_string(target_type)
         return 'CAST({} AS {})'.format(arg_formatted, sql_type)
@@ -147,8 +139,11 @@ def _cast(translator, expr):
 def _type_to_sql_string(tval):
     if isinstance(tval, dt.Decimal):
         return 'decimal({}, {})'.format(tval.precision, tval.scale)
-    else:
-        return _sql_type_names[tval.name.lower()]
+    name = tval.name.lower()
+    try:
+        return _sql_type_names[name]
+    except KeyError:
+        raise com.UnsupportedBackendType(name)
 
 
 def _between(translator, expr):
@@ -195,6 +190,99 @@ def _cumulative_to_window(translator, expr, window):
     return new_expr
 
 
+_map_interval_to_microseconds = dict(
+    W=604800000000,
+    D=86400000000,
+    h=3600000000,
+    m=60000000,
+    s=1000000,
+    ms=1000,
+    us=1,
+    ns=.001
+)
+
+
+_map_interval_op_to_op = {
+    # Literal Intervals have two args, i.e.
+    # Literal(1, Interval(value_type=int8, unit='D', nullable=True))
+    # Parse both args and multipy 1 * _map_interval_to_microseconds['D']
+    ops.Literal: mul,
+    ops.IntervalMultiply: mul,
+    ops.IntervalAdd: add,
+    ops.IntervalSubtract: sub,
+}
+
+
+def _replace_interval_with_scalar(expr):
+    """
+    Good old Depth-First Search to identify the Interval and IntervalValue
+    components of the expression and return a comparable scalar expression.
+
+    Parameters
+    ----------
+    expr : float or expression of intervals, i.e.
+      1 * ibis.day() + 5 * ibis.hour()
+
+    Returns
+    -------
+    preceding : float or ir.FloatingScalar, depending upon the expr
+    """
+    try:
+        expr_op = expr.op()
+    except AttributeError:
+        expr_op = None
+
+    if not isinstance(expr, (dt.Interval, ir.IntervalValue)):
+        # Literal expressions have op method but native types do not.
+        if isinstance(expr_op, ops.Literal):
+            return expr_op.value
+        else:
+            return expr
+    elif isinstance(expr, dt.Interval):
+        try:
+            microseconds = _map_interval_to_microseconds[expr.unit]
+            return microseconds
+        except KeyError:
+            raise ValueError(
+                "Expected preceding values of week(), " +
+                "day(), hour(), minute(), second(), millisecond(), " +
+                "microseconds(), nanoseconds(); got {}".format(expr)
+                )
+    elif expr_op.args and isinstance(expr, ir.IntervalValue):
+        if len(expr_op.args) > 2:
+            raise com.NotImplementedError(
+                "'preceding' argument cannot be parsed."
+            )
+        left_arg = _replace_interval_with_scalar(expr_op.args[0])
+        right_arg = _replace_interval_with_scalar(expr_op.args[1])
+        method = _map_interval_op_to_op[type(expr_op)]
+        return method(left_arg, right_arg)
+
+
+def _time_range_to_range_window(translator, window):
+    # Check that ORDER BY column is a single time column:
+    order_by_vars = [x.op().args[0] for x in window._order_by]
+    if len(order_by_vars) > 1:
+        raise com.IbisInputError(
+            "Expected 1 order-by variable, got {}".format(
+                len(order_by_vars)
+            )
+        )
+
+    order_var = window._order_by[0].op().args[0]
+    timestamp_order_var = order_var.cast('int64')
+    window = window._replace(order_by=timestamp_order_var,
+                             how='range')
+
+    # Need to change preceding interval expression to scalars
+    preceding = window.preceding
+    if isinstance(preceding, ir.IntervalScalar):
+        new_preceding = _replace_interval_with_scalar(preceding)
+        window = window._replace(preceding=new_preceding)
+
+    return window
+
+
 def _window(translator, expr):
     op = expr.op()
 
@@ -231,6 +319,13 @@ def _window(translator, expr):
             len(window._order_by) == 0):
         window = window.order_by(window_op.args[0])
 
+    # Time ranges need to be converted to microseconds.
+    if window.how == 'range':
+        order_by_types = [type(x.op().args[0]) for x in window._order_by]
+        time_range_types = (ir.TimeColumn, ir.DateColumn, ir.TimestampColumn)
+        if any(col_type in time_range_types for col_type in order_by_types):
+            window = _time_range_to_range_window(translator, window)
+
     window_formatted = _format_window(translator, window)
 
     arg_formatted = translator.translate(arg)
@@ -270,11 +365,14 @@ def _format_window(translator, window):
         return '{} FOLLOWING'.format(f) if f > 0 else 'CURRENT ROW'
 
     if p is not None and f is not None:
-        frame = 'ROWS BETWEEN {} AND {}'.format(_prec(p), _foll(f))
+        frame = '{} BETWEEN {} AND {}'.format(
+            window.how.upper(), _prec(p), _foll(f))
+
     elif p is not None:
         if isinstance(p, tuple):
             start, end = p
-            frame = 'ROWS BETWEEN {} AND {}'.format(_prec(start), _prec(end))
+            frame = '{} BETWEEN {} AND {}'.format(
+                window.how.upper(), _prec(start), _prec(end))
         else:
             kind = 'ROWS' if p > 0 else 'RANGE'
             frame = '{} BETWEEN {} AND UNBOUNDED FOLLOWING'.format(
@@ -283,7 +381,8 @@ def _format_window(translator, window):
     elif f is not None:
         if isinstance(f, tuple):
             start, end = f
-            frame = 'ROWS BETWEEN {} AND {}'.format(_foll(start), _foll(end))
+            frame = '{} BETWEEN {} AND {}'.format(
+                window.how.upper(), _foll(start), _foll(end))
         else:
             kind = 'ROWS' if f > 0 else 'RANGE'
             frame = '{} BETWEEN UNBOUNDED PRECEDING AND {}'.format(
@@ -482,17 +581,26 @@ def _needs_parens(op):
             op_klass in {ops.Negate, ops.IsNull, ops.NotNull})
 
 
-def _boolean_literal_format(expr):
+def _set_literal_format(translator, expr):
+    value_type = expr.type().value_type
+
+    formatted = [translator.translate(ir.literal(x, type=value_type))
+                 for x in expr.op().value]
+
+    return _parenthesize(', '.join(formatted))
+
+
+def _boolean_literal_format(translator, expr):
     value = expr.op().value
     return 'TRUE' if value else 'FALSE'
 
 
-def _string_literal_format(expr):
+def _string_literal_format(translator, expr):
     value = expr.op().value
     return "'{}'".format(value.replace("'", "\\'"))
 
 
-def _number_literal_format(expr):
+def _number_literal_format(translator, expr):
     value = expr.op().value
     formatted = repr(value)
 
@@ -502,8 +610,9 @@ def _number_literal_format(expr):
     return formatted
 
 
-def _interval_literal_format(expr):
-    return 'INTERVAL {} {}'.format(expr.op().value, expr.resolution.upper())
+def _interval_literal_format(translator, expr):
+    return 'INTERVAL {} {}'.format(expr.op().value,
+                                   expr.type().resolution.upper())
 
 
 def _interval_from_integer(translator, expr):
@@ -512,10 +621,11 @@ def _interval_from_integer(translator, expr):
     arg, unit = op.args
     arg_formatted = translator.translate(arg)
 
-    return 'INTERVAL {} {}'.format(arg_formatted, expr.resolution.upper())
+    return 'INTERVAL {} {}'.format(arg_formatted,
+                                   expr.type().resolution.upper())
 
 
-def _date_literal_format(expr):
+def _date_literal_format(translator, expr):
     value = expr.op().value
     if isinstance(value, datetime.date):
         value = value.strftime('%Y-%m-%d')
@@ -523,7 +633,7 @@ def _date_literal_format(expr):
     return repr(value)
 
 
-def _timestamp_literal_format(expr):
+def _timestamp_literal_format(translator, expr):
     value = expr.op().value
     if isinstance(value, datetime.datetime):
         value = value.strftime('%Y-%m-%d %H:%M:%S')
@@ -698,7 +808,7 @@ def _extract_field(sql_attr):
 
 def _day_of_week_index(t, expr):
     arg, = expr.op().args
-    return 'dayofweek({})'.format(t.translate(arg))
+    return 'pmod(dayofweek({}) - 2, 7)'.format(t.translate(arg))
 
 
 def _day_of_week_name(t, expr):
@@ -736,11 +846,7 @@ def _timestamp_from_unix(translator, expr):
     op = expr.op()
 
     val, unit = op.args
-
-    if unit == 'ms':
-        val = (val / 1000).cast('int32')
-    elif unit == 'us':
-        val = (val / 1000000).cast('int32')
+    val = util.convert_unit(val, unit, 's').cast('int32')
 
     arg = _from_unixtime(translator, val)
     return 'CAST({} AS timestamp)'.format(arg)
@@ -754,7 +860,7 @@ def _from_unixtime(translator, expr):
 def varargs(func_name):
     def varargs_formatter(translator, expr):
         op = expr.op()
-        return _format_call(translator, func_name, *op.args)
+        return _format_call(translator, func_name, *op.arg)
     return varargs_formatter
 
 
@@ -765,7 +871,7 @@ def _substring(translator, expr):
     start_formatted = translator.translate(start)
 
     # Impala is 1-indexed
-    if length is None or isinstance(length.op(), ir.Literal):
+    if length is None or isinstance(length.op(), ops.Literal):
         lvalue = length.op().value if length is not None else None
         if lvalue:
             return 'substr({}, {} + 1, {})'.format(
@@ -786,7 +892,7 @@ def _string_find(translator, expr):
     arg_formatted = translator.translate(arg)
     substr_formatted = translator.translate(substr)
 
-    if start is not None and not isinstance(start.op(), ir.Literal):
+    if start is not None and not isinstance(start.op(), ops.Literal):
         start_fmt = translator.translate(start)
         return 'locate({}, {}, {} + 1) - 1'.format(
             substr_formatted, arg_formatted, start_fmt
@@ -889,10 +995,12 @@ def _literal(translator, expr):
         typeclass = 'timestamp'
     elif isinstance(expr, ir.IntervalValue):
         typeclass = 'interval'
+    elif isinstance(expr, ir.SetValue):
+        typeclass = 'set'
     else:
         raise NotImplementedError
 
-    return _literal_formatters[typeclass](expr)
+    return _literal_formatters[typeclass](translator, expr)
 
 
 def _null_literal(translator, expr):
@@ -905,7 +1013,8 @@ _literal_formatters = {
     'string': _string_literal_format,
     'interval': _interval_literal_format,
     'timestamp': _timestamp_literal_format,
-    'date': _date_literal_format
+    'date': _date_literal_format,
+    'set': _set_literal_format
 }
 
 
@@ -976,6 +1085,15 @@ def _string_like(translator, expr):
     )
 
 
+def _sign(translator, expr):
+    arg, = expr.op().args
+    translated_arg = translator.translate(arg)
+    translated_type = _type_to_sql_string(expr.type())
+    if expr.type() != dt.float:
+        return 'CAST(sign({}) AS {})'.format(translated_arg, translated_type)
+    return 'sign({})'.format(translated_arg)
+
+
 _operation_registry = {
     # Unary operations
     ops.NotNull: _not_null,
@@ -999,7 +1117,7 @@ _operation_registry = {
     ops.Exp: unary('exp'),
     ops.Round: _round,
 
-    ops.Sign: unary('sign'),
+    ops.Sign: _sign,
     ops.Sqrt: unary('sqrt'),
 
     ops.Hash: _hash,
@@ -1070,10 +1188,10 @@ _operation_registry = {
     # Other operations
     ops.E: lambda *args: 'e()',
 
-    ir.Literal: _literal,
-    ir.NullLiteral: _null_literal,
+    ops.Literal: _literal,
+    ops.NullLiteral: _null_literal,
 
-    ir.ValueList: _value_list,
+    ops.ValueList: _value_list,
 
     ops.Cast: _cast,
 
