@@ -1,41 +1,31 @@
-# Copyright 2015 Cloudera Inc
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import unittest
+
 from posixpath import join as pjoin
+
 import pytest
+
+import numpy as np
 import pandas as pd
 
 import ibis
-
 import ibis.expr.types as ir
-
-from ibis.common import IbisTypeError
-from ibis.compat import Decimal
-from ibis.expr import datatypes as dt
-from ibis.expr.tests.mocks import MockConnection
-
+import ibis.expr.datatypes as dt
 import ibis.expr.rules as rules
 import ibis.common as com
 import ibis.util as util
 
+from ibis.common import IbisTypeError
+from ibis.compat import Decimal
+from ibis.expr.tests.mocks import MockConnection
+
 pytest.importorskip('hdfs')
 pytest.importorskip('sqlalchemy')
 pytest.importorskip('impala.dbapi')
 
+pytestmark = [pytest.mark.impala, pytest.mark.udf]
+
 from ibis.impala import ddl  # noqa: E402
 import ibis.impala as api  # noqa: E402
-from ibis.impala.tests.common import ImpalaE2E  # noqa: E402
 
 
 class TestWrapping(unittest.TestCase):
@@ -175,245 +165,269 @@ class TestWrapping(unittest.TestCase):
         return func
 
 
-class TestUDFE2E(ImpalaE2E, unittest.TestCase):
+@pytest.fixture(scope='session')
+def udfcon(con):
+    con.disable_codegen(False)
+    try:
+        yield con
+    finally:
+        con.disable_codegen(True)
 
-    def setUp(self):
-        super(TestUDFE2E, self).setUp()
-        self.udf_ll = pjoin(self.test_data_dir, 'udf/udf-sample.ll')
-        self.uda_ll = pjoin(self.test_data_dir, 'udf/uda-sample.ll')
-        self.uda_so = pjoin(self.test_data_dir, 'udf/libudasample.so')
-        self.con.disable_codegen(False)
 
-    def tearDown(self):
-        super(TestUDFE2E, self).tearDown()
-        self.con.disable_codegen(True)
+@pytest.fixture(scope='session')
+def alltypes(udfcon):
+    return udfcon.table('functional_alltypes')
 
-    @pytest.mark.udf
-    def test_identity_primitive_types(self):
-        cases = [
-            ('boolean', True, self.alltypes.bool_col),
-            ('int8', 5, self.alltypes.tinyint_col),
-            ('int16', 2**10, self.alltypes.smallint_col),
-            ('int32', 2**17, self.alltypes.int_col),
-            ('int64', 2**33, self.alltypes.bigint_col),
-            ('float', 3.14, self.alltypes.float_col),
-            ('double', 3.14, self.alltypes.double_col),
-            ('string', 'ibis', self.alltypes.string_col),
-            ('timestamp', ibis.timestamp('1961-04-10'),
-             self.alltypes.timestamp_col),
-        ]
 
-        for t, lit_val, array_val in cases:
-            if not isinstance(lit_val, ir.Expr):
-                lit_val = ibis.literal(lit_val)
-            self._identity_func_testing(t, lit_val, array_val)
+@pytest.fixture(scope='session')
+def udf_ll(udfcon, test_data_dir):
+    return pjoin(test_data_dir, 'udf/udf-sample.ll')
 
-    @pytest.mark.udf
-    def test_decimal(self):
-        col = self.con.table('tpch_customer').c_acctbal
-        literal = ibis.literal(1).cast('decimal(12,2)')
-        name = '__tmp_udf_' + util.guid()
-        func = self._udf_creation_to_op(name, 'Identity',
-                                        ['decimal(12,2)'],
-                                        'decimal(12,2)')
 
-        expr = func(literal)
-        assert issubclass(type(expr), ir.ScalarExpr)
-        result = self.con.execute(expr)
-        assert result == Decimal(1)
+@pytest.fixture(scope='session')
+def uda_ll(udfcon, test_data_dir):
+    return pjoin(test_data_dir, 'udf/uda-sample.ll')
 
-        expr = func(col)
-        assert issubclass(type(expr), ir.ColumnExpr)
-        self.con.execute(expr)
 
-    @pytest.mark.udf
-    def test_mixed_inputs(self):
-        name = 'two_args'
-        symbol = 'TwoArgs'
-        inputs = ['int32', 'int32']
-        output = 'int32'
-        func = self._udf_creation_to_op(name, symbol, inputs, output)
+@pytest.fixture(scope='session')
+def uda_so(udfcon, test_data_dir):
+    return pjoin(test_data_dir, 'udf/libudasample.so')
 
-        expr = func(self.alltypes.int_col, 1)
-        assert issubclass(type(expr), ir.ColumnExpr)
-        self.con.execute(expr)
 
-        expr = func(1, self.alltypes.int_col)
-        assert issubclass(type(expr), ir.ColumnExpr)
-        self.con.execute(expr)
+@pytest.mark.parametrize(
+    ('typ', 'lit_val', 'col_name'),
+    [
+        ('boolean', True, 'bool_col'),
+        ('int8', ibis.literal(5), 'tinyint_col'),
+        ('int16', ibis.literal(2**10), 'smallint_col'),
+        ('int32', ibis.literal(2**17), 'int_col'),
+        ('int64', ibis.literal(2**33), 'bigint_col'),
+        ('float', ibis.literal(3.14), 'float_col'),
+        ('double', ibis.literal(3.14), 'double_col'),
+        ('string', ibis.literal('ibis'), 'string_col'),
+        ('timestamp', ibis.timestamp('1961-04-10'), 'timestamp_col'),
+    ]
+)
+def test_identity_primitive_types(
+    udfcon, alltypes, test_data_db, udf_ll, typ, lit_val, col_name
+):
+    col_val = alltypes[col_name]
+    identity_func_testing(udf_ll, udfcon, test_data_db, typ, lit_val, col_val)
 
-        expr = func(self.alltypes.int_col, self.alltypes.tinyint_col)
-        self.con.execute(expr)
 
-    @pytest.mark.udf
-    def test_implicit_typecasting(self):
-        col = self.alltypes.tinyint_col
-        literal = ibis.literal(1000)
-        self._identity_func_testing('int32', literal, col)
+def test_decimal(udfcon, test_data_db, udf_ll):
+    col = udfcon.table('tpch_customer').c_acctbal
+    literal = ibis.literal(1).cast('decimal(12,2)')
+    name = '__tmp_udf_' + util.guid()
 
-    def _identity_func_testing(self, datatype, literal, column):
-        inputs = [datatype]
-        name = '__tmp_udf_' + util.guid()
-        func = self._udf_creation_to_op(name, 'Identity', inputs, datatype)
+    func = udf_creation_to_op(
+        udf_ll, udfcon, test_data_db, name, 'Identity',
+        ['decimal(12,2)'], 'decimal(12,2)')
 
-        expr = func(literal)
-        assert issubclass(type(expr), ir.ScalarExpr)
-        result = self.con.execute(expr)
-        # Hacky
-        if datatype is 'timestamp':
-            assert type(result) == pd.Timestamp
+    expr = func(literal)
+    assert issubclass(type(expr), ir.ScalarExpr)
+    result = udfcon.execute(expr)
+    assert result == Decimal(1)
+
+    expr = func(col)
+    assert issubclass(type(expr), ir.ColumnExpr)
+    udfcon.execute(expr)
+
+
+def test_mixed_inputs(udfcon, alltypes, test_data_db, udf_ll):
+    name = 'two_args'
+    symbol = 'TwoArgs'
+    inputs = ['int32', 'int32']
+    output = 'int32'
+    func = udf_creation_to_op(
+        udf_ll, udfcon, test_data_db, name, symbol, inputs, output)
+
+    expr = func(alltypes.int_col, 1)
+    assert issubclass(type(expr), ir.ColumnExpr)
+    udfcon.execute(expr)
+
+    expr = func(1, alltypes.int_col)
+    assert issubclass(type(expr), ir.ColumnExpr)
+    udfcon.execute(expr)
+
+    expr = func(alltypes.int_col, alltypes.tinyint_col)
+    udfcon.execute(expr)
+
+
+def test_implicit_typecasting(udfcon, alltypes, test_data_db, udf_ll):
+    col = alltypes.tinyint_col
+    literal = ibis.literal(1000)
+    identity_func_testing(udf_ll, udfcon, test_data_db, 'int32', literal, col)
+
+
+def identity_func_testing(
+    udf_ll, udfcon, test_data_db, datatype, literal, column
+):
+    inputs = [datatype]
+    name = '__tmp_udf_' + util.guid()
+    func = udf_creation_to_op(
+        udf_ll, udfcon, test_data_db, name, 'Identity', inputs, datatype)
+
+    expr = func(literal)
+    assert issubclass(type(expr), ir.ScalarExpr)
+    result = udfcon.execute(expr)
+    # Hacky
+    if datatype is 'timestamp':
+        assert type(result) == pd.Timestamp
+    else:
+        lop = literal.op()
+        if isinstance(lop, ir.Literal):
+            np.testing.assert_allclose(lop.value, 5)
         else:
-            lop = literal.op()
-            if isinstance(lop, ir.Literal):
-                self.assertAlmostEqual(result, lop.value, 5)
-            else:
-                self.assertAlmostEqual(result, self.con.execute(literal), 5)
+            np.testing.assert_allclose(result, udfcon.execute(literal), 5)
 
-        expr = func(column)
-        assert issubclass(type(expr), ir.ColumnExpr)
-        self.con.execute(expr)
+    expr = func(column)
+    assert issubclass(type(expr), ir.ColumnExpr)
+    udfcon.execute(expr)
 
-    @pytest.mark.udf
-    def test_mult_type_args(self):
-        symbol = 'AlmostAllTypes'
-        name = 'most_types'
-        inputs = ['string', 'boolean', 'int8', 'int16', 'int32',
-                  'int64', 'float', 'double']
-        output = 'int32'
 
-        func = self._udf_creation_to_op(name, symbol, inputs, output)
+def test_mult_type_args(udfcon, alltypes, test_data_db, udf_ll):
+    symbol = 'AlmostAllTypes'
+    name = 'most_types'
+    inputs = ['string', 'boolean', 'int8', 'int16', 'int32',
+              'int64', 'float', 'double']
+    output = 'int32'
 
-        expr = func('a', True, 1, 1, 1, 1, 1.0, 1.0)
-        result = self.con.execute(expr)
-        assert result == 8
+    func = udf_creation_to_op(
+        udf_ll, udfcon, test_data_db, name, symbol, inputs, output)
 
-        table = self.alltypes
-        expr = func(table.string_col, table.bool_col, table.tinyint_col,
-                    table.tinyint_col, table.smallint_col,
-                    table.smallint_col, 1.0, 1.0)
-        self.con.execute(expr)
+    expr = func('a', True, 1, 1, 1, 1, 1.0, 1.0)
+    result = udfcon.execute(expr)
+    assert result == 8
 
-    def test_all_type_args(self):
-        pytest.skip('failing test, to be fixed later')
+    table = alltypes
+    expr = func(table.string_col, table.bool_col, table.tinyint_col,
+                table.tinyint_col, table.smallint_col,
+                table.smallint_col, 1.0, 1.0)
+    udfcon.execute(expr)
 
-        symbol = 'AllTypes'
-        name = 'all_types'
-        inputs = ['string', 'boolean', 'int8', 'int16', 'int32',
-                  'int64', 'float', 'double', 'decimal']
-        output = 'int32'
 
-        func = self._udf_creation_to_op(name, symbol, inputs, output)
-        expr = func('a', True, 1, 1, 1, 1, 1.0, 1.0, 1.0)
-        result = self.con.execute(expr)
-        assert result == 9
+def test_all_type_args(udfcon, test_data_db, udf_ll):
+    pytest.skip('failing test, to be fixed later')
 
-    @pytest.mark.udf
-    def test_udf_varargs(self):
-        t = self.alltypes
+    symbol = 'AllTypes'
+    name = 'all_types'
+    inputs = ['string', 'boolean', 'int8', 'int16', 'int32',
+              'int64', 'float', 'double', 'decimal']
+    output = 'int32'
 
-        name = 'add_numbers_{0}'.format(util.guid()[:4])
+    func = udf_creation_to_op(
+        udf_ll, udfcon, test_data_db, name, symbol, inputs, output)
+    expr = func('a', True, 1, 1, 1, 1, 1.0, 1.0, 1.0)
+    result = udfcon.execute(expr)
+    assert result == 9
 
-        input_sig = rules.varargs(rules.double)
-        func = api.wrap_udf(self.udf_ll, input_sig, 'double', 'AddNumbers',
-                            name=name)
-        func.register(name, self.test_data_db)
-        self.con.create_function(func, database=self.test_data_db)
 
-        expr = func(t.double_col, t.double_col)
-        expr.execute()
+def test_udf_varargs(udfcon, alltypes, udf_ll, test_data_db):
+    t = alltypes
 
-    def test_drop_udf_not_exists(self):
-        random_name = util.guid()
-        self.assertRaises(Exception, self.con.drop_udf, random_name)
+    name = 'add_numbers_{0}'.format(util.guid()[:4])
 
-    def test_drop_uda_not_exists(self):
-        random_name = util.guid()
-        self.assertRaises(Exception, self.con.drop_uda, random_name)
+    input_sig = rules.varargs(rules.double)
+    func = api.wrap_udf(udf_ll, input_sig, 'double', 'AddNumbers',
+                        name=name)
+    func.register(name, test_data_db)
+    udfcon.create_function(func, database=test_data_db)
 
-    def _udf_creation_to_op(self, name, symbol, inputs, output):
-        func = api.wrap_udf(self.udf_ll, inputs, output, symbol, name)
+    expr = func(t.double_col, t.double_col)
+    expr.execute()
 
-        self.temp_udfs.append((name, inputs))
 
-        self.con.create_function(func, database=self.test_data_db)
+def test_drop_udf_not_exists(udfcon):
+    random_name = util.guid()
+    with pytest.raises(Exception):
+        udfcon.drop_udf(random_name)
 
-        func.register(name, self.test_data_db)
 
-        assert self.con.exists_udf(name, self.test_data_db)
-        return func
+def test_drop_uda_not_exists(udfcon):
+    random_name = util.guid()
+    with pytest.raises(Exception):
+        udfcon.drop_uda(random_name)
 
-    def test_ll_uda_not_supported(self):
-        # LLVM IR UDAs are not supported as of Impala 2.2
-        with self.assertRaises(com.IbisError):
-            self._conforming_wrapper(self.uda_ll, ['double'], 'double',
-                                     'Variance')
 
-    def _conforming_wrapper(self, where, inputs, output, prefix,
-                            serialize=True, name=None):
-        kwds = {
-            'name': name
-        }
-        if serialize:
-            kwds['serialize_fn'] = '{0}Serialize'.format(prefix)
-        return api.wrap_uda(where, inputs, output, '{0}Update'.format(prefix),
-                            init_fn='{0}Init'.format(prefix),
-                            merge_fn='{0}Merge'.format(prefix),
-                            finalize_fn='{0}Finalize'.format(prefix),
-                            **kwds)
+def udf_creation_to_op(
+    udf_ll, udfcon, test_data_db, name, symbol, inputs, output
+):
+    func = api.wrap_udf(udf_ll, inputs, output, symbol, name)
 
-    @pytest.mark.udf
-    def test_count_uda(self):
-        func = self._wrap_count_uda()
-        func.register(func.name, self.test_data_db)
-        self.con.create_function(func, database=self.test_data_db)
+    # self.temp_udfs.append((name, inputs))
 
-        # it works!
-        func(self.alltypes.int_col).execute()
-        self.temp_udas.append((func.name, ['int32']))
+    udfcon.create_function(func, database=test_data_db)
 
-    @pytest.mark.udf
-    def test_list_udas(self):
-        db = '__ibis_tmp_{0}'.format(util.guid())
-        self.con.create_database(db)
-        self.temp_databases.append(db)
+    func.register(name, test_data_db)
 
-        func = self._wrap_count_uda()
-        self.con.create_function(func, database=db)
+    assert udfcon.exists_udf(name, test_data_db)
+    return func
 
-        funcs = self.con.list_udas(database=db)
 
-        f = funcs[0]
-        assert f.name == func.name
-        assert f.inputs == func.inputs
-        assert f.output == func.output
+def test_ll_uda_not_supported(uda_ll):
+    # LLVM IR UDAs are not supported as of Impala 2.2
+    with pytest.raises(com.IbisError):
+        conforming_wrapper(uda_ll, ['double'], 'double', 'Variance')
 
-    @pytest.mark.udf
-    def test_drop_database_with_udfs_and_udas(self):
-        uda1 = self._wrap_count_uda()
-        uda2 = self._wrap_count_uda()
 
-        udf1 = api.wrap_udf(self.udf_ll, ['boolean'], 'boolean', 'Identity',
-                            'udf_{0}'.format(util.guid()))
+def conforming_wrapper(
+    where, inputs, output, prefix, serialize=True, name=None
+):
+    kwds = {'name': name}
+    if serialize:
+        kwds['serialize_fn'] = '{0}Serialize'.format(prefix)
+    return api.wrap_uda(where, inputs, output, '{0}Update'.format(prefix),
+                        init_fn='{0}Init'.format(prefix),
+                        merge_fn='{0}Merge'.format(prefix),
+                        finalize_fn='{0}Finalize'.format(prefix),
+                        **kwds)
 
-        db = '__ibis_tmp_{0}'.format(util.guid())
 
-        self.con.create_database(db)
+@pytest.fixture
+def wrapped_count_uda(uda_so):
+    name = 'user_count_{0}'.format(util.guid())
+    return api.wrap_uda(uda_so, ['int32'], 'int64', 'CountUpdate', name=name)
 
-        self.con.create_function(uda1, database=db)
-        self.con.create_function(uda2, database=db)
 
-        self.con.create_function(udf1, database=db)
+def test_count_uda(udfcon, alltypes, test_data_db, wrapped_count_uda):
+    func = wrapped_count_uda
+    func.register(func.name, test_data_db)
+    udfcon.create_function(func, database=test_data_db)
 
-        self.con.drop_database(db, force=True)
+    # it works!
+    func(alltypes.int_col).execute()
+    # self.temp_udas.append((func.name, ['int32']))
 
-        assert not self.con.exists_database(db)
 
-    def _wrap_count_uda(self, name=None):
-        if name is None:
-            name = 'user_count_{0}'.format(util.guid())
-        func = api.wrap_uda(self.uda_so, ['int32'], 'int64',
-                            'CountUpdate', name=name)
-        return func
+def test_list_udas(udfcon, temp_database, wrapped_count_uda):
+    func = wrapped_count_uda
+    db = temp_database
+    udfcon.create_function(func, database=db)
+
+    funcs = udfcon.list_udas(database=db)
+
+    f = funcs[0]
+    assert f.name == func.name
+    assert f.inputs == func.inputs
+    assert f.output == func.output
+
+
+def test_drop_database_with_udfs_and_udas(
+    udfcon, temp_database, wrapped_count_uda
+):
+    uda1 = wrapped_count_uda
+
+    udf1 = api.wrap_udf(udf_ll, ['boolean'], 'boolean', 'Identity',
+                        'udf_{0}'.format(util.guid()))
+
+    db = temp_database
+
+    udfcon.create_database(db)
+
+    udfcon.create_function(uda1, database=db)
+    udfcon.create_function(udf1, database=db)
+    # drop happens in test tear down
 
 
 class TestUDFDDL(unittest.TestCase):
@@ -425,8 +439,9 @@ class TestUDFDDL(unittest.TestCase):
         self.output = 'int64'
 
     def test_create_udf(self):
-        stmt = ddl.CreateFunction('/foo/bar.so', 'testFunc', self.inputs,
-                                  self.output, self.name)
+        func = api.wrap_udf('/foo/bar.so', self.inputs, self.output,
+                            so_symbol='testFunc', name=self.name)
+        stmt = ddl.CreateUDF(func)
         result = stmt.compile()
         expected = ("CREATE FUNCTION `test_name`(string, string) "
                     "returns bigint "
@@ -434,9 +449,14 @@ class TestUDFDDL(unittest.TestCase):
         assert result == expected
 
     def test_create_udf_type_conversions(self):
-        stmt = ddl.CreateFunction('/foo/bar.so', 'testFunc',
-                                  ['string', 'int8', 'int16', 'int32'],
-                                  self.output, self.name)
+        inputs = ['string', 'int8', 'int16', 'int32']
+        func = api.wrap_udf('/foo/bar.so', inputs, self.output,
+                            so_symbol='testFunc', name=self.name)
+        stmt = ddl.CreateUDF(func)
+
+        # stmt = ddl.CreateFunction('/foo/bar.so', 'testFunc',
+        #                           ,
+        #                           self.output, self.name)
         result = stmt.compile()
         expected = ("CREATE FUNCTION `test_name`(string, tinyint, "
                     "smallint, int) returns bigint "
@@ -483,11 +503,11 @@ class TestUDFDDL(unittest.TestCase):
                     ("\nfinalize_fn='Finalize'"))
 
         for ser in [True, False]:
-            stmt = ddl.CreateAggregateFunction('/foo/bar.so', self.inputs,
-                                               self.output, 'Update', 'Init',
-                                               'Merge',
-                                               'Serialize' if ser else None,
-                                               'Finalize', self.name, 'bar')
+            func = api.wrap_uda('/foo/bar.so', self.inputs, self.output,
+                                update_fn='Update', init_fn='Init',
+                                merge_fn='Merge', finalize_fn='Finalize',
+                                serialize_fn='Serialize' if ser else None)
+            stmt = ddl.CreateUDA(func, name=self.name, database='bar')
             result = stmt.compile()
             expected = make_ex(ser)
             assert result == expected
