@@ -1,7 +1,9 @@
+from ibis.pandas.execution.arrays import *  # noqa: F401,F403
+from ibis.pandas.execution.decimal import *  # noqa: F401,F403
 from ibis.pandas.execution.generic import *  # noqa: F401,F403
+from ibis.pandas.execution.join import *  # noqa: F401,F403
+from ibis.pandas.execution.maps import *  # noqa: F401,F403
+from ibis.pandas.execution.selection import *  # noqa: F401,F403
 from ibis.pandas.execution.strings import *  # noqa: F401,F403
 from ibis.pandas.execution.temporal import *  # noqa: F401,F403
-from ibis.pandas.execution.arrays import *  # noqa: F401,F403
-from ibis.pandas.execution.selection import *  # noqa: F401,F403
-from ibis.pandas.execution.join import *  # noqa: F401,F403
 from ibis.pandas.execution.window import *  # noqa: F401,F403
