@@ -1,56 +1,66 @@
 from __future__ import absolute_import
 
+import collections
 import datetime
 import decimal
+import math
 import numbers
 import operator
-
-import six
+from collections import Sized
 
 import numpy as np
 import pandas as pd
-
-from pandas.core.groupby import SeriesGroupBy, DataFrameGroupBy
-
+import six
 import toolz
+from pandas.core.groupby import DataFrameGroupBy, SeriesGroupBy
 
-from ibis import compat
-
+import ibis
 import ibis.common as com
 import ibis.expr.types as ir
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 
-from ibis.compat import functools
+from ibis.compat import functools, map, DatetimeTZDtype, zip
 
 from ibis.pandas.core import (
+    execute,
     boolean_types,
     integer_types,
     floating_types,
     simple_types,
     numeric_types,
     fixed_width_types,
-    scalar_types
+    scalar_types,
+    timedelta_types,
 )
-from ibis.pandas.dispatch import execute, execute_node
+
+from ibis.pandas.dispatch import execute_node, execute_literal
 from ibis.pandas.execution import constants
 
 
-@execute_node.register(ir.Literal)
-@execute_node.register(ir.Literal, object)
-@execute_node.register(ir.Literal, object, dt.DataType)
-def execute_node_literal(op, *args, **kwargs):
-    return op.value
+# By default return the literal value
+@execute_literal.register(ops.Literal, object, dt.DataType)
+def execute_node_literal_value_datatype(op, value, datatype, **kwargs):
+    return value
+
+
+@execute_literal.register(
+    ops.Literal,
+    timedelta_types + six.string_types + integer_types,
+    dt.Interval
+)
+def execute_interval_literal(op, value, dtype, **kwargs):
+    return pd.Timedelta(value, dtype.unit)
 
 
 @execute_node.register(ops.Limit, pd.DataFrame, integer_types, integer_types)
-def execute_limit_frame(op, data, limit, offset, **kwargs):
-    return data.iloc[offset:offset + limit]
+def execute_limit_frame(op, data, nrows, offset, **kwargs):
+    return data.iloc[offset:offset + nrows]
 
 
 @execute_node.register(ops.Cast, SeriesGroupBy, dt.DataType)
 def execute_cast_series_group_by(op, data, type, **kwargs):
-    result = execute_node(op, data.obj, type, **kwargs)
+    result = execute_cast_series_generic(op, data.obj, type, **kwargs)
     return result.groupby(data.grouper.groupings)
 
 
@@ -69,9 +79,7 @@ def execute_cast_series_array(op, data, type, **kwargs):
             '(e.g., number, string, or timestamp)'
         )
     return data.map(
-        lambda array, numpy_type=numpy_type: [
-            numpy_type(element) for element in array
-        ]
+        lambda array, numpy_type=numpy_type: list(map(numpy_type, array))
     )
 
 
@@ -87,7 +95,7 @@ def execute_cast_series_timestamp(op, data, type, **kwargs):
 
     if isinstance(from_type, (dt.Timestamp, dt.Date)):
         return data.astype(
-            'M8[ns]' if tz is None else compat.DatetimeTZDtype('ns', tz)
+            'M8[ns]' if tz is None else DatetimeTZDtype('ns', tz)
         )
 
     if isinstance(from_type, (dt.String, dt.Integer)):
@@ -142,19 +150,44 @@ def execute_cast_series_date(op, data, type, **kwargs):
     raise TypeError("Don't know how to cast {} to {}".format(from_type, type))
 
 
-@execute_node.register(ops.Negate, pd.Series)
-def execute_series_unary_op_negate(op, data, **kwargs):
+@execute_node.register(ops.SortKey, pd.Series, bool)
+def execute_sort_key_series_bool(op, data, ascending, **kwargs):
+    return data
+
+
+def call_numpy_ufunc(func, op, data, **kwargs):
     if data.dtype == np.dtype(np.object_):
         return data.apply(functools.partial(execute_node, op, **kwargs))
-    return np.negative(data)
+    return func(data)
+
+
+@execute_node.register(ops.Negate, fixed_width_types + timedelta_types)
+def execute_obj_negate(op, data, **kwargs):
+    return -data
+
+
+@execute_node.register(ops.Negate, pd.Series)
+def execute_series_negate(op, data, **kwargs):
+    return call_numpy_ufunc(np.negative, op, data, **kwargs)
+
+
+@execute_node.register(ops.Negate, SeriesGroupBy)
+def execute_series_group_by_negate(op, data, **kwargs):
+    return execute_series_negate(op, data.obj, **kwargs).groupby(
+        data.grouper.groupings)
 
 
 @execute_node.register(ops.UnaryOp, pd.Series)
 def execute_series_unary_op(op, data, **kwargs):
     function = getattr(np, type(op).__name__.lower())
-    if data.dtype == np.dtype(np.object_):
-        return data.apply(functools.partial(execute_node, op, **kwargs))
-    return function(data)
+    return call_numpy_ufunc(function, op, data, **kwargs)
+
+
+@execute_node.register((ops.Ceil, ops.Floor), pd.Series)
+def execute_series_ceil(op, data, **kwargs):
+    return_type = np.object_ if data.dtype == np.object_ else np.int64
+    func = getattr(np, type(op).__name__.lower())
+    return call_numpy_ufunc(func, op, data, **kwargs).astype(return_type)
 
 
 def vectorize_object(op, arg, *args, **kwargs):
@@ -191,28 +224,32 @@ def execute_series_clip(op, data, lower, upper, **kwargs):
 
 
 @execute_node.register(ops.Quantile, (pd.Series, SeriesGroupBy), numeric_types)
-def execute_series_quantile(op, data, quantile, context=None, **kwargs):
-    return context.agg(
+def execute_series_quantile(op, data, quantile, aggcontext=None, **kwargs):
+    return aggcontext.agg(
         data, 'quantile', q=quantile, interpolation=op.interpolation
     )
 
 
-@execute_node.register(ops.MultiQuantile, pd.Series, list)
-def execute_series_quantile_list(op, data, quantile, context=None, **kwargs):
-    result = context.agg(
+@execute_node.register(ops.MultiQuantile, pd.Series, collections.Sequence)
+def execute_series_quantile_sequence(
+    op, data, quantile, aggcontext=None, **kwargs
+):
+    result = aggcontext.agg(
         data, 'quantile', q=quantile, interpolation=op.interpolation
     )
     return list(result)
 
 
-@execute_node.register(ops.MultiQuantile, SeriesGroupBy, list)
+@execute_node.register(ops.MultiQuantile, SeriesGroupBy, collections.Sequence)
 def execute_series_quantile_groupby(
-        op, data, quantile, context=None, **kwargs):
+        op, data, quantile, aggcontext=None, **kwargs):
 
     def q(x, quantile, interpolation):
-        return [x.quantile(quantile, interpolation=interpolation).tolist()]
+        result = x.quantile(quantile, interpolation=interpolation).tolist()
+        res = [result for _ in range(len(x))]
+        return res
 
-    result = context.agg(data, q, quantile, op.interpolation)
+    result = aggcontext.agg(data, q, quantile, op.interpolation)
     return result
 
 
@@ -300,10 +337,7 @@ def execute_cast_string_literal(op, data, type, **kwargs):
     (six.integer_types, type(None))
 )
 def execute_round_scalars(op, data, places, **kwargs):
-    if places is None:
-        return np.around(data)
-    else:
-        return np.around(data, places)
+    return round(data, places) if places else round(data)
 
 
 @execute_node.register(
@@ -314,7 +348,8 @@ def execute_round_scalars(op, data, places, **kwargs):
 def execute_round_series(op, data, places, **kwargs):
     if data.dtype == np.dtype(np.object_):
         return vectorize_object(op, data, places, **kwargs)
-    return data.round(places if places is not None else 0)
+    result = data.round(places or 0)
+    return result if places else result.astype('int64')
 
 
 @execute_node.register(ops.TableColumn, (pd.DataFrame, DataFrameGroupBy))
@@ -335,7 +370,7 @@ def execute_aggregation_dataframe(op, data, scope=None, **kwargs):
     if predicates:
         predicate = functools.reduce(
             operator.and_,
-            (execute(p, scope, **kwargs) for p in predicates)
+            (execute(p, scope=scope, **kwargs) for p in predicates)
         )
         data = data.loc[predicate]
 
@@ -346,8 +381,8 @@ def execute_aggregation_dataframe(op, data, scope=None, **kwargs):
             zip(op.by, map(operator.methodcaller('op'), op.by))
         )
         grouping_keys = [
-            by_op.name if isinstance(by_op, ir.TableColumn)
-            else execute(by, scope, **kwargs).rename(by.get_name())
+            by_op.name if isinstance(by_op, ops.TableColumn)
+            else execute(by, scope=scope, **kwargs).rename(by.get_name())
             for by, by_op in grouping_key_pairs
         ]
         columns.update(
@@ -360,10 +395,12 @@ def execute_aggregation_dataframe(op, data, scope=None, **kwargs):
 
     new_scope = toolz.merge(scope, {op.table.op(): source})
     pieces = [
-        pd.Series(execute(metric, new_scope, **kwargs), name=metric.get_name())
+        pd.Series(
+            execute(metric, scope=new_scope, **kwargs), name=metric.get_name())
         for metric in op.metrics
     ]
 
+    # group by always needs a reset to get the grouping key back as a column
     result = pd.concat(pieces, axis=1).reset_index()
     result.columns = [columns.get(c, c) for c in result.columns]
 
@@ -379,17 +416,20 @@ def execute_aggregation_dataframe(op, data, scope=None, **kwargs):
         # TODO(phillipc): Don't recompute identical subexpressions
         predicate = functools.reduce(
             operator.and_,
-            (execute(having, new_scope, **kwargs) for having in op.having)
+            (execute(having, scope=new_scope, **kwargs)
+             for having in op.having)
         )
         assert len(predicate) == len(result), \
             'length of predicate does not match length of DataFrame'
-        result = result.loc[predicate.values].reset_index(drop=True)
+        result = result.loc[predicate.values]
     return result
 
 
 @execute_node.register(ops.Reduction, SeriesGroupBy, type(None))
-def execute_reduction_series_groupby(op, data, mask, context=None, **kwargs):
-    return context.agg(data, type(op).__name__.lower())
+def execute_reduction_series_groupby(
+    op, data, mask, aggcontext=None, **kwargs
+):
+    return aggcontext.agg(data, type(op).__name__.lower())
 
 
 variance_ddof = {
@@ -399,26 +439,33 @@ variance_ddof = {
 
 
 @execute_node.register(ops.Variance, SeriesGroupBy, type(None))
-def execute_reduction_series_groupby_var(op, data, _, context=None, **kwargs):
-    return context.agg(data, 'var', ddof=variance_ddof[op.how])
+def execute_reduction_series_groupby_var(
+    op, data, _, aggcontext=None, **kwargs
+):
+    return aggcontext.agg(data, 'var', ddof=variance_ddof[op.how])
 
 
 @execute_node.register(ops.StandardDev, SeriesGroupBy, type(None))
-def execute_reduction_series_groupby_std(op, data, _, context=None, **kwargs):
-    return context.agg(data, 'std', ddof=variance_ddof[op.how])
+def execute_reduction_series_groupby_std(
+    op, data, _, aggcontext=None, **kwargs
+):
+    return aggcontext.agg(data, 'std', ddof=variance_ddof[op.how])
 
 
-@execute_node.register(ops.CountDistinct, SeriesGroupBy, type(None))
-def execute_count_distinct_series_groupby(op, data, _, context=None, **kwargs):
-    return context.agg(data, 'nunique')
+@execute_node.register(
+    (ops.CountDistinct, ops.HLLCardinality), SeriesGroupBy, type(None))
+def execute_count_distinct_series_groupby(
+    op, data, _, aggcontext=None, **kwargs
+):
+    return aggcontext.agg(data, 'nunique')
 
 
 @execute_node.register(ops.Arbitrary, SeriesGroupBy, type(None))
-def execute_arbitrary_series_groupby(op, data, _, context=None, **kwargs):
+def execute_arbitrary_series_groupby(op, data, _, aggcontext=None, **kwargs):
     if op.how not in {'first', 'last'}:
         raise com.OperationNotDefinedError(
             'Arbitrary {!r} is not supported'.format(op.how))
-    return context.agg(data, op.how)
+    return aggcontext.agg(data, op.how)
 
 
 def _filtered_reduction(mask, method, data):
@@ -426,27 +473,33 @@ def _filtered_reduction(mask, method, data):
 
 
 @execute_node.register(ops.Reduction, SeriesGroupBy, SeriesGroupBy)
-def execute_reduction_series_gb_mask(op, data, mask, context=None, **kwargs):
+def execute_reduction_series_gb_mask(
+    op, data, mask, aggcontext=None, **kwargs
+):
     method = operator.methodcaller(type(op).__name__.lower())
-    return context.agg(
+    return aggcontext.agg(
         data,
         functools.partial(_filtered_reduction, mask.obj, method)
     )
 
 
-@execute_node.register(ops.CountDistinct, SeriesGroupBy, SeriesGroupBy)
+@execute_node.register(
+    (ops.CountDistinct, ops.HLLCardinality),
+    SeriesGroupBy,
+    SeriesGroupBy
+)
 def execute_count_distinct_series_groupby_mask(
-    op, data, mask, context=None, **kwargs
+    op, data, mask, aggcontext=None, **kwargs
 ):
-    return context.agg(
+    return aggcontext.agg(
         data,
         functools.partial(_filtered_reduction, mask.obj, pd.Series.nunique)
     )
 
 
 @execute_node.register(ops.Variance, SeriesGroupBy, SeriesGroupBy)
-def execute_var_series_groupby_mask(op, data, mask, context=None, **kwargs):
-    return context.agg(
+def execute_var_series_groupby_mask(op, data, mask, aggcontext=None, **kwargs):
+    return aggcontext.agg(
         data,
         lambda x, mask=mask.obj, ddof=variance_ddof[op.how]: (
             x[mask[x.index]].var(ddof=ddof)
@@ -455,8 +508,8 @@ def execute_var_series_groupby_mask(op, data, mask, context=None, **kwargs):
 
 
 @execute_node.register(ops.StandardDev, SeriesGroupBy, SeriesGroupBy)
-def execute_std_series_groupby_mask(op, data, mask, context=None, **kwargs):
-    return context.agg(
+def execute_std_series_groupby_mask(op, data, mask, aggcontext=None, **kwargs):
+    return aggcontext.agg(
         data,
         lambda x, mask=mask.obj, ddof=variance_ddof[op.how]: (
             x[mask[x.index]].std(ddof=ddof)
@@ -473,18 +526,24 @@ def execute_count_frame_groupby(op, data, _, **kwargs):
 
 
 @execute_node.register(ops.Reduction, pd.Series, (pd.Series, type(None)))
-def execute_reduction_series_mask(op, data, mask, context=None, **kwargs):
+def execute_reduction_series_mask(op, data, mask, aggcontext=None, **kwargs):
     operand = data[mask] if mask is not None else data
-    return context.agg(operand, type(op).__name__.lower())
+    return aggcontext.agg(operand, type(op).__name__.lower())
 
 
-@execute_node.register(ops.CountDistinct, pd.Series, (pd.Series, type(None)))
-def execute_count_distinct_series_mask(op, data, mask, context=None, **kwargs):
-    return context.agg(data[mask] if mask is not None else data, 'nunique')
+@execute_node.register(
+    (ops.CountDistinct, ops.HLLCardinality),
+    pd.Series,
+    (pd.Series, type(None))
+)
+def execute_count_distinct_series_mask(
+    op, data, mask, aggcontext=None, **kwargs
+):
+    return aggcontext.agg(data[mask] if mask is not None else data, 'nunique')
 
 
 @execute_node.register(ops.Arbitrary, pd.Series, (pd.Series, type(None)))
-def execute_arbitrary_series_mask(op, data, mask, context=None, **kwargs):
+def execute_arbitrary_series_mask(op, data, mask, aggcontext=None, **kwargs):
     if op.how == 'first':
         index = 0
     elif op.how == 'last':
@@ -498,8 +557,8 @@ def execute_arbitrary_series_mask(op, data, mask, context=None, **kwargs):
 
 
 @execute_node.register(ops.StandardDev, pd.Series, (pd.Series, type(None)))
-def execute_standard_dev_series(op, data, mask, context=None, **kwargs):
-    return context.agg(
+def execute_standard_dev_series(op, data, mask, aggcontext=None, **kwargs):
+    return aggcontext.agg(
         data[mask] if mask is not None else data,
         'std',
         ddof=variance_ddof[op.how]
@@ -507,8 +566,8 @@ def execute_standard_dev_series(op, data, mask, context=None, **kwargs):
 
 
 @execute_node.register(ops.Variance, pd.Series, (pd.Series, type(None)))
-def execute_variance_series(op, data, mask, context=None, **kwargs):
-    return context.agg(
+def execute_variance_series(op, data, mask, aggcontext=None, **kwargs):
+    return aggcontext.agg(
         data[mask] if mask is not None else data,
         'var',
         ddof=variance_ddof[op.how]
@@ -516,18 +575,18 @@ def execute_variance_series(op, data, mask, context=None, **kwargs):
 
 
 @execute_node.register((ops.Any, ops.All), pd.Series)
-def execute_any_all_series(op, data, context=None, **kwargs):
-    return context.agg(data, type(op).__name__.lower())
+def execute_any_all_series(op, data, aggcontext=None, **kwargs):
+    return aggcontext.agg(data, type(op).__name__.lower())
 
 
 @execute_node.register(ops.NotAny, pd.Series)
-def execute_notany_series(op, data, context=None, **kwargs):
-    return ~context.agg(data, 'any')
+def execute_notany_series(op, data, aggcontext=None, **kwargs):
+    return ~aggcontext.agg(data, 'any')
 
 
 @execute_node.register(ops.NotAll, pd.Series)
-def execute_notall_series(op, data, context=None, **kwargs):
-    return ~context.agg(data, 'all')
+def execute_notall_series(op, data, aggcontext=None, **kwargs):
+    return ~aggcontext.agg(data, 'all')
 
 
 @execute_node.register(ops.Count, pd.DataFrame, type(None))
@@ -540,21 +599,28 @@ def execute_not_bool(op, data, **kwargs):
     return not data
 
 
+@execute_node.register(ops.BinaryOp, pd.Series, pd.Series)
 @execute_node.register(
-    ops.BinaryOp, (pd.Series, numeric_types), (pd.Series, numeric_types)
-)
-@execute_node.register(ops.StringConcat, pd.Series, pd.Series)
-@execute_node.register(
-    (ops.Comparison, ops.StringConcat), six.string_types, six.string_types
+    (ops.NumericBinaryOp, ops.LogicalBinaryOp, ops.Comparison),
+    numeric_types,
+    pd.Series,
 )
 @execute_node.register(
-    (ops.Comparison, ops.StringConcat, ops.Multiply),
-    pd.Series, six.string_types
+    (ops.NumericBinaryOp, ops.LogicalBinaryOp, ops.Comparison),
+    pd.Series,
+    numeric_types,
 )
 @execute_node.register(
-    (ops.Comparison, ops.Multiply, ops.StringConcat),
-    six.string_types, pd.Series
+    (ops.NumericBinaryOp, ops.LogicalBinaryOp, ops.Comparison),
+    numeric_types,
+    numeric_types,
 )
+@execute_node.register(
+    (ops.Comparison, ops.Add, ops.Multiply), pd.Series, six.string_types)
+@execute_node.register(
+    (ops.Comparison, ops.Add, ops.Multiply), six.string_types, pd.Series)
+@execute_node.register(
+    (ops.Comparison, ops.Add), six.string_types, six.string_types)
 @execute_node.register(ops.Multiply, integer_types, six.string_types)
 @execute_node.register(ops.Multiply, six.string_types, integer_types)
 def execute_binary_op(op, left, right, **kwargs):
@@ -578,19 +644,19 @@ def execute_binary_op_series_group_by(op, left, right, **kwargs):
             'Cannot perform {} operation on two series with '
             'different groupings'.format(type(op).__name__)
         )
-    result = execute_node(op, left.obj, right.obj, **kwargs)
+    result = execute_binary_op(op, left.obj, right.obj, **kwargs)
     return result.groupby(left_groupings)
 
 
 @execute_node.register(ops.BinaryOp, SeriesGroupBy, simple_types)
 def execute_binary_op_series_gb_simple(op, left, right, **kwargs):
-    result = execute_node(op, left.obj, right, **kwargs)
+    result = execute_binary_op(op, left.obj, right, **kwargs)
     return result.groupby(left.grouper.groupings)
 
 
 @execute_node.register(ops.BinaryOp, simple_types, SeriesGroupBy)
 def execute_binary_op_simple_series_gb(op, left, right, **kwargs):
-    result = execute_node(op, left, right.obj, **kwargs)
+    result = execute_binary_op(op, left, right.obj, **kwargs)
     return result.groupby(right.grouper.groupings)
 
 
@@ -678,18 +744,36 @@ def execute_node_self_reference_dataframe(op, data, **kwargs):
     return data
 
 
-@execute_node.register(ir.ValueList)
-def execute_node_value_list(op, **kwargs):
+@execute_node.register(ops.ValueList, collections.Sequence)
+def execute_node_value_list(op, _, **kwargs):
     return [execute(arg, **kwargs) for arg in op.values]
 
 
-@execute_node.register(ops.Contains, pd.Series, list)
-def execute_node_contains_series_list(op, data, elements, **kwargs):
+@execute_node.register(ops.StringConcat, collections.Sequence)
+def execute_node_string_concat(op, args, **kwargs):
+    return functools.reduce(operator.add, args)
+
+
+@execute_node.register(ops.StringJoin, collections.Sequence)
+def execute_node_string_join(op, args, **kwargs):
+    return op.sep.join(args)
+
+
+@execute_node.register(
+    ops.Contains,
+    pd.Series,
+    (collections.Sequence, collections.Set)
+)
+def execute_node_contains_series_sequence(op, data, elements, **kwargs):
     return data.isin(elements)
 
 
-@execute_node.register(ops.NotContains, pd.Series, list)
-def execute_node_not_contains_series_list(op, data, elements, **kwargs):
+@execute_node.register(
+    ops.NotContains,
+    pd.Series,
+    (collections.Sequence, collections.Set)
+)
+def execute_node_not_contains_series_sequence(op, data, elements, **kwargs):
     return ~data.isin(elements)
 
 
@@ -742,3 +826,168 @@ def execute_node_where_scalar_series_scalar(op, cond, true, false, **kwargs):
 @execute_node.register(ops.Where, boolean_types, scalar_types, pd.Series)
 def execute_node_where_scalar_scalar_series(op, cond, true, false, **kwargs):
     return pd.Series(np.repeat(true, len(false))) if cond else false
+
+
+@execute_node.register(
+    ibis.pandas.client.PandasTable, ibis.pandas.client.PandasClient)
+def execute_database_table_client(op, client, **kwargs):
+    return client.dictionary[op.name]
+
+
+MATH_FUNCTIONS = {
+    ops.Floor: math.floor,
+    ops.Ln: math.log,
+    ops.Log2: lambda x: math.log(x, 2),
+    ops.Log10: math.log10,
+    ops.Exp: math.exp,
+    ops.Sqrt: math.sqrt,
+    ops.Abs: abs,
+    ops.Ceil: math.ceil,
+    ops.Sign: lambda x: 0 if not x else -1 if x < 0 else 1,
+}
+
+MATH_FUNCTION_TYPES = tuple(MATH_FUNCTIONS.keys())
+
+
+@execute_node.register(MATH_FUNCTION_TYPES, numeric_types)
+def execute_node_math_function_number(op, value, **kwargs):
+    return MATH_FUNCTIONS[type(op)](value)
+
+
+@execute_node.register(ops.Log, numeric_types, numeric_types)
+def execute_node_log_number_number(op, value, base, **kwargs):
+    return math.log(value, base)
+
+
+@execute_node.register(ops.IfNull, pd.Series, scalar_types + (type(None),))
+@execute_node.register(ops.IfNull, pd.Series, pd.Series)
+def execute_node_ifnull_series(op, value, replacement, **kwargs):
+    return value.fillna(replacement)
+
+
+@execute_node.register(ops.IfNull, scalar_types + (type(None),), pd.Series)
+def execute_node_ifnull_scalar_series(op, value, replacement, **kwargs):
+    return (
+        pd.Series(value, index=replacement.index)
+        if pd.notnull(value) else replacement
+    )
+
+
+@execute_node.register(
+    ops.IfNull, scalar_types + (type(None),), scalar_types + (type(None),))
+def execute_node_if_scalars(op, value, replacement, **kwargs):
+    return replacement if pd.isnull(value) else value
+
+
+@execute_node.register(ops.NullIf, bool, scalar_types + (type(None),))
+def execute_node_nullif_scalars(op, condition, value, **kwargs):
+    return np.nan if condition else value
+
+
+@execute_node.register(ops.NullIf, pd.Series, pd.Series)
+def execute_node_nullif_series(op, condition, series, **kwargs):
+    return pd.Series(np.where(condition.values, np.nan, series.values))
+
+
+@execute_node.register(ops.NullIf, pd.Series, scalar_types + (type(None),))
+def execute_node_nullif_series_scalar(op, condition, value, **kwargs):
+    values = np.repeat(value, len(condition))
+    return pd.Series(np.where(condition.values, np.nan, values))
+
+
+@execute_node.register(ops.NullIf, bool, pd.Series)
+def execute_node_nullif_scalar_series(op, condition, value, **kwargs):
+    return pd.Series([None], index=value.index) if condition else value
+
+
+def coalesce(values):
+    return functools.reduce(lambda x, y: x if not pd.isnull(x) else y, values)
+
+
+@toolz.curry
+def promote_to_sequence(length, obj):
+    return obj.values if isinstance(obj, pd.Series) else np.repeat(obj, length)
+
+
+def compute_row_reduction(func, value, **kwargs):
+    final_sizes = {len(x) for x in value if isinstance(x, Sized)}
+    if not final_sizes:
+        return func(value)
+
+    final_size, = final_sizes
+    raw = func(list(map(promote_to_sequence(final_size), value)), **kwargs)
+    return pd.Series(raw).squeeze()
+
+
+@execute_node.register(ops.Greatest, collections.Sequence)
+def execute_node_greatest_list(op, value, **kwargs):
+    return compute_row_reduction(np.maximum.reduce, value, axis=0)
+
+
+@execute_node.register(ops.Least, collections.Sequence)
+def execute_node_least_list(op, value, **kwargs):
+    return compute_row_reduction(np.minimum.reduce, value, axis=0)
+
+
+@execute_node.register(ops.Coalesce, collections.Sequence)
+def execute_node_coalesce(op, values, **kwargs):
+    # TODO: this is slow
+    return compute_row_reduction(coalesce, values)
+
+
+@execute_node.register(ops.ExpressionList, collections.Sequence)
+def execute_node_expr_list(op, sequence, **kwargs):
+    # TODO: no true approx count distinct for pandas, so we use exact for now
+    columns = [e.get_name() for e in op.exprs]
+    schema = ibis.schema(list(zip(columns, (e.type() for e in op.exprs))))
+    data = {col: [execute(el, **kwargs)] for col, el in zip(columns, sequence)}
+    return schema.apply_to(pd.DataFrame(data, columns=columns))
+
+
+def wrap_case_result(raw, expr):
+    """Wrap a CASE statement result in a Series and handle returning scalars.
+
+    Parameters
+    ----------
+    raw : ndarray[T]
+        The raw results of executing the ``CASE`` expression
+    expr : ValueExpr
+        The expression from the which `raw` was computed
+
+    Returns
+    -------
+    Union[scalar, Series]
+    """
+    raw_1d = np.atleast_1d(raw)
+    if np.any(pd.isnull(raw_1d)):
+        result = pd.Series(raw_1d)
+    else:
+        result = pd.Series(
+            raw_1d, dtype=constants.IBIS_TYPE_TO_PANDAS_TYPE[expr.type()])
+    if result.size == 1 and isinstance(expr, ir.ScalarExpr):
+        return result.item()
+    return result
+
+
+@execute_node.register(ops.SearchedCase, list, list, object)
+def execute_searched_case(op, whens, thens, otherwise, **kwargs):
+    if otherwise is None:
+        otherwise = np.nan
+    raw = np.select(whens, thens, otherwise)
+    return wrap_case_result(raw, op.to_expr())
+
+
+@execute_node.register(ops.SimpleCase, object, list, list, object)
+def execute_simple_case_scalar(op, value, whens, thens, otherwise, **kwargs):
+    if otherwise is None:
+        otherwise = np.nan
+    raw = np.select(np.asarray(whens) == value, thens, otherwise)
+    return wrap_case_result(raw, op.to_expr())
+
+
+@execute_node.register(ops.SimpleCase, pd.Series, list, list, object)
+def execute_simple_case_series(op, value, whens, thens, otherwise, **kwargs):
+    if otherwise is None:
+        otherwise = np.nan
+    raw = np.select([value == when for when in whens], thens, otherwise)
+    return wrap_case_result(raw, op.to_expr())
