@@ -3,7 +3,6 @@
 
 from __future__ import absolute_import
 
-import itertools
 import operator
 
 from collections import OrderedDict
@@ -19,7 +18,8 @@ import ibis.expr.operations as ops
 
 from ibis.compat import functools
 
-from ibis.pandas.dispatch import execute, execute_node
+from ibis.pandas.dispatch import execute_node
+from ibis.pandas.core import execute
 from ibis.pandas.execution import constants, util
 
 
@@ -44,8 +44,7 @@ Notes
 -----
 :class:`~ibis.expr.types.ScalarExpr` instances occur when a specific column
 projection is a window operation.
-"""
-)
+""")
 
 
 @compute_projection.register(ir.ScalarExpr, ops.Selection, pd.DataFrame)
@@ -71,8 +70,10 @@ def compute_projection_scalar_expr(expr, parent, data, scope=None, **kwargs):
     )
 
     new_scope = toolz.merge(scope, additional_scope, factory=OrderedDict)
-    result = execute(expr, new_scope, **kwargs)
-    return pd.Series([result], name=name, index=data.index)
+    scalar = execute(expr, scope=new_scope, **kwargs)
+    result = pd.Series([scalar], name=name).repeat(len(data.index))
+    result.index = data.index
+    return result
 
 
 @compute_projection.register(ir.ColumnExpr, ops.Selection, pd.DataFrame)
@@ -81,7 +82,7 @@ def compute_projection_column_expr(expr, parent, data, scope=None, **kwargs):
     op = expr.op()
     parent_table_op = parent.table.op()
 
-    if isinstance(op, ir.TableColumn):
+    if isinstance(op, ops.TableColumn):
         # slightly faster path for simple column selection
         name = op.name
 
@@ -92,7 +93,7 @@ def compute_projection_column_expr(expr, parent, data, scope=None, **kwargs):
             raise KeyError(name)
 
         root_table, = op.root_tables()
-        left_root, right_root = ir.distinct_roots(
+        left_root, right_root = ops.distinct_roots(
             parent_table_op.left, parent_table_op.right
         )
         suffixes = {left_root: constants.LEFT_JOIN_SUFFIX,
@@ -110,7 +111,7 @@ def compute_projection_column_expr(expr, parent, data, scope=None, **kwargs):
     }
 
     new_scope = toolz.merge(scope, additional_scope)
-    result = execute(expr, new_scope, **kwargs)
+    result = execute(expr, scope=new_scope, **kwargs)
     assert result_name is not None, 'Column selection name is None'
     return result.rename(result_name)
 
@@ -133,13 +134,6 @@ def compute_projection_table_expr(expr, parent, data, **kwargs):
     return map_new_column_names_to_data(mapping, data)
 
 
-@compute_projection.register(object, ops.Selection, pd.DataFrame)
-def compute_projection_default(op, parent, data, **kwargs):
-    raise TypeError(
-        "Don't know how to compute projection of {}".format(type(op).__name__)
-    )
-
-
 def remap_overlapping_column_names(table_op, root_table, data_columns):
     """Return an ``OrderedDict`` mapping possibly suffixed column names to
     column names without suffixes.
@@ -162,7 +156,7 @@ def remap_overlapping_column_names(table_op, root_table, data_columns):
     if not isinstance(table_op, ops.Join):
         return None
 
-    left_root, right_root = ir.distinct_roots(table_op.left, table_op.right)
+    left_root, right_root = ops.distinct_roots(table_op.left, table_op.right)
     suffixes = {left_root: constants.LEFT_JOIN_SUFFIX,
                 right_root: constants.RIGHT_JOIN_SUFFIX}
     column_names = [
@@ -225,7 +219,7 @@ def _compute_predicates(table_op, predicates, data, scope, **kwargs):
             additional_scope[root_table] = new_data
 
         new_scope = toolz.merge(scope, additional_scope)
-        yield execute(predicate, new_scope, **kwargs)
+        yield execute(predicate, scope=new_scope, **kwargs)
 
 
 physical_tables = Dispatcher(
@@ -236,11 +230,11 @@ Return the underlying physical tables nodes of a
 
 Parameters
 ----------
-op : ir.Node
+op : ops.Node
 
 Returns
 -------
-tables : List[ir.Node]
+tables : List[ops.Node]
 """
 )
 
@@ -261,7 +255,7 @@ def physical_tables_join(join):
     # Physical roots of Join nodes are the unique physical roots of their
     # left and right TableNodes.
     return list(toolz.unique(
-        itertools.chain(
+        toolz.concatv(
             toolz.unique(physical_tables(join.left.op()), key=id),
             toolz.unique(physical_tables(join.right.op()), key=id)
         ),
@@ -269,7 +263,7 @@ def physical_tables_join(join):
     ))
 
 
-@physical_tables.register(ir.Node)
+@physical_tables.register(ops.Node)
 def physical_tables_node(node):
     # Iterative case. Any other Node's physical roots are the unique physical
     # roots of that Node's root tables.
@@ -287,13 +281,24 @@ def execute_selection_dataframe(op, data, scope=None, **kwargs):
     # Build up the individual pandas structures from column expressions
     if selections:
         data_pieces = []
-
         for selection in selections:
             pandas_object = compute_projection(
-                selection, op, data, scope=scope, **kwargs
+                selection,
+                op,
+                data,
+                scope=scope,
+                **kwargs
             )
             data_pieces.append(pandas_object)
-        result = pd.concat(data_pieces, axis=1)
+
+        new_pieces = [
+            piece.reset_index(
+                level=list(range(1, piece.index.nlevels)), drop=True)
+            if piece.index.nlevels > 1
+            else piece
+            for piece in data_pieces
+        ]
+        result = pd.concat(new_pieces, axis=1)
 
     if predicates:
         predicates = _compute_predicates(
@@ -305,5 +310,23 @@ def execute_selection_dataframe(op, data, scope=None, **kwargs):
         result = result.loc[predicate]
 
     if sort_keys:
-        result = util.compute_sorted_frame(sort_keys, result, **kwargs)
-    return result.reset_index(drop=True)
+        result, grouping_keys, ordering_keys = util.compute_sorted_frame(
+            result, order_by=sort_keys, scope=scope, **kwargs)
+    else:
+        grouping_keys = ordering_keys = ()
+
+    # return early if we do not have any temporary grouping or ordering columns
+    assert not grouping_keys, 'group by should never show up in Selection'
+    if not ordering_keys:
+        return result
+
+    # create a sequence of columns that we need to drop
+    temporary_columns = pd.Index(
+        toolz.concatv(grouping_keys, ordering_keys)).difference(data.columns)
+
+    # no reason to call drop if we don't need to
+    if temporary_columns.empty:
+        return result
+
+    # drop every temporary column we created for ordering or grouping
+    return result.drop(temporary_columns, axis=1)
