@@ -49,6 +49,9 @@ def df():
         'array_of_float64': [[1.0, 2.0], [3.0], []],
         'array_of_int64': [[1, 2], [], [3]],
         'array_of_strings': [['a', 'b'], [], ['c']],
+        'map_of_strings_integers': [{'a': 1, 'b': 2}, None, {}],
+        'map_of_integers_strings': [{}, None, {1: 'a', 2: 'b'}],
+        'map_of_complex_values': [None, {'a': [1, 2, 3], 'b': []}, {}],
     })
 
 
@@ -136,7 +139,7 @@ def time_keyed_df2():
 
 @pytest.fixture(scope='module')
 def client(
-    df, df1, df2, df3, time_df1, time_df2, time_keyed_df1, time_keyed_df2
+    df, df1, df2, df3, time_df1, time_df2, time_keyed_df1, time_keyed_df2,
 ):
     return ibis.pandas.connect(
         dict(
@@ -149,7 +152,7 @@ def client(
             time_df1=time_df1,
             time_df2=time_df2,
             time_keyed_df1=time_keyed_df1,
-            time_keyed_df2=time_keyed_df2
+            time_keyed_df2=time_keyed_df2,
         )
     )
 
@@ -164,17 +167,20 @@ def df3():
     })
 
 
+t_schema = {
+    'decimal': dt.Decimal(4, 3),
+    'array_of_float64': dt.Array(dt.double),
+    'array_of_int64': dt.Array(dt.int64),
+    'array_of_strings': dt.Array(dt.string),
+    'map_of_strings_integers': dt.Map(dt.string, dt.int64),
+    'map_of_integers_strings': dt.Map(dt.int64, dt.string),
+    'map_of_complex_values': dt.Map(dt.string, dt.Array(dt.int64)),
+}
+
+
 @pytest.fixture(scope='module')
 def t(client):
-    return client.table(
-        'df',
-        schema={
-            'decimal': dt.Decimal(4, 3),
-            'array_of_float64': dt.Array(dt.double),
-            'array_of_int64': dt.Array(dt.int64),
-            'array_of_strings': dt.Array(dt.string),
-        }
-    )
+    return client.table('df', schema=t_schema)
 
 
 @pytest.fixture(scope='module')
