@@ -12,7 +12,6 @@ import pandas.util.testing as tm
 
 import ibis
 import ibis.expr.datatypes as dt
-from ibis.common import IbisTypeError
 
 pytest.importorskip('multipledispatch')
 
@@ -35,40 +34,6 @@ def test_read_with_undiscoverable_type(client):
         client.table('df')
 
 
-merge_asof_minversion = pytest.mark.skipif(
-    pd.__version__ < '0.19.2',
-    reason="at least pandas-0.19.2 required for merge_asof")
-
-
-@merge_asof_minversion
-def test_asof_join(time_left, time_right, time_df1, time_df2):
-    expr = time_left.asof_join(time_right, 'time')
-    _test_asof(expr, time_df1, time_df2)
-
-
-@merge_asof_minversion
-def test_asof_join_predicate(time_left, time_right, time_df1, time_df2):
-    expr = time_left.asof_join(
-        time_right, time_left.time == time_right.time)
-    _test_asof(expr, time_df1, time_df2)
-
-
-def _test_asof(expr, df1, df2):
-    result = expr.execute()
-    expected = pd.merge_asof(df1, df2, on='time')
-    tm.assert_frame_equal(result[expected.columns], expected)
-
-
-@merge_asof_minversion
-def test_keyed_asof_join(
-        time_keyed_left, time_keyed_right, time_keyed_df1, time_keyed_df2):
-    expr = time_keyed_left.asof_join(time_keyed_right, 'time', by='key')
-    result = expr.execute()
-    expected = pd.merge_asof(
-        time_keyed_df1, time_keyed_df2, on='time', by='key')
-    tm.assert_frame_equal(result[expected.columns], expected)
-
-
 def test_selection(t, df):
     expr = t[
         ((t.plain_strings == 'a') | (t.plain_int64 == 3)) &
@@ -89,6 +54,25 @@ def test_mutate(t, df):
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
+def test_project_scope_does_not_override(t, df):
+    col = t.plain_int64
+    expr = t[[
+        col.name('new_col'),
+        col.sum().over(ibis.window(group_by='dup_strings')).name('grouped')
+    ]]
+    result = expr.execute()
+    expected = pd.concat(
+        [
+            df[['plain_int64', 'dup_strings']].rename(
+                columns={'plain_int64': 'new_col'}),
+            df.groupby('dup_strings').plain_int64.transform('sum').reset_index(
+                drop=True).rename('grouped')
+        ],
+        axis=1
+    )[['new_col', 'grouped']]
+    tm.assert_frame_equal(result, expected)
+
+
 @pytest.mark.parametrize(
     'where',
     [
@@ -108,10 +92,10 @@ def test_mutate(t, df):
         (methodcaller('log10'), np.log10),
         (methodcaller('log', 2), lambda x: np.log(x) / np.log(2)),
         (methodcaller('log2'), np.log2),
-        (methodcaller('round', 0), methodcaller('round', 0)),
+        (methodcaller('round', 0), lambda x: x.round(0).astype('int64')),
         (methodcaller('round', -2), methodcaller('round', -2)),
         (methodcaller('round', 2), methodcaller('round', 2)),
-        (methodcaller('round'), methodcaller('round')),
+        (methodcaller('round'), lambda x: x.round().astype('int64')),
         (methodcaller('sign'), np.sign),
         (methodcaller('sqrt'), np.sqrt),
     ]
@@ -157,7 +141,13 @@ def test_aggregation_group_by(t, df, where, ibis_func, pandas_func):
     result['neg_mean_int64_with_zeros'] = (
         result.neg_mean_int64_with_zeros.astype('float64')
     )
-    tm.assert_frame_equal(result[expected.columns], expected)
+    expected['mean_float64_positive'] = (
+        expected.mean_float64_positive.astype('float64'))
+    result['mean_float64_positive'] = result.mean_float64_positive.astype(
+        'float64')
+    lhs = result[expected.columns]
+    rhs = expected
+    tm.assert_frame_equal(lhs, rhs)
 
 
 def test_aggregation_without_group_by(t, df):
@@ -279,7 +269,7 @@ def test_frame_limit(t, df, offset):
     n = 5
     df_expr = t.limit(n, offset=offset)
     result = df_expr.execute()
-    expected = df.iloc[offset:offset + n]
+    expected = df.iloc[offset:offset + n].reset_index(drop=True)
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
@@ -437,197 +427,6 @@ def test_notnull(t, df):
     tm.assert_series_equal(result, expected)
 
 
-def test_array_length(t, df):
-    expr = t.projection([
-        t.array_of_float64.length().name('array_of_float64_length'),
-        t.array_of_int64.length().name('array_of_int64_length'),
-        t.array_of_strings.length().name('array_of_strings_length'),
-    ])
-    result = expr.execute()
-    expected = pd.DataFrame({
-        'array_of_float64_length': [2, 1, 0],
-        'array_of_int64_length': [2, 0, 1],
-        'array_of_strings_length': [2, 0, 1],
-    })
-
-    tm.assert_frame_equal(result, expected)
-
-
-def test_array_length_scalar(client):
-    raw_value = [1, 2, 4]
-    value = ibis.literal(raw_value)
-    expr = value.length()
-    result = client.execute(expr)
-    expected = len(raw_value)
-    assert result == expected
-
-
-def test_array_collect(t, df):
-    expr = t.group_by(
-        t.dup_strings
-    ).aggregate(collected=t.float64_with_zeros.collect())
-    result = expr.execute().sort_values('dup_strings').reset_index(drop=True)
-    expected = df.groupby(
-        'dup_strings'
-    ).float64_with_zeros.apply(list).reset_index().rename(
-        columns={'float64_with_zeros': 'collected'}
-    )
-    tm.assert_frame_equal(result, expected)
-
-
-@pytest.mark.xfail(raises=IbisTypeError, reason='Not sure if this should work')
-def test_array_collect_scalar(client):
-    raw_value = 'abcd'
-    value = ibis.literal(raw_value)
-    expr = value.collect()
-    result = client.execute(expr)
-    expected = [raw_value]
-    assert result == expected
-
-
-@pytest.mark.parametrize(
-    ['start', 'stop'],
-    [
-        (1, 3),
-        (1, 1),
-        (2, 3),
-        (2, 5),
-
-        (None, 3),
-        (None, None),
-        (3, None),
-
-        # negative slices are not supported
-        pytest.mark.xfail(
-            (-3, None),
-            raises=ValueError,
-            reason='Negative slicing not supported'
-        ),
-        pytest.mark.xfail(
-            (None, -3),
-            raises=ValueError,
-            reason='Negative slicing not supported'
-        ),
-        pytest.mark.xfail(
-            (-3, -1),
-            raises=ValueError,
-            reason='Negative slicing not supported'
-        ),
-    ]
-)
-def test_array_slice(t, df, start, stop):
-    expr = t.array_of_strings[start:stop]
-    result = expr.execute()
-    slicer = operator.itemgetter(slice(start, stop))
-    expected = df.array_of_strings.apply(slicer)
-    tm.assert_series_equal(result, expected)
-
-
-@pytest.mark.parametrize(
-    ['start', 'stop'],
-    [
-        (1, 3),
-        (1, 1),
-        (2, 3),
-        (2, 5),
-
-        (None, 3),
-        (None, None),
-        (3, None),
-
-        # negative slices are not supported
-        pytest.mark.xfail(
-            (-3, None),
-            raises=ValueError,
-            reason='Negative slicing not supported'
-        ),
-        pytest.mark.xfail(
-            (None, -3),
-            raises=ValueError,
-            reason='Negative slicing not supported'
-        ),
-        pytest.mark.xfail(
-            (-3, -1),
-            raises=ValueError,
-            reason='Negative slicing not supported'
-        ),
-    ]
-)
-def test_array_slice_scalar(client, start, stop):
-    raw_value = [-11, 42, 10]
-    value = ibis.literal(raw_value)
-    expr = value[start:stop]
-    result = client.execute(expr)
-    expected = raw_value[start:stop]
-    assert result == expected
-
-
-@pytest.mark.parametrize('index', [1, 3, 4, 11, -11])
-def test_array_index(t, df, index):
-    expr = t[t.array_of_float64[index].name('indexed')]
-    result = expr.execute()
-    expected = pd.DataFrame({
-        'indexed': df.array_of_float64.apply(
-            lambda x: x[index] if -len(x) <= index < len(x) else None
-        )
-    })
-    tm.assert_frame_equal(result, expected)
-
-
-@pytest.mark.parametrize('index', [1, 3, 4, 11])
-def test_array_index_scalar(client, index):
-    raw_value = [-10, 1, 2, 42]
-    value = ibis.literal(raw_value)
-    expr = value[index]
-    result = client.execute(expr)
-    expected = raw_value[index] if index < len(raw_value) else None
-    assert result == expected
-
-
-@pytest.mark.parametrize('n', [1, 3, 4, 7, -2])  # negative returns empty list
-@pytest.mark.parametrize('mul', [lambda x, n: x * n, lambda x, n: n * x])
-def test_array_repeat(t, df, n, mul):
-    expr = t.projection([mul(t.array_of_strings, n).name('repeated')])
-    result = expr.execute()
-    expected = pd.DataFrame({'repeated': df.array_of_strings * n})
-    tm.assert_frame_equal(result, expected)
-
-
-@pytest.mark.parametrize('n', [1, 3, 4, 7, -2])  # negative returns empty list
-@pytest.mark.parametrize('mul', [lambda x, n: x * n, lambda x, n: n * x])
-def test_array_repeat_scalar(client, n, mul):
-    raw_array = [1, 2]
-    array = ibis.literal(raw_array)
-    expr = mul(array, n)
-    result = client.execute(expr)
-    expected = mul(raw_array, n)
-    assert result == expected
-
-
-@pytest.mark.parametrize('op', [lambda x, y: x + y, lambda x, y: y + x])
-def test_array_concat(t, df, op):
-    x = t.array_of_float64.cast('array<string>')
-    y = t.array_of_strings
-    expr = op(x, y)
-    result = expr.execute()
-    expected = op(
-        df.array_of_float64.apply(lambda x: list(map(str, x))),
-        df.array_of_strings
-    )
-    tm.assert_series_equal(result, expected)
-
-
-@pytest.mark.parametrize('op', [lambda x, y: x + y, lambda x, y: y + x])
-def test_array_concat_scalar(client, op):
-    raw_left = [1, 2, 3]
-    raw_right = [3, 4]
-    left = ibis.literal(raw_left)
-    right = ibis.literal(raw_right)
-    expr = op(left, right)
-    result = client.execute(expr)
-    assert result == op(raw_left, raw_right)
-
-
 @pytest.mark.parametrize('raw_value', [0.0, 1.0])
 def test_scalar_parameter(t, df, raw_value):
     value = ibis.param(dt.double)
@@ -637,17 +436,7 @@ def test_scalar_parameter(t, df, raw_value):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize(
-    'elements',
-    [
-        [1],
-        (1,),
-        pytest.mark.xfail({1}, raises=TypeError, reason='Not yet implemented'),
-        pytest.mark.xfail(
-            frozenset({1}), raises=TypeError, reason='Not yet implemented'
-        ),
-    ]
-)
+@pytest.mark.parametrize('elements', [[1], (1,), {1}, frozenset({1})])
 def test_isin(t, df, elements):
     expr = t.plain_float64.isin(elements)
     expected = df.plain_float64.isin(elements)
@@ -655,17 +444,7 @@ def test_isin(t, df, elements):
     tm.assert_series_equal(result, expected)
 
 
-@pytest.mark.parametrize(
-    'elements',
-    [
-        [1],
-        (1,),
-        pytest.mark.xfail({1}, raises=TypeError, reason='Not yet implemented'),
-        pytest.mark.xfail(
-            frozenset({1}), raises=TypeError, reason='Not yet implemented'
-        ),
-    ]
-)
+@pytest.mark.parametrize('elements', [[1], (1,), {1}, frozenset({1})])
 def test_notin(t, df, elements):
     expr = t.plain_float64.notin(elements)
     expected = ~df.plain_float64.isin(elements)
@@ -774,7 +553,8 @@ def test_round(t, df):
 
 def test_quantile_groupby(batting, batting_df):
     def q_fun(x, quantile, interpolation):
-        return [x.quantile(quantile, interpolation=interpolation).tolist()]
+        res = x.quantile(quantile, interpolation=interpolation).tolist()
+        return [res for _ in range(len(x))]
 
     frac = 0.2
     intp = 'linear'
@@ -790,3 +570,132 @@ def test_quantile_groupby(batting, batting_df):
                            interpolation=intp)
                 .rename('res'))
     tm.assert_series_equal(result, expected)
+
+
+def test_summary_numeric(batting, batting_df):
+    expr = batting.G.summary()
+    result = expr.execute()
+    assert len(result) == 1
+
+    G = batting_df.G
+    expected = dict(
+        count=G.count(),
+        nulls=G.isnull().sum(),
+        min=G.min(),
+        max=G.max(),
+        sum=G.sum(),
+        mean=G.mean(),
+        approx_nunique=G.nunique(),
+    )
+    assert dict(result.iloc[0]) == expected
+
+
+def test_summary_numeric_group_by(batting, batting_df):
+    expr = batting.groupby('teamID').G.summary()
+    result = expr.execute()
+    expected = batting_df.groupby('teamID').G.apply(
+        lambda s: pd.DataFrame(
+            dict(
+                count=s.count(),
+                nulls=s.isnull().sum(),
+                min=s.min(),
+                max=s.max(),
+                sum=s.sum(),
+                mean=s.mean(),
+                approx_nunique=s.nunique()
+            ),
+            index=[0]
+        )
+    ).reset_index(level=1, drop=True).reset_index()
+    columns = expected.columns
+
+    # TODO: fix isnull().sum() in the pandas backend: the type is incorrect
+    tm.assert_frame_equal(result[columns], expected, check_dtype=False)
+
+
+def test_summary_non_numeric(batting, batting_df):
+    expr = batting.teamID.summary()
+    result = expr.execute()
+    assert len(result) == 1
+    assert len(result.columns) == 3
+    expected = dict(
+        count=batting_df.teamID.count(),
+        nulls=batting_df.teamID.isnull().sum(),
+        uniques=batting_df.teamID.nunique(),
+    )
+    assert dict(result.iloc[0]) == expected
+
+
+def test_summary_non_numeric_group_by(batting, batting_df):
+    expr = batting.groupby('teamID').playerID.summary()
+    result = expr.execute()
+    expected = batting_df.groupby('teamID').playerID.apply(
+        lambda s: pd.DataFrame(
+            dict(
+                count=s.count(),
+                nulls=s.isnull().sum(),
+                uniques=s.nunique()
+            ),
+            index=[0]
+        )
+    ).reset_index(level=1, drop=True).reset_index()
+    columns = expected.columns
+    tm.assert_frame_equal(result[columns], expected, check_dtype=False)
+
+
+def test_searched_case_scalar(client):
+    expr = ibis.case().when(True, 1).when(False, 2).end()
+    result = client.execute(expr)
+    expected = np.int8(1)
+    assert result == expected
+
+
+def test_searched_case_column(batting, batting_df):
+    t = batting
+    df = batting_df
+    expr = (
+        ibis.case()
+            .when(t.RBI < 5, 'really bad team')
+            .when(t.teamID == 'PH1', 'ph1 team')
+            .else_(t.teamID)
+            .end()
+    )
+    result = expr.execute()
+    expected = pd.Series(
+        np.select(
+            [df.RBI < 5, df.teamID == 'PH1'],
+            ['really bad team', 'ph1 team'],
+            df.teamID
+        )
+    )
+    tm.assert_series_equal(result, expected)
+
+
+def test_simple_case_scalar(client):
+    x = ibis.literal(2)
+    expr = x.case().when(2, x - 1).when(3, x + 1).when(4, x + 2).end()
+    result = client.execute(expr)
+    expected = np.int8(1)
+    assert result == expected
+
+
+def test_simple_case_column(batting, batting_df):
+    t = batting
+    df = batting_df
+    expr = (
+        t.RBI.case()
+             .when(5, 'five')
+             .when(4, 'four')
+             .when(3, 'three')
+             .else_('could be good?')
+             .end()
+    )
+    result = expr.execute()
+    expected = pd.Series(
+        np.select(
+            [df.RBI == 5, df.RBI == 4, df.RBI == 3],
+            ['five', 'four', 'three'],
+            'could be good?'
+        )
+    )
+    tm.assert_series_equal(result, expected)
