@@ -8,21 +8,88 @@ from pandas.util import testing as tm
 import pytest
 
 import ibis
+import ibis.expr.operations as ops
+
+from ibis.pandas.dispatch import pre_execute
+
+execute = ibis.pandas.execute
 
 pytestmark = pytest.mark.pandas
 
 
-def test_lead(t, df):
-    expr = t.dup_strings.lead()
+@pytest.fixture(scope='session')
+def sort_kind():
+    return 'mergesort'
+
+
+default = pytest.mark.parametrize('default', [ibis.NA, ibis.literal('a')])
+row_offset = pytest.mark.parametrize(
+    'row_offset', list(map(ibis.literal, [-1, 1, 0])))
+range_offset = pytest.mark.parametrize(
+    'range_offset',
+    [ibis.day(), 2 * ibis.day(), -2 * ibis.day()]
+)
+
+
+@pytest.fixture
+def row_window():
+    return ibis.window(following=0, order_by='plain_int64')
+
+
+@pytest.fixture
+def range_window():
+    return ibis.window(following=0, order_by='plain_datetimes_naive')
+
+
+@default
+@row_offset
+def test_lead(t, df, row_offset, default, row_window):
+    expr = t.dup_strings.lead(row_offset, default=default).over(row_window)
     result = expr.execute()
-    expected = df.dup_strings.shift(-1)
+    expected = df.dup_strings.shift(execute(-row_offset))
+    if default is not ibis.NA:
+        expected = expected.fillna(execute(default))
     tm.assert_series_equal(result, expected)
 
 
-def test_lag(t, df):
-    expr = t.dup_strings.lag()
+@default
+@row_offset
+def test_lag(t, df, row_offset, default, row_window):
+    expr = t.dup_strings.lag(row_offset, default=default).over(row_window)
     result = expr.execute()
-    expected = df.dup_strings.shift(1)
+    expected = df.dup_strings.shift(execute(row_offset))
+    if default is not ibis.NA:
+        expected = expected.fillna(execute(default))
+    tm.assert_series_equal(result, expected)
+
+
+@default
+@range_offset
+def test_lead_delta(t, df, range_offset, default, range_window):
+    expr = t.dup_strings.lead(range_offset, default=default).over(
+        range_window)
+    result = expr.execute()
+    expected = df[['plain_datetimes_naive', 'dup_strings']].set_index(
+        'plain_datetimes_naive').squeeze().tshift(
+                freq=execute(-range_offset)).reindex(
+                df.plain_datetimes_naive).reset_index(drop=True)
+    if default is not ibis.NA:
+        expected = expected.fillna(execute(default))
+    tm.assert_series_equal(result, expected)
+
+
+@default
+@range_offset
+def test_lag_delta(t, df, range_offset, default, range_window):
+    expr = t.dup_strings.lag(range_offset, default=default).over(
+        range_window)
+    result = expr.execute()
+    expected = df[['plain_datetimes_naive', 'dup_strings']].set_index(
+        'plain_datetimes_naive').squeeze().tshift(
+            freq=execute(range_offset)).reindex(
+                df.plain_datetimes_naive).reset_index(drop=True)
+    if default is not ibis.NA:
+        expected = expected.fillna(execute(default))
     tm.assert_series_equal(result, expected)
 
 
@@ -45,8 +112,7 @@ def test_group_by_mutate_analytic(t, df):
         last_value=t.plain_strings.last(),
         avg_broadcast=t.plain_float64 - t.plain_float64.mean(),
         delta=(t.plain_int64 - t.plain_int64.lag()) / (
-            t.plain_float64 - t.plain_float64.lag()
-        )
+               t.plain_float64 - t.plain_float64.lag()),
     )
     result = expr.execute()
 
@@ -66,11 +132,12 @@ def test_group_by_mutate_analytic(t, df):
 
 def test_players(players, players_df):
     lagged = players.mutate(pct=lambda t: t.G - t.G.lag())
-    result = lagged.execute()
     expected = players_df.assign(
         pct=players_df.G - players_df.groupby('playerID').G.shift(1)
     )
-    tm.assert_frame_equal(result[expected.columns], expected)
+    cols = expected.columns.tolist()
+    result = lagged.execute()[cols].sort_values(cols).reset_index(drop=True)
+    tm.assert_frame_equal(result, expected)
 
 
 def test_batting_filter_mean(batting, batting_df):
@@ -84,20 +151,21 @@ def test_batting_filter_mean(batting, batting_df):
 
 def test_batting_zscore(players, players_df):
     expr = players.mutate(g_z=lambda t: (t.G - t.G.mean()) / t.G.std())
-    result = expr.execute()
 
     gb = players_df.groupby('playerID')
     expected = players_df.assign(
         g_z=(players_df.G - gb.G.transform('mean')) / gb.G.transform('std')
     )
-    tm.assert_frame_equal(result[expected.columns], expected)
+    cols = expected.columns.tolist()
+    result = expr.execute()[cols].sort_values(cols).reset_index(
+        drop=True)
+    tm.assert_frame_equal(result, expected)
 
 
 def test_batting_avg_change_in_games_per_year(players, players_df):
     expr = players.mutate(
         delta=lambda t: (t.G - t.G.lag()) / (t.yearID - t.yearID.lag())
     )
-    result = expr.execute()
 
     gb = players_df.groupby('playerID')
     expected = players_df.assign(
@@ -106,7 +174,9 @@ def test_batting_avg_change_in_games_per_year(players, players_df):
         )
     )
 
-    tm.assert_frame_equal(result[expected.columns], expected)
+    cols = expected.columns.tolist()
+    result = expr.execute()[cols].sort_values(cols).reset_index(drop=True)
+    tm.assert_frame_equal(result, expected)
 
 
 @pytest.mark.xfail(AssertionError, reason='NYI')
@@ -126,28 +196,30 @@ def test_batting_most_hits(players, players_df):
 
 def test_batting_quantile(players, players_df):
     expr = players.mutate(hits_quantile=lambda t: t.H.quantile(0.25))
-    result = expr.execute()
     hits_quantile = players_df.groupby('playerID').H.transform(
         'quantile', 0.25
     )
     expected = players_df.assign(hits_quantile=hits_quantile)
-    tm.assert_frame_equal(result[expected.columns], expected)
+    cols = expected.columns.tolist()
+    result = expr.execute()[cols].sort_values(cols).reset_index(drop=True)
+    tm.assert_frame_equal(result, expected)
 
 
 @pytest.mark.parametrize('op', ['sum', 'mean', 'min', 'max'])
-def test_batting_specific_cumulative(batting, batting_df, op):
+def test_batting_specific_cumulative(batting, batting_df, op, sort_kind):
     ibis_method = methodcaller('cum{}'.format(op))
     expr = ibis_method(batting.sort_by([batting.yearID]).G)
     result = expr.execute().astype('float64')
 
     pandas_method = methodcaller(op)
     expected = pandas_method(
-        batting_df[['G', 'yearID']].sort_values('yearID').G.expanding()
+        batting_df[['G', 'yearID']].sort_values(
+            'yearID', kind=sort_kind).G.expanding()
     ).reset_index(drop=True)
     tm.assert_series_equal(result, expected)
 
 
-def test_batting_cumulative(batting, batting_df):
+def test_batting_cumulative(batting, batting_df, sort_kind):
     expr = batting.mutate(
         more_values=lambda t: t.G.sum().over(
             ibis.cumulative_window(order_by=t.yearID)
@@ -156,37 +228,33 @@ def test_batting_cumulative(batting, batting_df):
     result = expr.execute()
 
     columns = ['G', 'yearID']
-    more_values = batting_df[columns].sort_values('yearID').G.cumsum()
+    more_values = batting_df[columns].sort_values(
+        'yearID', kind=sort_kind).G.expanding().sum()
     expected = batting_df.assign(more_values=more_values)
 
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-def test_batting_cumulative_partitioned(batting, batting_df):
-    expr = batting.mutate(
-        more_values=lambda t: t.G.sum().over(
-            ibis.cumulative_window(order_by=t.yearID, group_by=t.lgID)
-        )
-    )
-    result = expr.execute().more_values
+def test_batting_cumulative_partitioned(batting, batting_df, sort_kind):
+    group_by = 'playerID'
+    order_by = 'yearID'
 
-    columns = ['G', 'yearID', 'lgID']
-    key = 'lgID'
-    expected_result = batting_df[columns].groupby(
-        key, sort=False, as_index=False
-    ).apply(lambda df: df.sort_values('yearID')).groupby(
-        key, sort=False
-    ).G.cumsum().sort_index(level=-1)
-    expected = expected_result.reset_index(
-        list(range(expected_result.index.nlevels - 1)),
-        drop=True
-    ).reindex(batting_df.index)
-    expected.name = result.name
+    t = batting
+    expr = t.G.sum().over(
+        ibis.cumulative_window(order_by=order_by, group_by=group_by))
+    expr = t.mutate(cumulative=expr)
+    result = expr.execute()
 
-    tm.assert_series_equal(result, expected)
+    columns = [group_by, order_by, 'G']
+    expected = batting_df[columns].set_index(
+        order_by).groupby(group_by).G.expanding().sum().rename('cumulative')
+
+    tm.assert_series_equal(
+        result.set_index([group_by, order_by]).sort_index().cumulative,
+        expected.sort_index())
 
 
-def test_batting_rolling(batting, batting_df):
+def test_batting_rolling(batting, batting_df, sort_kind):
     expr = batting.mutate(
         more_values=lambda t: t.G.sum().over(
             ibis.trailing_window(5, order_by=t.yearID)
@@ -195,34 +263,30 @@ def test_batting_rolling(batting, batting_df):
     result = expr.execute()
 
     columns = ['G', 'yearID']
-    more_values = batting_df[columns].sort_values('yearID').G.rolling(5).sum()
+    more_values = batting_df[columns].sort_values(
+        'yearID', kind=sort_kind).G.rolling(5).sum()
     expected = batting_df.assign(more_values=more_values)
 
     tm.assert_frame_equal(result[expected.columns], expected)
 
 
-def test_batting_rolling_partitioned(batting, batting_df):
-    expr = batting.mutate(
-        more_values=lambda t: t.G.sum().over(
-            ibis.trailing_window(3, order_by=t.yearID, group_by=t.lgID)
-        )
+def test_batting_rolling_partitioned(batting, batting_df, sort_kind):
+    t = batting
+    group_by = 'playerID'
+    order_by = 'yearID'
+    expr = t.G.sum().over(
+        ibis.trailing_window(3, order_by=t[order_by], group_by=t[group_by])
     )
-    result = expr.execute().more_values
+    expr = t.mutate(rolled=expr)
+    result = expr.execute()
 
-    columns = ['G', 'yearID', 'lgID']
-    key = 'lgID'
-    expected_result = batting_df[columns].groupby(
-        key, sort=False, as_index=False
-    ).apply(lambda df: df.sort_values('yearID')).groupby(
-        key, sort=False
-    ).G.rolling(3).sum().sort_index(level=-1)
-    expected = expected_result.reset_index(
-        list(range(expected_result.index.nlevels - 1)),
-        drop=True
-    ).reindex(batting_df.index)
-    expected.name = result.name
+    columns = [group_by, order_by, 'G']
+    expected = batting_df[columns].set_index(
+        order_by).groupby(group_by).G.rolling(3).sum().rename('rolled')
 
-    tm.assert_series_equal(result, expected)
+    tm.assert_series_equal(
+        result.set_index([group_by, order_by]).sort_index().rolled,
+        expected.sort_index())
 
 
 @pytest.mark.parametrize(
@@ -246,7 +310,7 @@ def test_scalar_broadcasting(batting, batting_df):
     tm.assert_frame_equal(result, expected)
 
 
-def test_mutate_with_window_after_join():
+def test_mutate_with_window_after_join(sort_kind):
     left_df = pd.DataFrame({
         'ints': [0, 1, 2],
         'strings': ['a', 'b', 'c'],
@@ -266,7 +330,7 @@ def test_mutate_with_window_after_join():
     expected = pd.DataFrame({
         'dates': pd.concat(
             [left_df.dates] * 3
-        ).sort_values().reset_index(drop=True),
+        ).sort_values(kind=sort_kind).reset_index(drop=True),
         'ints': [0] * 3 + [1] * 3 + [2] * 3,
         'strings': ['a'] * 3 + ['b'] * 3 + ['c'] * 3,
         'value': [0.0, 3.0, 6.0, 1.0, 4.0, 7.0, np.nan, np.nan, 8.0],
@@ -324,3 +388,48 @@ def test_project_list_scalar():
     result = expr.mutate(res=expr.ints.quantile([0.5, 0.95])).execute()
     tm.assert_series_equal(
         result.res, pd.Series([[1.0, 1.9] for _ in range(0, 3)], name='res'))
+
+
+def test_window_with_preceding_expr():
+    index = pd.date_range('20180101', '20180110')
+    start = 2
+    data = np.arange(start, start + len(index))
+    df = pd.DataFrame({'value': data, 'time': index}, index=index)
+    client = ibis.pandas.connect({'df': df})
+    t = client.table('df')
+    expected = df.set_index('time').value.rolling(
+        '3d', closed='both').mean().reset_index(drop=True)
+    expected.index.name = None
+    day = ibis.day()
+    window = ibis.trailing_window(3 * day, order_by=t.time)
+    expr = t.value.mean().over(window)
+    result = expr.execute()
+    tm.assert_series_equal(result, expected)
+
+
+def test_window_has_pre_execute_scope():
+    signature = ops.Lag, ibis.pandas.PandasClient
+    called = [0]
+
+    @pre_execute.register(*signature)
+    def test_pre_execute(op, client, **kwargs):
+        called[0] += 1
+        return {}
+
+    data = {
+        'key': list('abc'),
+        'value': [1, 2, 3],
+        'dup': list('ggh'),
+    }
+    df = pd.DataFrame(data, columns=['key', 'value', 'dup'])
+    client = ibis.pandas.connect(dict(df=df))
+    t = client.table('df')
+    window = ibis.window(order_by='value')
+    expr = t.key.lag(1).over(window).name('foo')
+    result = expr.execute()
+    assert result is not None
+
+    # once in window op at the top to pickup any scope changes before computing
+    # twice in window op when calling execute on the ops.Lag node at the
+    # beginning of execute and once before the actual computation
+    assert called[0] == 3
