@@ -1,32 +1,24 @@
-# Copyright 2015 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
-import six
+import contextlib
 import numbers
 import operator
-import contextlib
+
+import six
 
 import sqlalchemy as sa
 import sqlalchemy.sql as sql
 
 from sqlalchemy.sql.elements import Over as _Over
 from sqlalchemy.ext.compiler import compiles as sa_compiles
+from sqlalchemy.engine.interfaces import Dialect as SQLAlchemyDialect
+
+from sqlalchemy.dialects.sqlite.base import SQLiteDialect
+from sqlalchemy.dialects.postgresql.base import PGDialect as PostgreSQLDialect
+from sqlalchemy.dialects.mysql.base import MySQLDialect
 
 import pandas as pd
 
 from ibis.compat import parse_version, functools
-from ibis.client import SQLClient, AsyncQuery, Query, Database
+from ibis.client import SQLClient, Query, Database
 from ibis.sql.compiler import Select, Union, TableSetFormatter, Dialect
 
 import ibis
@@ -87,74 +79,91 @@ def _to_sqla_type(itype, type_map=None):
         return type_map[type(itype)]
 
 
-@dt.dtype.register(sa.types.NullType)
-def sa_null(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.NullType)
+def sa_null(_, satype, nullable=True):
     return dt.null
 
 
-@dt.dtype.register(sa.types.Boolean)
-def sa_boolean(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.Boolean)
+def sa_boolean(_, satype, nullable=True):
     return dt.Boolean(nullable=nullable)
 
 
-@dt.dtype.register(sa.types.Numeric)
-def sa_numeric(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.Numeric)
+def sa_numeric(_, satype, nullable=True):
     return dt.Decimal(satype.precision, satype.scale, nullable=nullable)
 
 
-@dt.dtype.register(sa.types.SmallInteger)
-def sa_smallint(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.SmallInteger)
+def sa_smallint(_, satype, nullable=True):
     return dt.Int16(nullable=nullable)
 
 
-@dt.dtype.register(sa.types.Integer)
-def sa_integer(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.Integer)
+def sa_integer(_, satype, nullable=True):
     return dt.Int32(nullable=nullable)
 
 
-@dt.dtype.register(sa.types.BigInteger)
-def sa_bigint(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.dialects.mysql.TINYINT)
+def sa_mysql_tinyint(_, satype, nullable=True):
+    return dt.Int8(nullable=nullable)
+
+
+@dt.dtype.register(SQLAlchemyDialect, sa.types.BigInteger)
+def sa_bigint(_, satype, nullable=True):
     return dt.Int64(nullable=nullable)
 
 
-@dt.dtype.register(sa.types.Float)
-def sa_float(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.Float)
+def sa_float(_, satype, nullable=True):
     return dt.Float(nullable=nullable)
 
 
-@dt.dtype.register(sa.types.String)
-def sa_string(satype, nullable=True):
+@dt.dtype.register(SQLiteDialect, sa.types.Float)
+@dt.dtype.register(PostgreSQLDialect, sa.dialects.postgresql.DOUBLE_PRECISION)
+def sa_double(_, satype, nullable=True):
+    return dt.Double(nullable=nullable)
+
+
+@dt.dtype.register(MySQLDialect, sa.dialects.mysql.DOUBLE)
+def sa_mysql_double(_, satype, nullable=True):
+    # TODO: handle asdecimal=True
+    return dt.Double(nullable=nullable)
+
+
+@dt.dtype.register(SQLAlchemyDialect, sa.types.String)
+def sa_string(_, satype, nullable=True):
     return dt.String(nullable=nullable)
 
 
-@dt.dtype.register(sa.types.Binary)
-def sa_binary(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.types.Binary)
+def sa_binary(_, satype, nullable=True):
     return dt.Binary(nullable=nullable)
 
 
-@dt.dtype.register(sa.Time)
-def sa_time(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.Time)
+def sa_time(_, satype, nullable=True):
     return dt.Time(nullable=nullable)
 
 
-@dt.dtype.register(sa.Date)
-def sa_date(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.Date)
+def sa_date(_, satype, nullable=True):
     return dt.Date(nullable=nullable)
 
 
-@dt.dtype.register(sa.DateTime)
-def sa_datetime(satype, nullable=True, default_timezone='UTC'):
+@dt.dtype.register(SQLAlchemyDialect, sa.DateTime)
+def sa_datetime(_, satype, nullable=True, default_timezone='UTC'):
     timezone = default_timezone if satype.timezone else None
     return dt.Timestamp(timezone=timezone, nullable=nullable)
 
 
-@dt.dtype.register(sa.ARRAY)
-def sa_array(satype, nullable=True):
+@dt.dtype.register(SQLAlchemyDialect, sa.ARRAY)
+def sa_array(dialect, satype, nullable=True):
     dimensions = satype.dimensions
     if dimensions is not None and dimensions != 1:
         raise NotImplementedError('Nested array types not yet supported')
 
-    value_dtype = dt.dtype(satype.item_type)
+    value_dtype = dt.dtype(dialect, satype.item_type)
     return dt.Array(value_dtype, nullable=nullable)
 
 
@@ -177,7 +186,11 @@ def schema_from_table(table, schema=None):
         if name in schema:
             dtype = dt.dtype(schema[name])
         else:
-            dtype = dt.dtype(column.type, nullable=column.nullable)
+            dtype = dt.dtype(
+                getattr(table.bind, 'dialect', SQLAlchemyDialect()),
+                column.type,
+                nullable=column.nullable
+            )
         pairs.append((name, dtype))
     return sch.schema(pairs)
 
@@ -246,7 +259,7 @@ def fixed_arity(sa_func, arity):
 def varargs(sa_func):
     def formatter(t, expr):
         op = expr.op()
-        trans_args = [t.translate(arg) for arg in op.args]
+        trans_args = [t.translate(arg) for arg in op.arg]
         return sa_func(*trans_args)
     return formatter
 
@@ -328,6 +341,7 @@ def _contains(t, expr):
     op = expr.op()
 
     left, right = [t.translate(arg) for arg in op.args]
+
     return left.in_(right)
 
 
@@ -357,7 +371,13 @@ def _reduction_format(t, sa_func, arg, where):
 
 
 def _literal(t, expr):
-    return sa.literal(expr.op().value)
+    dtype = expr.type()
+    value = expr.op().value
+
+    if isinstance(dtype, dt.Set):
+        return list(map(sa.literal, value))
+
+    return sa.literal(value)
 
 
 def _value_list(t, expr):
@@ -574,9 +594,9 @@ _operation_registry = {
 
     ops.TypeOf: unary(sa.func.typeof),
 
-    ir.Literal: _literal,
-    ir.ValueList: _value_list,
-    ir.NullLiteral: lambda *args: sa.null(),
+    ops.Literal: _literal,
+    ops.ValueList: _value_list,
+    ops.NullLiteral: lambda *args: sa.null(),
 
     ops.SimpleCase: _simple_case,
     ops.SearchedCase: _searched_case,
@@ -682,6 +702,16 @@ class AlchemyContext(comp.QueryContext):
         super(AlchemyContext, self).__init__(*args, **kwargs)
         self._table_objects = {}
 
+    def collapse(self, queries):
+        if isinstance(queries, six.string_types):
+            return queries
+
+        if len(queries) > 1:
+            raise NotImplementedError(
+                'Only a single query is supported for SQLAlchemy backends'
+            )
+        return queries[0]
+
     def subcontext(self):
         return type(self)(
             dialect=self.dialect, parent=self, params=self.params
@@ -710,13 +740,28 @@ class AlchemyContext(comp.QueryContext):
         return self._get_table_item('_table_objects', expr)
 
 
+class AlchemyUnion(Union):
+
+    def compile(self):
+        def reduce_union(left, right, distincts=iter(self.distincts)):
+            distinct = next(distincts)
+            sa_func = sa.union if distinct else sa.union_all
+            return sa_func(left, right)
+
+        context = self.context
+        selects = []
+
+        for table in self.tables:
+            table_set = context.get_compiled_expr(table)
+            selects.append(table_set.cte().select())
+
+        return functools.reduce(reduce_union, selects)
+
+
 class AlchemyQueryBuilder(comp.QueryBuilder):
 
     select_builder = AlchemySelectBuilder
-
-    @property
-    def _union_class(self):
-        return AlchemyUnion
+    union_class = AlchemyUnion
 
 
 def to_sqlalchemy(expr, context, exists=False):
@@ -840,10 +885,6 @@ class AlchemyQuery(Query):
         return self.schema().apply_to(df)
 
 
-class AlchemyAsyncQuery(AsyncQuery):
-    pass
-
-
 class AlchemyDialect(Dialect):
 
     translator = AlchemyExprTranslator
@@ -873,7 +914,7 @@ def invalidates_reflection_cache(f):
 class AlchemyClient(SQLClient):
 
     dialect = AlchemyDialect
-    sync_query = AlchemyQuery
+    query_class = AlchemyQuery
 
     def __init__(self, con):
         super(AlchemyClient, self).__init__()
@@ -883,12 +924,6 @@ class AlchemyClient(SQLClient):
         self._reflection_cache_is_dirty = False
         self._schemas = {}
 
-    @property
-    def async_query(self):
-        raise NotImplementedError(
-            'async_query not implemented in {}'.format(type(self).__name__)
-        )
-
     @property
     def inspector(self):
         if self._reflection_cache_is_dirty:
@@ -993,8 +1028,8 @@ class AlchemyClient(SQLClient):
         return sa.Table(name, self.meta, schema=schema, autoload=True)
 
     def _sqla_table_to_expr(self, table):
-        node = AlchemyTable(table, self)
-        return self._table_expr_klass(node)
+        node = self.table_class(table, self)
+        return self.table_expr_class(node)
 
     @property
     def version(self):
@@ -1285,23 +1320,6 @@ def _and_all(clauses):
     return result
 
 
-class AlchemyUnion(Union):
-
-    def compile(self):
-        context = self.context
-        sa_func = sa.union if self.distinct else sa.union_all
-
-        left_set = context.get_compiled_expr(self.left)
-        left_cte = left_set.cte()
-        left_select = left_cte.select()
-
-        right_set = context.get_compiled_expr(self.right)
-        right_cte = right_set.cte()
-        right_select = right_cte.select()
-
-        return sa_func(left_select, right_select)
-
-
 class AlchemyProxy(object):
     """
     Wraps a SQLAlchemy ResultProxy and ensures that .close() is called on
