@@ -1,29 +1,71 @@
 """Core expression to SQL infrastructure.
 """
 
-from six import StringIO
-from collections import defaultdict
+import abc
 
+from collections import defaultdict
+from itertools import chain
+
+import six
+
+from six import StringIO
+
+import toolz
+
+import ibis
 import ibis.common as com
+import ibis.util as util
+import ibis.expr.types as ir
+import ibis.expr.format as fmt
 import ibis.expr.analysis as L
 import ibis.expr.analytics as analytics
-
 import ibis.expr.operations as ops
-import ibis.expr.types as ir
-import ibis.expr.format as format
-
 import ibis.sql.transforms as transforms
-import ibis.util as util
-import ibis
+
+from ibis.compat import map
+
+
+@six.add_metaclass(abc.ABCMeta)
+class DML(object):
+    @abc.abstractmethod
+    def compile(self):
+        pass
+
+
+@six.add_metaclass(abc.ABCMeta)
+class DDL(object):
+    @abc.abstractmethod
+    def compile(self):
+        pass
 
 
 class QueryAST(object):
 
-    __slots__ = 'context', 'queries'
+    __slots__ = 'context', 'dml', 'setup_queries', 'teardown_queries'
 
-    def __init__(self, context, queries):
+    def __init__(
+        self, context, dml, setup_queries=None, teardown_queries=None
+    ):
         self.context = context
-        self.queries = queries
+        self.dml = dml
+        self.setup_queries = setup_queries
+        self.teardown_queries = teardown_queries
+
+    @property
+    def queries(self):
+        return [self.dml]
+
+    def compile(self):
+        compiled_setup_queries = [q.compile() for q in self.setup_queries]
+        compiled_queries = [q.compile() for q in self.queries]
+        compiled_teardown_queries = [
+            q.compile() for q in self.teardown_queries
+        ]
+        return self.context.collapse(list(chain(
+            compiled_setup_queries,
+            compiled_queries,
+            compiled_teardown_queries,
+        )))
 
 
 class SelectBuilder(object):
@@ -62,31 +104,15 @@ class SelectBuilder(object):
 
     def get_result(self):
         # make idempotent
-        if len(self.queries) > 0:
+        if self.queries:
             return self._wrap_result()
 
-        # Generate other kinds of DDL statements that may be required to
-        # execute the passed query. For example, loding
-        setup_queries = self._generate_setup_queries()
-
-        # Make DDL statements to be executed after the main primary select
-        # statement(s)
-        teardown_queries = self._generate_teardown_queries()
-
         select_query = self._build_result_query()
 
-        self.queries.extend(setup_queries)
         self.queries.append(select_query)
-        self.queries.extend(teardown_queries)
 
         return select_query
 
-    def _generate_setup_queries(self):
-        return []
-
-    def _generate_teardown_queries(self):
-        return []
-
     def _build_result_query(self):
         self._collect_elements()
 
@@ -250,7 +276,7 @@ class SelectBuilder(object):
 
         unchanged = True
         if isinstance(expr, ir.ScalarExpr):
-            if ops.is_reduction(expr):
+            if L.is_reduction(expr):
                 return self._rewrite_reduction_filter(expr)
 
         if isinstance(op, ops.BinaryOp):
@@ -258,11 +284,11 @@ class SelectBuilder(object):
             right = self._visit_filter(op.right)
             unchanged = left is op.left and right is op.right
             if not unchanged:
-                return type(expr)(type(op)(left, right))
+                return expr._factory(type(op)(left, right))
             else:
                 return expr
         elif isinstance(op, (ops.Any, ops.BooleanValueOp,
-                             ops.TableColumn, ir.Literal)):
+                             ops.TableColumn, ops.Literal)):
             return expr
         elif isinstance(op, ops.ValueOp):
             visited = [self._visit_filter(arg)
@@ -273,7 +299,7 @@ class SelectBuilder(object):
                 if new is not old:
                     unchanged = False
             if not unchanged:
-                return type(expr)(type(op)(*visited))
+                return expr._factory(type(op)(*visited))
             else:
                 return expr
         else:
@@ -343,7 +369,7 @@ class SelectBuilder(object):
                 raise com.InternalError('no table set')
         else:
             # Expressions not depending on any table
-            if isinstance(root_op, ir.ExpressionList):
+            if isinstance(root_op, ops.ExpressionList):
                 self.select_set = source_expr.exprs()
             else:
                 self.select_set = [source_expr]
@@ -613,7 +639,7 @@ class _ExtractSubqueries(object):
         return to_extract
 
     def observe(self, expr):
-        if expr in self.observed_exprs:
+        if self.seen(expr):
             key = self.observed_exprs.get(expr)
         else:
             # this key only needs to be unique because of the IbisMap
@@ -622,7 +648,7 @@ class _ExtractSubqueries(object):
 
         self.expr_counts[key] += 1
 
-    def _has_been_observed(self, expr):
+    def seen(self, expr):
         return expr in self.observed_exprs
 
     def visit(self, expr):
@@ -675,6 +701,7 @@ class _ExtractSubqueries(object):
         op = expr.op()
         self.visit(op.left)
         self.visit(op.right)
+        self.observe(expr)
 
     def _visit_MaterializedJoin(self, expr):
         self.visit(expr.op().join)
@@ -689,7 +716,7 @@ class _ExtractSubqueries(object):
 
     def _visit_TableColumn(self, expr):
         table = expr.op().table
-        if not self._has_been_observed(table):
+        if not self.seen(table):
             self.visit(table)
 
     def _visit_SelfReference(self, expr):
@@ -818,7 +845,7 @@ def _adapt_expr(expr):
 
     if isinstance(expr, ir.ScalarExpr):
 
-        if L.is_scalar_reduce(expr):
+        if L.is_scalar_reduction(expr):
             table_expr, name = L.reduction_to_aggregation(
                 expr, default_name='tmp')
             return table_expr, _get_scalar(name)
@@ -827,7 +854,7 @@ def _adapt_expr(expr):
             if base_table is None:
                 # exprs with no table refs
                 # TODO(phillipc): remove ScalarParameter hack
-                if isinstance(expr.op(), ir.ScalarParameter):
+                if isinstance(expr.op(), ops.ScalarParameter):
                     name = expr.get_name()
                     assert name is not None, \
                         'scalar parameter {} has no name'.format(expr)
@@ -846,7 +873,7 @@ def _adapt_expr(expr):
         any_aggregation = False
 
         for x in exprs:
-            if not L.is_scalar_reduce(x):
+            if not L.is_scalar_reduction(x):
                 is_aggregation = False
             else:
                 any_aggregation = True
@@ -894,21 +921,105 @@ def _adapt_expr(expr):
                                    .format(type(expr)))
 
 
+class Union(DML):
+
+    def __init__(self, tables, expr, context, distincts):
+        self.context = context
+        self.tables = tables
+        self.distincts = distincts
+        self.table_set = expr
+        self.filters = []
+
+    def _extract_subqueries(self):
+        self.subqueries = _extract_subqueries(self)
+        for subquery in self.subqueries:
+            self.context.set_extracted(subquery)
+
+    def format_subqueries(self):
+        context = self.context
+        subqueries = self.subqueries
+
+        return ',\n'.join(
+            '{} AS (\n{}\n)'.format(
+                context.get_ref(expr),
+                util.indent(context.get_compiled_expr(expr), 2)
+            ) for expr in subqueries
+        )
+
+    def format_relation(self, expr):
+        ref = self.context.get_ref(expr)
+        if ref is not None:
+            return 'SELECT *\nFROM {}'.format(ref)
+        return self.context.get_compiled_expr(expr)
+
+    @staticmethod
+    def keyword(distinct):
+        return 'UNION' if distinct else 'UNION ALL'
+
+    def compile(self):
+        self._extract_subqueries()
+
+        extracted = self.format_subqueries()
+
+        buf = []
+
+        if extracted:
+            buf.append('WITH {}'.format(extracted))
+
+        # interleave correct keyword for the backend in between the formatted
+        # UNION expressions
+        buf.extend(
+            toolz.interleave(
+                (
+                    map(self.format_relation, self.tables),
+                    map(self.keyword, self.distincts)
+                )
+            )
+        )
+        return '\n'.join(buf)
+
+
+def flatten_union(table):
+    """Extract all union queries from `table`.
+
+    Parameters
+    ----------
+    table : TableExpr
+
+    Returns
+    -------
+    Iterable[Union[TableExpr, bool]]
+    """
+    op = table.op()
+    if isinstance(op, ops.Union):
+        return toolz.concatv(
+            flatten_union(op.left), [op.distinct], flatten_union(op.right)
+        )
+    return [table]
+
+
 class QueryBuilder(object):
 
     select_builder = SelectBuilder
+    union_class = Union
 
     def __init__(self, expr, context):
         self.expr = expr
         self.context = context
 
-    @property
-    def _union_class(self):
-        return Union
+    def generate_setup_queries(self):
+        return []
+
+    def generate_teardown_queries(self):
+        return []
 
     def get_result(self):
         op = self.expr.op()
 
+        # collect setup and teardown queries
+        setup_queries = self.generate_setup_queries()
+        teardown_queries = self.generate_teardown_queries()
+
         # TODO: any setup / teardown DDL statements will need to be done prior
         # to building the result set-generating statements.
         if isinstance(op, ops.Union):
@@ -916,13 +1027,32 @@ class QueryBuilder(object):
         else:
             query = self._make_select()
 
-        return QueryAST(self.context, [query])
+        return QueryAST(
+            self.context,
+            query,
+            setup_queries=setup_queries,
+            teardown_queries=teardown_queries,
+        )
 
     def _make_union(self):
-        op = self.expr.op()
-        return self._union_class(op.left, op.right, self.expr,
-                                 distinct=op.distinct,
-                                 context=self.context)
+        # flatten unions so that we can codegen them all at once
+        union_info = list(flatten_union(self.expr))
+
+        # since op is a union, we have at least 3 elements in union_info (left
+        # distinct right) and if there is more than a single union we have an
+        # additional two elements per union (distinct right) which means the
+        # total number of elements is at least 3 + (2 * number of unions - 1)
+        # and is therefore an odd number
+        npieces = len(union_info)
+        assert npieces >= 3 and npieces % 2 != 0, 'Invalid union expression'
+
+        # 1. every other object starting from 0 is a TableExpr instance
+        # 2. every other object starting from 1 is a bool indicating the type
+        #    of union (distinct or not distinct)
+        table_exprs, distincts = union_info[::2], union_info[1::2]
+        return self.union_class(table_exprs, self.expr,
+                                distincts=distincts,
+                                context=self.context)
 
     def _make_select(self):
         builder = self.select_builder(self.expr, self.context)
@@ -951,7 +1081,7 @@ class QueryContext(object):
         self.query = None
 
         self._table_key_memo = {}
-        self.memo = memo or format.FormatMemo()
+        self.memo = memo or fmt.FormatMemo()
         self.dialect = dialect
         self.params = params if params is not None else {}
 
@@ -962,6 +1092,19 @@ class QueryContext(object):
     def _to_sql(self, expr, ctx):
         raise NotImplementedError
 
+    def collapse(self, queries):
+        """Turn a sequence of queries into something executable.
+
+        Parameters
+        ----------
+        queries : List[str]
+
+        Returns
+        -------
+        query : str
+        """
+        return '\n\n'.join(queries)
+
     @property
     def top_context(self):
         if self.parent is None:
@@ -1007,7 +1150,7 @@ class QueryContext(object):
 
             i += len(ctx._table_refs)
 
-        alias = 't%d' % i
+        alias = 't{:d}'.format(i)
         self.set_ref(expr, alias)
 
     def need_aliases(self, expr=None):
@@ -1149,7 +1292,7 @@ class ExprTranslator(object):
             op = expr.op()
 
         # TODO: use op MRO for subclasses instead of this isinstance spaghetti
-        if isinstance(op, ir.ScalarParameter):
+        if isinstance(op, ops.ScalarParameter):
             return self._trans_param(expr)
         elif isinstance(op, ops.TableNode):
             # HACK/TODO: revisit for more complex cases
@@ -1299,11 +1442,7 @@ class Dialect(object):
         return cls.translator.context_class(dialect=cls(), params=params)
 
 
-class DDL(object):
-    pass
-
-
-class Select(DDL):
+class Select(DML):
 
     """
     A SELECT statement which, after execution, might yield back to the user a
@@ -1552,10 +1691,10 @@ class Select(DDL):
         buf = StringIO()
         buf.write('WHERE ')
         fmt_preds = []
+        npreds = len(self.where)
         for pred in self.where:
             new_pred = self._translate(pred, permit_subquery=True)
-            if isinstance(pred.op(), (ops.Or, ops.Xor)):
-                # parens for OR exprs because it binds looser than AND
+            if npreds > 1:
                 new_pred = '({})'.format(new_pred)
             fmt_preds.append(new_pred)
 
@@ -1730,10 +1869,10 @@ class TableSetFormatter(object):
             buf.write(util.indent('{} {}'.format(jtype, table), self.indent))
 
             fmt_preds = []
+            npreds = len(preds)
             for pred in preds:
                 new_pred = self._translate(pred)
-                if isinstance(pred.op(), (ops.Or, ops.Xor)):
-                    # parens for OR exprs because it binds looser than AND
+                if npreds > 1:
                     new_pred = '({})'.format(new_pred)
                 fmt_preds.append(new_pred)
 
@@ -1746,54 +1885,3 @@ class TableSetFormatter(object):
                 buf.write(fmt_preds)
 
         return buf.getvalue()
-
-
-class Union(DDL):
-
-    def __init__(self, left_table, right_table, expr, context, distinct=False):
-        self.context = context
-        self.left = left_table
-        self.right = right_table
-        self.distinct = distinct
-        self.table_set = expr
-        self.filters = []
-
-    def _extract_subqueries(self):
-        self.subqueries = _extract_subqueries(self)
-        for subquery in self.subqueries:
-            self.context.set_extracted(subquery)
-
-    def format_subqueries(self):
-        context = self.context
-        subqueries = self.subqueries
-
-        return ',\n'.join([
-            '{} AS (\n{}\n)'.format(
-                context.get_ref(expr),
-                util.indent(context.get_compiled_expr(expr), 2)
-            ) for expr in subqueries
-        ])
-
-    def format_relation(self, expr):
-        ref = self.context.get_ref(expr)
-        if ref is not None:
-            return 'SELECT *\nFROM {}'.format(ref)
-        return self.context.get_compiled_expr(expr)
-
-    def compile(self):
-        union_keyword = 'UNION' if self.distinct else 'UNION ALL'
-
-        self._extract_subqueries()
-
-        left_set = self.format_relation(self.left)
-        right_set = self.format_relation(self.right)
-        extracted = self.format_subqueries()
-
-        buf = []
-
-        if extracted:
-            buf.append('WITH {}'.format(extracted))
-
-        buf.extend([left_set, union_keyword, right_set])
-
-        return '\n'.join(buf)
