@@ -1,17 +1,3 @@
-# Copyright 2014 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import re
 import locale
 import string
@@ -30,7 +16,6 @@ from ibis.compat import functools
 from ibis.sql.alchemy import (unary, fixed_arity, infix_op,
                               _variance_reduction, _get_sqla_table)
 import ibis.common as com
-import ibis.expr.types as ir
 import ibis.expr.datatypes as dt
 import ibis.expr.operations as ops
 
@@ -123,7 +108,7 @@ def _timestamp_truncate(t, expr):
 def _interval_from_integer(t, expr):
     arg, unit = expr.op().args
     sa_arg = t.translate(arg)
-    interval = sa.text("INTERVAL '1 {}'".format(expr.resolution))
+    interval = sa.text("INTERVAL '1 {}'".format(expr.type().resolution))
     return sa_arg * interval
 
 
@@ -133,14 +118,16 @@ def _timestamp_add(t, expr):
 
 
 def _is_nan(t, expr):
-    arg = t.translate(expr.op().args[0])
-    return arg == sa.literal('nan', sa.Float)
+    arg, = expr.op().args
+    sa_arg = t.translate(arg)
+    return sa_arg == float('nan')
 
 
 def _is_inf(t, expr):
-    arg = t.translate(expr.op().args[0])
-    return sa.or_(arg == sa.literal('inf', sa.Float),
-                  arg == sa.literal('-inf', sa.Float))
+    arg, = expr.op().args
+    sa_arg = t.translate(arg)
+    inf = float('inf')
+    return sa.or_(sa_arg == inf, sa_arg == -inf)
 
 
 def _cast(t, expr):
@@ -400,11 +387,17 @@ def _reduction(func_name):
 
 def _log(t, expr):
     arg, base = expr.op().args
-    arg = t.translate(arg)
+    sa_arg = t.translate(arg)
     if base is not None:
-        return sa.func.log(t.translate(base), arg)
-    else:
-        return sa.func.ln(arg)
+        sa_base = t.translate(base)
+        return sa.cast(
+            sa.func.log(
+                sa.cast(sa_base, sa.NUMERIC),
+                sa.cast(sa_arg, sa.NUMERIC)
+            ),
+            t.get_sqla_type(expr.type())
+        )
+    return sa.func.ln(sa_arg)
 
 
 class regex_extract(GenericFunction):
@@ -540,7 +533,8 @@ def _round(t, expr):
     result = sa.func.round(sa.cast(sa_arg, sa.NUMERIC), t.translate(digits))
     if digits is not None and isinstance(arg.type(), dt.Decimal):
         return result
-    return sa.cast(result, sa.dialects.postgresql.DOUBLE_PRECISION())
+    result = sa.cast(result, sa.dialects.postgresql.DOUBLE_PRECISION())
+    return result
 
 
 def _mod(t, expr):
@@ -577,16 +571,23 @@ def _string_join(t, expr):
 
 
 def _literal(t, expr):
-    if isinstance(expr, ir.IntervalValue):
-        return sa.text("INTERVAL '{} {}'".format(expr.op().value,
-                                                 expr.resolution))
+    dtype = expr.type()
+    value = expr.op().value
+
+    if isinstance(dtype, dt.Interval):
+        return sa.text("INTERVAL '{} {}'".format(value, dtype.resolution))
+    elif isinstance(dtype, dt.Set):
+        return list(map(sa.literal, value))
     else:
-        return sa.literal(expr.op().value)
+        return sa.literal(value)
 
 
 def _day_of_week_index(t, expr):
     sa_arg, = map(t.translate, expr.op().args)
-    return sa.cast(sa.func.to_char(sa_arg, 'D'), sa.INTEGER)
+    return sa.cast(
+        sa.cast(sa.extract('dow', sa_arg) + 6, sa.SMALLINT) % 7,
+        sa.SMALLINT
+    )
 
 
 def _day_of_week_name(t, expr):
@@ -595,7 +596,7 @@ def _day_of_week_name(t, expr):
 
 
 _operation_registry.update({
-    ir.Literal: _literal,
+    ops.Literal: _literal,
 
     # We override this here to support time zones
     ops.TableColumn: _table_column,
@@ -694,7 +695,7 @@ class PostgreSQLExprTranslator(alch.AlchemyExprTranslator):
     _rewrites = alch.AlchemyExprTranslator._rewrites.copy()
     _type_map = alch.AlchemyExprTranslator._type_map.copy()
     _type_map.update({
-        dt.Double: pg.FLOAT,
+        dt.Double: pg.DOUBLE_PRECISION,
         dt.Float: pg.REAL
     })
 
