@@ -1,22 +1,8 @@
-# Copyright 2015 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import os
 import sys
-import math
 import operator
 import string
+import warnings
 
 from operator import methodcaller
 from datetime import date, datetime
@@ -72,7 +58,8 @@ def guid2(con):
         ),
         (
             lambda t: t.string_col.cast('double'),
-            lambda at: sa.cast(at.c.string_col, sa.FLOAT),
+            lambda at: sa.cast(
+                at.c.string_col, sa.dialects.postgresql.DOUBLE_PRECISION),
         ),
         (
             lambda t: t.string_col.cast('float'),
@@ -151,7 +138,7 @@ def test_timestamp_cast_noop(alltypes, at, translate):
         (methodcaller('minute'), 48),
         (methodcaller('second'), 5),
         (methodcaller('millisecond'), 359),
-        (lambda x: x.day_of_week.index(), 3),
+        (lambda x: x.day_of_week.index(), 1),
         (lambda x: x.day_of_week.full_name(), 'Tuesday'),
     ]
 )
@@ -397,33 +384,6 @@ def test_string_functions(con, expr, expected):
     assert con.execute(expr) == expected
 
 
-@pytest.mark.parametrize(
-    ('expr', 'expected'),
-    [
-        (L(-5).abs(), 5),
-        (L(5).abs(), 5),
-        (ibis.least(L(5), L(10), L(1)), 1),
-        (ibis.greatest(L(5), L(10), L(1)), 10),
-
-        (L(5.5).round(), 6.0),
-        (L(5.556).round(2), 5.56),
-        (L(5.556).ceil(), 6.0),
-        (L(5.556).floor(), 5.0),
-        (L(5.556).exp(), math.exp(5.556)),
-        (L(5.556).sign(), 1),
-        (L(-5.556).sign(), -1),
-        (L(0).sign(), 0),
-        (L(5.556).sqrt(), math.sqrt(5.556)),
-        (L(5.556).log(2), math.log(5.556, 2)),
-        (L(5.556).ln(), math.log(5.556)),
-        (L(5.556).log2(), math.log(5.556, 2)),
-        (L(5.556).log10(), math.log10(5.556)),
-    ]
-)
-def test_math_functions(con, expr, expected):
-    assert con.execute(expr) == expected
-
-
 @pytest.mark.parametrize(
     ('expr', 'expected'),
     [
@@ -586,7 +546,12 @@ def test_category_label(alltypes, df):
     labels = ['a', 'b', 'c', 'd']
     bucket = d.bucket(bins)
     expr = bucket.label(labels)
-    result = expr.execute().astype('category', ordered=True)
+    result = expr.execute()
+
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore')
+        result = result.astype('category', ordered=True)
+
     result.name = 'double_col'
 
     expected = pd.cut(df.double_col, bins, labels=labels, right=False)
@@ -610,6 +575,45 @@ def test_union(alltypes):
     tm.assert_frame_equal(result, expected)
 
 
+@pytest.mark.parametrize(
+    ('distinct1', 'distinct2', 'expected1', 'expected2'),
+    [
+        (True, True, 'UNION', 'UNION'),
+        (True, False, 'UNION', 'UNION ALL'),
+        (False, True, 'UNION ALL', 'UNION'),
+        (False, False, 'UNION ALL', 'UNION ALL'),
+    ]
+)
+def test_union_cte(alltypes, distinct1, distinct2, expected1, expected2):
+    t = alltypes
+    expr1 = t.group_by(t.string_col).aggregate(metric=t.double_col.sum())
+    expr2 = expr1.view()
+    expr3 = expr1.view()
+    expr = expr1.union(
+        expr2, distinct=distinct1).union(expr3, distinct=distinct2)
+    result = '\n'.join(map(
+        lambda line: line.rstrip(),  # strip trailing whitespace
+        str(
+            expr.compile().compile(compile_kwargs=dict(literal_binds=True))
+        ).splitlines()
+    ))
+    expected = """\
+WITH anon_1 AS
+(SELECT t0.string_col AS string_col, sum(t0.double_col) AS metric
+FROM functional_alltypes AS t0 GROUP BY t0.string_col),
+anon_2 AS
+(SELECT t0.string_col AS string_col, sum(t0.double_col) AS metric
+FROM functional_alltypes AS t0 GROUP BY t0.string_col),
+anon_3 AS
+(SELECT t0.string_col AS string_col, sum(t0.double_col) AS metric
+FROM functional_alltypes AS t0 GROUP BY t0.string_col)
+ (SELECT anon_1.string_col, anon_1.metric
+FROM anon_1 {} SELECT anon_2.string_col, anon_2.metric
+FROM anon_2) {} SELECT anon_3.string_col, anon_3.metric
+FROM anon_3""".format(expected1, expected2)
+    assert str(result) == expected
+
+
 @pytest.mark.parametrize(
     ('func', 'pandas_func'),
     [
