@@ -1,17 +1,3 @@
-# Copyright 2015 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import os
 import regex as re
 import math
@@ -202,9 +188,18 @@ def _ibis_sqlite_sign(arg):
 
 @udf
 def _ibis_sqlite_floordiv(left, right):
+    if left is None or right is None:
+        return None
     return left // right
 
 
+@udf
+def _ibis_sqlite_mod(left, right):
+    if left is None or right is None:
+        return None
+    return left % right
+
+
 @udf
 def _ibis_sqlite_power(arg, power):
     """Raise `arg` to the `power` power.
@@ -336,11 +331,12 @@ class SQLiteClient(alch.AlchemyClient):
 
     dialect = SQLiteDialect
     database_class = SQLiteDatabase
+    table_class = SQLiteTable
 
     def __init__(self, path=None, create=False):
         super(SQLiteClient, self).__init__(sa.create_engine('sqlite://'))
         self.name = path
-        self.database_name = 'default'
+        self.database_name = 'base'
 
         if path is not None:
             self.attach(self.database_name, path, create=create)
@@ -406,8 +402,8 @@ class SQLiteClient(alch.AlchemyClient):
         table : TableExpr
         """
         alch_table = self._get_sqla_table(name, schema=database)
-        node = SQLiteTable(alch_table, self)
-        return self._table_expr_klass(node)
+        node = self.table_class(alch_table, self)
+        return self.table_expr_class(node)
 
     def list_tables(self, like=None, database=None, schema=None):
         if database is None:
