@@ -1,17 +1,3 @@
-# Copyright 2015 Cloudera Inc.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import os
 import uuid
 
@@ -50,7 +36,7 @@ def test_table(con):
     assert isinstance(table, ir.TableExpr)
 
 
-def test_array_execute(alltypes, df):
+def test_column_execute(alltypes, df):
     expr = alltypes.double_col
     result = expr.execute()
     expected = df.double_col
