@@ -18,6 +18,7 @@ import operator
 import math
 import datetime
 import sqlite3
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -461,7 +462,11 @@ def test_bucket(alltypes, df, func, expected_func):
     expr = func(alltypes.double_col)
     result = expr.execute()
     expected = expected_func(df.double_col)
-    expected = expected.astype('category')
+
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore')
+        expected = expected.astype('category')
+
     tm.assert_series_equal(result, expected, check_names=False)
 
 
@@ -469,8 +474,12 @@ def test_category_label(alltypes, df):
     bins = [0, 10, 25, 50, 100]
     labels = ['a', 'b', 'c', 'd']
     expr = alltypes.double_col.bucket(bins).label(labels)
+    result = expr.execute()
+
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore')
+        result = result.astype('category', ordered=True)
 
-    result = expr.execute().astype('category', ordered=True)
     result.name = 'double_col'
 
     expected = pd.cut(df.double_col, bins, labels=labels, right=False)
@@ -833,5 +842,5 @@ def test_count_on_order_by(db):
         expr.compile().compile(compile_kwargs={'literal_binds': True})
     )
     expected = ('SELECT count(\'*\') AS count \n'
-                'FROM "default".batting AS t0')  # noqa: W291
+                'FROM base.batting AS t0')  # noqa: W291
     assert result == expected
