@@ -2,6 +2,7 @@ import pytest
 
 import ibis
 import ibis.tests.util as tu
+import ibis.expr.datatypes as dt
 from ibis.compat import parse_version
 
 
@@ -13,9 +14,9 @@ def test_version(backend, con):
 
 
 @pytest.mark.parametrize(('expr_fn', 'expected'), [
-    (lambda t: t.string_col, [('string_col', 'string')]),
+    (lambda t: t.string_col, [('string_col', dt.String)]),
     (lambda t: t[t.string_col, t.bigint_col],
-     [('string_col', 'string'), ('bigint_col', 'int64')])
+     [('string_col', dt.String), ('bigint_col', dt.Int64)])
 ])
 def test_query_schema(backend, con, alltypes, expr_fn, expected):
     if not hasattr(con, '_build_ast'):
@@ -29,8 +30,11 @@ def test_query_schema(backend, con, alltypes, expr_fn, expected):
 
     # we might need a public API for it
     ast = con._build_ast(expr, backend.make_context())
-    query = con.sync_query(con, ast.queries[0])
+    query = con.query_class(con, ast)
+    schema = query.schema()
 
-    # test single columns
-    expected = ibis.schema(expected)
+    # clickhouse columns has been defined as non-nullable
+    # whereas other backends don't support non-nullable columns yet
+    expected = ibis.schema([(name, dtype(nullable=schema[name].nullable))
+                            for name, dtype in expected])
     assert query.schema().equals(expected)
