@@ -41,3 +41,15 @@ def skipif_unsupported(f):
                 NotImplementedError) as e:
             pytest.skip('{} using {}'.format(e, str(backend)))
     return wrapper
+
+
+def skipif_backend(skip_backend):
+    def wrapped(f):
+        @functools.wraps(f)
+        def wrapper(backend, *args, **kwargs):
+            if isinstance(backend, skip_backend):
+                pytest.skip('Skipping {} test'.format(str(backend)))
+            else:
+                return f(backend, *args, **kwargs)
+        return wrapper
+    return wrapped
