@@ -1,6 +1,12 @@
 print("import: 'ibis'")
 import ibis
 
+print("import: 'ibis.bigquery'")
+import ibis.bigquery
+
+print("import: 'ibis.bigquery.tests'")
+import ibis.bigquery.tests
+
 print("import: 'ibis.expr'")
 import ibis.expr
 
@@ -13,6 +19,18 @@ import ibis.hive
 print("import: 'ibis.hive.tests'")
 import ibis.hive.tests
 
+print("import: 'ibis.mapd'")
+import ibis.mapd
+
+print("import: 'ibis.mapd.tests'")
+import ibis.mapd.tests
+
+print("import: 'ibis.pandas'")
+import ibis.pandas
+
+print("import: 'ibis.pandas.tests'")
+import ibis.pandas.tests
+
 print("import: 'ibis.spark'")
 import ibis.spark
 
@@ -22,6 +40,12 @@ import ibis.spark.tests
 print("import: 'ibis.sql'")
 import ibis.sql
 
+print("import: 'ibis.sql.postgres'")
+import ibis.sql.postgres
+
+print("import: 'ibis.sql.postgres.tests'")
+import ibis.sql.postgres.tests
+
 print("import: 'ibis.sql.presto'")
 import ibis.sql.presto
 
